;;; w3-about.el --- About pages for emacs-w3
;; Author: wmperry
;; Created: 1997/10/17 14:08:10
;; Version: 1.11
;; Keywords: hypermedia

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Copyright (c) 1993 - 1996 by William M. Perry <wmperry@cs.indiana.edu>
;;; Copyright (c) 1996, 1997 Free Software Foundation, Inc.
;;;
;;; This file is part of GNU Emacs.
;;;
;;; GNU Emacs is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.
;;;
;;; GNU Emacs is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with GNU Emacs; see the file COPYING.  If not, write to the
;;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;;; Boston, MA 02111-1307, USA.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;###autoload
(defun w3-about (url)
  ;; Fetch an about page url
  (let* ((data (url-generic-parse-url url))
	 (node (downcase (url-filename data))))
    (if (string= "document" node)
	(w3-document-information)
      (save-excursion
	(set-buffer (get-buffer-create url-working-buffer))
	(erase-buffer)
	(setq url-current-mime-viewer (mm-mime-info "text/html" nil 5)
	      url-current-mime-headers '(("content-type" . "text/html")))
	(cond
	 ((string= "" node)
	  (insert (format
		   "
<html>
  <head>
    <title>Emacs-W3 v%s</title>
    <link rel=\"made\" href=\"mailto:wmperry+w3@cs.indiana.edu\">
    <link rel=\"stylesheet\" href=\"about:style\">
  </head>
  <body>
    <h1>Emacs-W3 &trade;<br>v%s</h1>
    <p align=\"center\">
      Copyright &copy; 1993-1995 William M. Perry<br>
      All rights reserved.
    </p>
    <hr width=\"50%%\">
    <p>
      Welcome to Emacs-w3!  Please see the <a href=\"info:w3#Top\">info
	documentation</a>, or the <a
	href=\"http://www.cs.indiana.edu/elisp/w3/docs.html\">HTML
	version</a> online.
    </p>
    <p>
      Information about the <a href=\"authors\">authors</a>, <a
	href=\"emacs\">versions of emacs</a>, and <a
	href=\"eggs\">easter eggs</a> is also available.
    </p>
    <address>
      Please send any bugs/comments to<br>
      <a href=\"mailto:%s\">%s</a>
    </address>
    <hr>
    <wired><pinhead></wired>
  </body>
</html>
"
		   w3-version-number w3-version-number w3-bug-address w3-bug-address)))
	 ((string= "style" node)
	  (insert
	   "
/* This is the stylesheet for the about pages for Emacs-w3 */

address,h1,h2,h3,h4,h5,h6 { text-align: center }
wired { color: yellow }
wired { background: red }
"))
	 ((string= "license" node)
	  (kill-buffer (current-buffer))
	  (describe-copying))
	 ((string= "warranty" node)
	  (kill-buffer (current-buffer))
	  (describe-no-warranty))
	 ((string= "arena.xpm" node)
	  (insert
	   "/* XPM */
static char *arena[] = {
/* width height num_colors chars_per_pixel */
\"   109    84        3            1\",
/* colors */
\". c #d2c7b1\",
\"# c #dcd1ba\",
\"a c #e6dac2\",
/* pixels */
\"a.a.aa.a##a###aa##a.a.#.aa.....a##a#aa.#a#a###.#.#aa.aa.#aa#aa#.a.a.aa.a##a###aa##a.a.#.aa.....a##a#aa.#a#a##\",
\"#a.a#a##a###...a#aa#aa..aa.aa###aa#.a...a.aa#..a##..aa#a.##aa.a.#a.a#a##a###...a#aa#aa..aa.aa###aa#.a...a.aa#\",
\"a.aaa.aaa#aa#aa#.aa#....aa..###a.#a...#.#.#.....a.#.#aaa#aa..#a#a.aaa.aaa#aa#aa#.aa#....aa..###a.#a...#.#.#..\",
\".#a#.##a..a#aa#.###a.#a..a#.aa.##.#.a#.#..a#.a#a#a.aa#.##a####.#.#a#.##a..a#aa#.###a.#a..a#.aa.##.#.a#.#..a#.\",
\"####.###a##a.#aa#.#..a..##a.a#a##a##..aa##.#..#.....a..##..a.a..####.###a##a.#aa#.#..a..##a.a#a##a##..aa##.#.\",
\"..aa..a#aa#.#a#aaa.a#aa.#....#a#a.###a.#a.a.aaa.###.aaaa#.##aa.#..aa..a#aa#.#a#aaa.a#aa.#....#a#a.###a.#a.a.a\",
\"##a##aa#a#..#a..a.##a..#..##..aaa#a.aaa.#a..###.a.#.#aa#.##.a.a###a##aa#a#..#a..a.##a..#..##..aaa#a.aaa.#a..#\",
\".aaaaaaa#a#.aaa#.a#.aa.#a.a#aa..##aa#.##..#.a..#.a#.#a####aaaaa#.aaaaaaa#a#.aaa#.a#.aa.#a.a#aa..##aa#.##..#.a\",
\".a#.aa#.#a..a#aa###.aa#.a.a#.#.a.##.aaaa#####aaa.a.a#.a..a.aa#...a#.aa#.#a..a#aa###.aa#.a.a#.#.a.##.aaaa#####\",
\"#.#.##.##a.#.a#..#a###a.##aa..a.#a#a.##a#.##...a#.aa.aa#a..aa####.#.##.##a.#.a#..#a###a.##aa..a.#a#a.##a#.##.\",
\".#.##a..a..#a.#.#aa##a.#..#aa#.#a#...#..#.a#a#aa#.#a.aa..##.aaaa.#.##a..a..#a.#.#aa##a.#..#aa#.#a#...#..#.a#a\",
\".#aa...a.a##aaa..#a.aa.#a##...#a##aa#aa#..#...#.##...a.a...aa..a.#aa...a.a##aaa..#a.aa.#a##...#a##aa#aa#..#..\",
\".aaa###.a#..a#.#..aa.a##..##.#a#.a##a#a.#aa#....#..#a.a.a#aa.aa#.aaa###.a#..a#.#..aa.a##..##.#a#.a##a#a.#aa#.\",
\"#.#...aa.#aaaa.#####.##..a..a.##.#aa##aaaaa.aa.aa#.#....##aa.a###.#...aa.#aaaa.#####.##..a..a.##.#aa##aaaaa.a\",
\".a...a.aa.#a#aa#.aa.a.a...#a#..a#a.a.a##.#aaaa.#a##aa.#aa.#a.#.#.a...a.aa.#a#aa#.aa.a.a...#a#..a#a.a.a##.#aaa\",
\"#.a.a#aaaaa#aa#.a#.aaa.#aa#..a##aa#a#.aaa..##.a.aa.aa.a....##a.##.a.a#aaaaa#aa#.a#.aaa.#aa#..a##aa#a#.aaa..##\",
\"..##.aaaa#aa.aa.#a#a#a#.a##.###..##.a#.#aa.a#.a#.###a.###a#.#aa...##.aaaa#aa.aa.#a#a#a#.a##.###..##.a#.#aa.a#\",
\".a.#.aaa######a##.##a#..a#.#.aa.aa#aa...a#.a#.aaaa#a..a.aa#...a#.a.#.aaa######a##.##a#..a#.#.aa.aa#aa...a#.a#\",
\".a#.##a#a#####.##a.#.a#aaa#a.##a.aa##aa##aa.aa##..a###a#a.aaa..a.a#.##a#a#####.##a.#.a#aaa#a.##a.aa##aa##aa.a\",
\"a.aa#.##.a.a#aaa...aa.aaaa.#a#a.a.a.#aa..#a#aa#.aaaa.aa#....#a.#a.aa#.##.a.a#aaa...aa.aaaa.#a#a.a.a.#aa..#a#a\",
\"#.aaa.###aa.#.aaa##aa#.aaaa#.###a#.a.#..aaa#.a.a#.#....a#.#a#a.##.aaa.###aa.#.aaa##aa#.aaaa#.###a#.a.#..aaa#.\",
\"##a.aa..#.a.##.a.a###a##a#....a#...a#a..#a..#.aa..a.#a.#..a..a.a##a.aa..#.a.##.a.a###a##a#....a#...a#a..#a..#\",
\"a#.a#a..#a.####..#aa###.#aa#a..aa.a.aa#aa#a..a#..aaa#a..a#a.aaa.a#.a#a..#a.####..#aa###.#aa#a..aa.a.aa#aa#a..\",
\"#a..aaa.a.#.a##aa##aa#aa.aaa##.###...aa.aa#a.##a##a.a##.#..#.##a#a..aaa.a.#.a##aa##aa#aa.aaa##.###...aa.aa#a.\",
\".a.a.a#a#a#.#a...aaa##...#.#aa#aaa...##.###.a#a.#.#a.a..#aaa.a.a.a.a.a#a#a#.#a...aaa##...#.#aa#aaa...##.###.a\",
\"a.#a#.##...#.aa#####aaa#.###a##..a.a.##..#.#a####.#.###.a....##aa.#a#.##...#.aa#####aaa#.###a##..a.a.##..#.#a\",
\".#a#a.###....##..a#a#.a##..#aaa#a.#aa###.#..a####a..###aaa..#.##.#a#a.###....##..a#a#.a##..#aaa#a.#aa###.#..a\",
\".##aa#a.#..##aa.a.#..###a#.aa.#.##..#.##...#..a.####.#.a.a##aa.#.##aa#a.#..##aa.a.#..###a#.aa.#.##..#.##...#.\",
\".#a#.#a#a#..a#a.a#a.a#.aa#.#aa##aaa##.##...aa#..#.##...##.####aa.#a#.#a#a#..a#a.a#a.a#.aa#.#aa##aaa##.##...aa\",
\"..aaaa.a#a.#.aa.....a.aa...aa#aa.a####aa##a.##a####.aa.#a.a..#.#..aaaa.a#a.#.aa.....a.aa...aa#aa.a####aa##a.#\",
\".#...#..####.#.a.a.aa.#aa...#aa#a.###a#aa..a#.#a.##.#a##.aaaa.##.#...#..####.#.a.a.aa.#aa...#aa#a.###a#aa..a#\",
\"aa##..a.a..a.aa.###aa.##aa.a##a..#.###.a.#a#a#...##a.aaaa#..#.aaaa##..a.a..a.aa.###aa.##aa.a##a..#.###.a.#a#a\",
\"#aaaaa#aa#a##.a#aa...aaa.aaaa..a#.a.##.a.#.a#.a#a###.##a#aa.aaa##aaaaa#aa#a##.a#aa...aaa.aaaa..a#.a.##.a.#.a#\",
\"..a#..#####aa#a.aa.a##.a.a#.#aa#.a.#a.#aa#...a...#.a#.a##a.aa..#..a#..#####aa#a.aa.a##.a.a#.#aa#.a.#a.#aa#...\",
\"#aa#a....#aa####a.#a.#aa#a#..#.a#.aa#.aa##a.#a.#..a#a#..#a.a.aaa#aa#a....#aa####a.#a.#aa#a#..#.a#.aa#.aa##a.#\",
\"#aa...#aa#a.#.#.a.a#a#####a#.a.#a#a##.aa.#aaaa#.aaa..#..a...aa.a#aa...#aa#a.#.#.a.a#a#####a#.a.#a#a##.aa.#aaa\",
\"....#..#a##.#.##aa###aaaaa..a..#.#..##.##..#aa.#.aa#a.a.#aa.a.aa....#..#a##.#.##aa###aaaaa..a..#.#..##.##..#a\",
\"#a#a..#a#aa.a..a...##.a##.#a##.....a.##.aa.aa....aa..###a##..##a#a#a..#a#aa.a..a...##.a##.#a##.....a.##.aa.aa\",
\"aa#aa.a##.#a##a#...#.#a#aa#.#a#aaa.##a.##.aa.#.aaa##.a#a.##a#.a.aa#aa.a##.#a##a#...#.#a#aa#.#a#aaa.##a.##.aa.\",
\"a.##.a##.a#a.###..aa.#..a#.a#.a#.a#a..#a##a.a#..a...aaaaa###....a.##.a##.a#a.###..aa.#..a#.a#.a#.a#a..#a##a.a\",
\"#a......##.##a.aaaa#.##a...###.#.#a.a.aa#.#a..aaa...#.#aaaa###a##a......##.##a.aaaa#.##a...###.#.#a.a.aa#.#a.\",
\".aa#a#..a.#a...a..a##.###...####a..#aa#a..a..a.a#....a##a.a#..a..aa#a#..a.#a...a..a##.###...####a..#aa#a..a..\",
\".#..#.aa#aaaa.aa..a.aa#.#.#..a##.##a##a.aaaaa...#.##a#.aaaa##.a..#..#.aa#aaaa.aa..a.aa#.#.#..a##.##a##a.aaaaa\",
\"aa..#.a##a.a##aaaa#aa#aa#a.#a...a#a##a#.a.#a.a#..a.a..####a.a##.aa..#.a##a.a##aaaa#aa#aa#a.#a...a#a##a#.a.#a.\",
\"a#...##..#.#aa#.aa...#..#.aa#a...###.##a###a#.a#.#.a.#.a.aa#.#a#a#...##..#.#aa#.aa...#..#.aa#a...###.##a###a#\",
\"#a.aa.##a.aa#.a.###.a#a##aaa.aa###.a.#a####...a.a...#.a#a##.###a#a.aa.##a.aa#.a.###.a#a##aaa.aa###.a.#a####..\",
\"a##.a.#.a..a#...###aa###a.a.aaaa.##.#a.#.#.a.##aa.a#.a.###.#aa.#a##.a.#.a..a#...###aa###a.a.aaaa.##.#a.#.#.a.\",
\"#a##..##.#.aa..######.a#aaa#a.aa#a.###a.###.aaa.a.a#a#aaaa##a#aa#a##..##.#.aa..######.a#aaa#a.aa#a.###a.###.a\",
\".a#a#.a###a##.a....##aa..a.a...#a#.a#a..###.aa#.a..a#.a.aa#a.a##.a#a#.a###a##.a....##aa..a.a...#a#.a#a..###.a\",
\"a.a#.#aa#aa#.#aaa.#a.a##.aa..aa#a.a..a..a##.a#aa..#a.aa#a##a.a#.a.a#.#aa#aa#.#aaa.#a.a##.aa..aa#a.a..a..a##.a\",
\"..a##a.##.####a##..a#a.aa.aa.a.aa##.##a##.###a.aaa..#aaa#.#.a#....a##a.##.####a##..a#a.aa.aa.a.aa##.##a##.###\",
\".#..#....a..a#..####.#a##.#a#a.aa.##a.a.a#a.aaa#a#...a#.....a..a.#..#....a..a#..####.#a##.#a#a.aa.##a.a.a#a.a\",
\"..#.#.a...a.#.a#.a#a##.a...#a.#.###a....a#....a#.#.#a.##..#a...#..#.#.a...a.#.a#.a#a##.a...#a.#.###a....a#...\",
\"aa###.aaaa##aa..a#aaa.#a..a.a#.a#a..#..#.##aa#.##.a#.#.aaa#.aa#aaa###.aaaa##aa..a#aaa.#a..a.a#.a#a..#..#.##aa\",
\"#.a#a..#aaa..#aa##aa#..aa...a#####a..###.aa..a.#.#a.#a#.a#a.a#aa#.a#a..#aaa..#aa##aa#..aa...a#####a..###.aa..\",
\"a#a.aa##a..#.a#aa..#a#.#a#..#.#.###a#.#...aaaa##aa.#aaa...###.#aa#a.aa##a..#.a#aa..#a#.#a#..#.#.###a#.#...aaa\",
\"aa.#...##...a#.##.a..a###..#..#.#.a.a#aa.a##.aa.aaa..a.a##.a.#a#aa.#...##...a#.##.a..a###..#..#.#.a.a#aa.a##.\",
\"a###..##aa##.#.####.###..aa.a#..aa.###a#a##a#a#aa.##a.a.a#.#aaa#a###..##aa##.#.####.###..aa.a#..aa.###a#a##a#\",
\"#a#...a.aaa#.#a.a#aa#a#a#a.aa....a#.#..a.##....aa#.a.#a..#.##aaa#a#...a.aaa#.#a.a#aa#a#a#a.aa....a#.#..a.##..\",
\"a.aaa#a######a##.a#aaa.###.##aaa.aa##a#.#a#aaa###a..#a##a..aa.a.a.aaa#a######a##.a#aaa.###.##aaa.aa##a#.#a#aa\",
\"a##...##aaaa##a##..#aa#..##.#.a####a.......#a#a##a###a####a..a.aa##...##aaaa##a##..#aa#..##.#.a####a.......#a\",
\".a.a..#a....#a#...a#..#.a.#.aa.a..##a...#a.#aa...#.a##.aa.#.aa#..a.a..#a....#a#...a#..#.a.#.aa.a..##a...#a.#a\",
\".a..aa#..a.aa#.aaaaa##.a.a.a.a#.a.#..a....#.#a..aa...a...a#aaa#a.a..aa#..a.aa#.aaaaa##.a.a.a.a#.a.#..a....#.#\",
\"##.a.a.a.##...#..#aaa.a.#.a#.#a#.a..#aa.#a##aaaa.aa#.##.aa..aa####.a.a.a.##...#..#aaa.a.#.a#.#a#.a..#aa.#a##a\",
\"a.a.aa.a##a###aa##a.a.#.aa.....a##a#aa.#a#a###.#.#aa.aa.#aa#aa#.a.a.aa.a##a###aa##a.a.#.aa.....a##a#aa.#a#a##\",
\"#a.a#a##a###...a#aa#aa..aa.aa###aa#.a...a.aa#..a##..aa#a.##aa.a.#a.a#a##a###...a#aa#aa..aa.aa###aa#.a...a.aa#\",
\"a.aaa.aaa#aa#aa#.aa#....aa..###a.#a...#.#.#.....a.#.#aaa#aa..#a#a.aaa.aaa#aa#aa#.aa#....aa..###a.#a...#.#.#..\",
\".#a#.##a..a#aa#.###a.#a..a#.aa.##.#.a#.#..a#.a#a#a.aa#.##a####.#.#a#.##a..a#aa#.###a.#a..a#.aa.##.#.a#.#..a#.\",
\"####.###a##a.#aa#.#..a..##a.a#a##a##..aa##.#..#.....a..##..a.a..####.###a##a.#aa#.#..a..##a.a#a##a##..aa##.#.\",
\"..aa..a#aa#.#a#aaa.a#aa.#....#a#a.###a.#a.a.aaa.###.aaaa#.##aa.#..aa..a#aa#.#a#aaa.a#aa.#....#a#a.###a.#a.a.a\",
\"##a##aa#a#..#a..a.##a..#..##..aaa#a.aaa.#a..###.a.#.#aa#.##.a.a###a##aa#a#..#a..a.##a..#..##..aaa#a.aaa.#a..#\",
\".aaaaaaa#a#.aaa#.a#.aa.#a.a#aa..##aa#.##..#.a..#.a#.#a####aaaaa#.aaaaaaa#a#.aaa#.a#.aa.#a.a#aa..##aa#.##..#.a\",
\".a#.aa#.#a..a#aa###.aa#.a.a#.#.a.##.aaaa#####aaa.a.a#.a..a.aa#...a#.aa#.#a..a#aa###.aa#.a.a#.#.a.##.aaaa#####\",
\"#.#.##.##a.#.a#..#a###a.##aa..a.#a#a.##a#.##...a#.aa.aa#a..aa####.#.##.##a.#.a#..#a###a.##aa..a.#a#a.##a#.##.\",
\".#.##a..a..#a.#.#aa##a.#..#aa#.#a#...#..#.a#a#aa#.#a.aa..##.aaaa.#.##a..a..#a.#.#aa##a.#..#aa#.#a#...#..#.a#a\",
\".#aa...a.a##aaa..#a.aa.#a##...#a##aa#aa#..#...#.##...a.a...aa..a.#aa...a.a##aaa..#a.aa.#a##...#a##aa#aa#..#..\",
\".aaa###.a#..a#.#..aa.a##..##.#a#.a##a#a.#aa#....#..#a.a.a#aa.aa#.aaa###.a#..a#.#..aa.a##..##.#a#.a##a#a.#aa#.\",
\"#.#...aa.#aaaa.#####.##..a..a.##.#aa##aaaaa.aa.aa#.#....##aa.a###.#...aa.#aaaa.#####.##..a..a.##.#aa##aaaaa.a\",
\".a...a.aa.#a#aa#.aa.a.a...#a#..a#a.a.a##.#aaaa.#a##aa.#aa.#a.#.#.a...a.aa.#a#aa#.aa.a.a...#a#..a#a.a.a##.#aaa\",
\"#.a.a#aaaaa#aa#.a#.aaa.#aa#..a##aa#a#.aaa..##.a.aa.aa.a....##a.##.a.a#aaaaa#aa#.a#.aaa.#aa#..a##aa#a#.aaa..##\",
\"..##.aaaa#aa.aa.#a#a#a#.a##.###..##.a#.#aa.a#.a#.###a.###a#.#aa...##.aaaa#aa.aa.#a#a#a#.a##.###..##.a#.#aa.a#\",
\".a.#.aaa######a##.##a#..a#.#.aa.aa#aa...a#.a#.aaaa#a..a.aa#...a#.a.#.aaa######a##.##a#..a#.#.aa.aa#aa...a#.a#\",
\".a#.##a#a#####.##a.#.a#aaa#a.##a.aa##aa##aa.aa##..a###a#a.aaa..a.a#.##a#a#####.##a.#.a#aaa#a.##a.aa##aa##aa.a\",
\"a.aa#.##.a.a#aaa...aa.aaaa.#a#a.a.a.#aa..#a#aa#.aaaa.aa#....#a.#a.aa#.##.a.a#aaa...aa.aaaa.#a#a.a.a.#aa..#a#a\"
};
")
	  (setq url-current-mime-headers
		'(("content-type"              . "image/x-pixmap"))
		url-current-mime-type "image/x-pixmap"
		url-current-mime-viewer (mm-mime-info "image/x-pixmap" nil 5)))
	 ((string= "cXVvdGVz" (base64-encode node))
	  (insert
	   "

QN4UE7C2FZxu54iv23yPFj0H3fvIhW/0atSQC+ZSWvc/LM46zb4fAAA=
")
	  (setq url-current-mime-headers
		'(("content-transfer-encoding" . "base64")
		  ("content-encoding" . "x-gzip"))))
	 ((string= "emlwcHk=" (base64-encode node))
	  (insert
	   "

D/epxqqSuHtY9j//2Q==
")
	  (setq url-current-mime-viewer (mm-mime-info "image/jpeg" nil 5)
		url-current-mime-type "image/gif"
		url-current-mime-headers
		'(("content-transfer-encoding" . "base64")
		  ("content-type" . "image/gif"))))
	 ((string= "emacs" node)
	  (insert
	   "
<html>
  <head>
    <title>Versions of Emacs</title>
    <link rel=\"made\" href=\"mailto:wmperry+w3@cs.indiana.edu\">
    <link rel=\"stylesheet\" href=\"about:style\">
  </head>
  <body>
    <h1>Pointers to versions of emacs</h1>
    <dl>
      <dt> XEmacs
      <dd> An extremely X-aware version of emacs 19 from Sun and UIUC.
	This is the recommended emacsen to run Emacs-W3 on, allowing
	inlined images, inlined mpegs, and variable height fonts,
	among lots of other cool features.  Distribution is at <a
	href=\"ftp://ftp.xemacs.org/pub/xemacs/\">
	ftp://ftp.xemacs.org/pub/xemacs/</a>, or check out the <a
	href=\"http://www.xemacs.org/\">XEmacs web page</a>.
      <dt> Emacs 19
      <dd> A slightly-less X-aware version, direct from the FSF.  This
	is the second choice for most capable version of emacsen for
	emacs-w3, and allows for different fonts (of the same height),
	coloring of links, mouse, and menu support.  Distribution is
	at <a href=\"ftp://ftp.gnu.ai.mit.edu/pub/gnu/\">
	  ftp://ftp.gnu.ai.mit.edu/pub/gnu/</a>.
      <dt> Emacs 19 for NeXTStep
      <dd> A version of Emacs 19 that runs as a native NeXTStep
	application.  Fonts/colors/menus/mouse support.  Distribution
	is at <a
	  href=\"ftp://lynx.ps.uci.edu/pub/NeXT/emacs-19-for-NeXTstep\">
	  ftp://lynx.ps.uci.edu/pub/NeXT/emacs-19-for-NeXTstep</a>.
      <dt> OEmacs for DOS
      <dd> A version of Emacs-19 for MS-DOG.  Distribution at <a
	  href=\"ftp://oak.oakland.edu/pub/msdos/oemacs/\">OEmacs
	  4.1</a>.  Color and mouse support.  NOTE: Emacs-19 now has
	native support for MS-DOS, so this version will likely
	disappear.
      <dt> XEmacs for Windows
      <dd> A port of XEmacs 19.6 to Windows 3.1.  Shareware/Nagware
	application, but is well worth the money.  Distribution is at
	<a href=\"ftp://ftp.pearlsoft.com/\">
	  ftp://ftp.netcom.com/pub/pe/pearl/</a>.
      <dt> AmigaDOS
      <dd> A version of emacs 19 are available for AmigaDOS.  Emacs 19
	  is at <a href=\"ftp://ftp.wustl.edu:/pub/aminet/utils/gnu/\">
	  ftp://ftp.wustl.edu:/pub/aminet/utils/gnu/</a> under the
	  filename a2.0bEmacs-bin.LHA.
    </dl>
    <hr width=\"75%\">
    <p><cookie src=\"about:quotes\"></p>
  </body>
</html>
"))
	 ((string= "eggs" node)
	  (insert "
<html>
 <head>
  <title>Emacs-W3 Easter Eggs</title>
  <link rel=\"stylesheet\" href=\"about:style\">
 </head>
 <body>
  <h1>Emacs-W3 Easter Eggs</h1>
  <hr width=\"75%\">
  <p>
   Did you really think it would be that easy?  Read the source, or
experiment! :)
  </p>
  <hr width=\"75%\">
  <cookie href=\"about:quotes\">
 </body>
</html>
"))
	 ((string= "authors" node)
	  (insert "
<html>
 <head>
  <title>The Emacs-W3 Team</title>
  <link rel=\"stylesheet\" href=\"about:style\">
 </head>
 <body>
  <hr width=\"85%\">
  <hr width=\"75%\" label=\" The Emacs-W3 Team \">
  <hr width=\"65%\" label=\" 1993 - 1996 \">
  <hr width=\"55%\">
  <dl>
   <dt> Author
   <dd> William Perry &lt;wmperry+w3@cs.indiana.edu&gt; (Hey, that's <i>me</i>)!
        <br>
        The main author of Emacs-W3.  Currently unsure what to put in here
        about his employer. :)
   <dt> Supporting Cast
   <dd>
    <dl>
     <dt> Chuck Thompson &lt;cthomp@xemacs.org&gt;
     <dd> The master of the XEmacs display code, which makes many of
          the really cool things in Emacs-w3 possible.  I owe him
          copious amounts of beer.
     <dt> Ben Wing &lt;wing@666.com&gt;
     <dd> Another main XEmacs developer, and the man behind the
          PearlSoft port of XEmacs to windows.  I owe him his choice
          of alcoholic beverage as well.
     <dt> Jamie Zawinski &lt;jwz@netscape.com&gt;
     <dd> The main Lucid Emacs developer, now working for netscape on
          Mozilla.  Helped me do the initial integration with Lucid
          Emacs by telling me about stupid mistakes I made. :)
     <dt> Marc Andreesen, Lou Montulli &lt;marca/montulli@netscape.com&gt;
     <dd> The inspiration behind my efforts to create Emacs-W3
          initially, because I hated the interfaces to Mosaic
          and Lynx so much.
    </dl>
  </dl>
  <p><cookie href=\"about:quotes\"></p>
 </body>
</html>
"))

	 ((string= "cGhvdG8=" (base64-encode node))
	  (insert "

oTW7tBCrAr/JoNOKGGwLByEAADs=
")
	  (setq url-current-mime-viewer (mm-mime-info "image/gif" nil 5)
		url-current-mime-type "image/gif"
		url-current-mime-headers
		'(("content-transfer-encoding" . "base64")
		  ("content-type" . "image/gif"))))
	 ((string= "bW96aWxsYQ==" (base64-encode node))
	  (insert
	   "
PGh0bWw+CiAgPGhlYWQ+CiAgICA8dGl0bGU+VGhlIEJvb2sgb2YgTW96aWxsYSwgMTg6Mjc8L3Rp
dGxlPgogIDwvaGVhZD4KICA8Ym9keSBiZ2NvbG9yPSIjODAwMDAwIiB0ZXh0PSIjRkZGRkZGIj4K
ICAgIDxociBhbGlnbj1jZW50ZXIgd2lkdGg9Ijc1JSI+CiAgICA8cD4KICAgICBBbmQgSSBzYXcg
YW4gU0dNTCBwdXJpc3QgY29tZSBkb3duIGZyb20gTUlULCBoYXZpbmcgdGhlIGtleSBvZgogICAg
IHRoZSBib3R0b21sZXNzIHBpdCBhbmQgYSBncmVhdCBjaGFpbiBpbiBoaXMgaGFuZC4KICAgIDwv
cD4KICAgIDxwPgogICAgIEFuZCBoZSBsYWlkIGhvbGQgb24gTW96aWxsYSwgdGhhdCBvbGQgc2Vy
cGVudCwgd2hpY2ggaXMgdGhlCiAgICAgPHNlY3JldD5NYXJjQTwvc2VjcmV0PiwgYW5kIHRoZSA8
c2VjcmV0PkppbUM8L3NlY3JldD4sIGFuZAogICAgIGJvdW5kIGhpbSBhIHRob3VzYW5kIHllYXJz
LgogICAgPC9wPgogICAgPHA+CiAgICAgQW5kIGNhc3QgaGltIGludG8gdGhlIGJvdHRvbWxlc3Mg
cGl0LCBhbmQgc2h1dCBoaW0gdXAsIGFuZCBzZXQgYQogICAgIHNlYWwgdXBvbiBoaW0sIHRoYXQg
aGUgc2hvdWxkIGRlY2VpdmUgdGhlIG5ldCBubyBtb3JlLCB0aWxsIHRoZQogICAgIHRob3VzYW5k
IHllYXJzIHNob3VsZCBiZSBmdWxmaWxsZWQ6ICBhbmQgYWZ0ZXIgdGhhdCBoZSBtdXN0IGJlCiAg
ICAgbG9vc2VkIGEgbGl0dGxlIHNlYXNvbi4KICAgIDwvcD4KICAgIDxociB3aWR0aD0iNzUlIj4K
ICAgIDxiPjxociB3aWR0aD0iODUlIiBsYWJlbD0iIGZyb20gYGBUaGUgQm9vayBvZiBNb3ppbGxh
JycsIDE4OjI3ICI+PC9iPgogICAgPGhyIHdpZHRoPSI3NSUiPgogIDwvYm9keT4KPC9odG1sPgo=
")
	  (setq url-current-mime-headers
		'(("content-transfer-encoding" . "base64"))))
	 (t
	  (insert "
<html>
 <head>
  
 </head>
 <body>
  <p>
    I don't know what you are talking about.  What about " node "?
  </p>
  <hr width=\"75%\">
  <hype>
 </body>
</html>
")))))))

(provide 'w3-about)
