;;; custom-load.el --- automatically extracted custom dependencies


;;; Code:

(custom-add-loads 'extensions '("wid-edit"))
(custom-add-loads 'custom-buffer '("cus-edit"))
(custom-add-loads 'custom-faces '("cus-edit"))
(custom-add-loads 'widgets '("wid-browse" "wid-edit"))
(custom-add-loads 'environment '("cus-edit"))
(custom-add-loads 'custom-menu '("cus-edit"))
(custom-add-loads 'internal '("cus-edit"))
(custom-add-loads 'hypermedia '("wid-edit"))
(custom-add-loads 'applications '("cus-edit"))
(custom-add-loads 'help '("cus-edit"))
(custom-add-loads 'widget-browse '("wid-browse"))
(custom-add-loads 'widget-documentation '("wid-edit"))
(custom-add-loads 'customize '("cus-edit" "wid-edit"))
(custom-add-loads 'custom-browse '("cus-edit"))
(custom-add-loads 'abbrev '("cus-edit"))
(custom-add-loads 'programming '("cus-edit"))
(custom-add-loads 'widget-button '("wid-edit"))
(custom-add-loads 'files '("cus-edit"))
(custom-add-loads 'external '("cus-edit"))
(custom-add-loads 'development '("cus-edit"))
(custom-add-loads 'widget-faces '("wid-edit"))
(custom-add-loads 'languages '("cus-edit"))
(custom-add-loads 'custom-magic-faces '("cus-edit"))
(custom-add-loads 'faces '("cus-edit" "wid-edit"))
(custom-add-loads 'emacs '("cus-edit"))
(custom-add-loads 'processes '("cus-edit"))
(custom-add-loads 'wp '("cus-edit"))
(custom-add-loads 'editing '("cus-edit"))
(custom-add-loads 'i18n '("cus-edit"))

;;; custom-load.el ends here
