;;; custom-load.el --- automatically extracted custom dependencies


;;; Code:

(custom-add-loads 'shell '("term"))
(custom-add-loads 'term '("term"))
(custom-add-loads 'processes '("term"))
(custom-add-loads 'unix '("term"))

;;; custom-load.el ends here
