;;; custom-load.el --- automatically extracted custom dependencies


;;; Code:

(custom-add-loads 'extensions '("auto-show" "strokes"))
(custom-add-loads 'message '("sendmail"))
(custom-add-loads 'prolog '("prolog"))
(custom-add-loads 'auto-show '("auto-show"))
(custom-add-loads 'mouse '("outl-mouse" "strokes"))
(custom-add-loads 'mail-abbrevs '("mail-abbrevs"))
(custom-add-loads 'pascal '("pascal"))
(custom-add-loads 'tex '("bibtex" "reftex" "texinfo"))
(custom-add-loads 'tcl '("tcl"))
(custom-add-loads 'icon '("icon"))
(custom-add-loads 'texinfo '("texinfo"))
(custom-add-loads 'xrdb '("xrdb-mode"))
(custom-add-loads 'mail '("mail-abbrevs"))
(custom-add-loads 'fortran-indent '("fortran"))
(custom-add-loads 'docs '("texinfo"))
(custom-add-loads 'lisp-indent '("cl-indent"))
(custom-add-loads 'tools '("hideshow" "lazy-shot" "make-mode"))
(custom-add-loads 'lisp '("lisp-mode" "cl-indent" "strokes"))
(custom-add-loads 'reftex '("reftex"))
(custom-add-loads 'outlines '("hideshow" "outl-mouse" "whitespace-mode"))
(custom-add-loads 'f90 '("f90"))
(custom-add-loads 'perl '("cperl-mode"))
(custom-add-loads 'asm '("asm-mode"))
(custom-add-loads 'ada '("ada-mode"))
(custom-add-loads 'reftex-label-support '("reftex"))
(custom-add-loads 'data '("arc-mode" "xrdb-mode"))
(custom-add-loads 'fortran-comment '("fortran"))
(custom-add-loads 'outl-mouse '("outl-mouse"))
(custom-add-loads 'frames '("rsz-minibuf"))
(custom-add-loads 'cperl-electric '("cperl-mode"))
(custom-add-loads 'verilog '("verilog-mode"))
(custom-add-loads 'abbrev '("abbrev"))
(custom-add-loads 'f90-indent '("f90"))
(custom-add-loads 'strokes '("strokes"))
(custom-add-loads 'lazy-shot '("lazy-shot"))
(custom-add-loads 'archive-lzh '("arc-mode"))
(custom-add-loads 'scribe '("scribe"))
(custom-add-loads 'archive '("arc-mode"))
(custom-add-loads 'c-macro '("cmacexp"))
(custom-add-loads 'bibtex '("bibtex"))
(custom-add-loads 'vrml '("vrml-mode"))
(custom-add-loads 'simula '("simula"))
(custom-add-loads 'archive-arc '("arc-mode"))
(custom-add-loads 'fortran '("f90" "fortran"))
(custom-add-loads 'cperl-faces '("cperl-mode"))
(custom-add-loads 'resize-minibuffer '("rsz-minibuf"))
(custom-add-loads 'languages '("ada-mode" "asm-mode" "cperl-mode" "fortran" "icon" "pascal" "prolog" "python-mode" "rexx-mode" "sh-script" "simula" "tcl" "verilog-mode" "vhdl-mode" "vrml-mode" "winmgr-mode" "xrdb-mode"))
(custom-add-loads 'cperl-indent '("cperl-mode"))
(custom-add-loads 'archive-zoo '("arc-mode"))
(custom-add-loads 'archive-zip '("arc-mode"))
(custom-add-loads 'display '("auto-show"))
(custom-add-loads 'faces '("cperl-mode" "lazy-shot"))
(custom-add-loads 'hideshow '("hideshow"))
(custom-add-loads 'vhdl '("vhdl-mode"))
(custom-add-loads 'enriched '("enriched"))
(custom-add-loads 'processes '("executable"))
(custom-add-loads 'rexx '("rexx-mode"))
(custom-add-loads 'executable '("executable"))
(custom-add-loads 'wp '("enriched" "scribe" "texinfo"))
(custom-add-loads 'sh '("sh-script"))
(custom-add-loads 'python '("python-mode"))
(custom-add-loads 'makefile-mode '("make-mode"))
(custom-add-loads 'whitespace '("whitespace-mode"))
(custom-add-loads 'matching '("whitespace-mode"))
(custom-add-loads 'winmgr '("winmgr-mode"))
(custom-add-loads 'unix '("sh-script"))
(custom-add-loads 'bibtex-autokey '("bibtex"))
(custom-add-loads 'c '("cmacexp"))

;;; custom-load.el ends here
