;; Copyright (C) 1992 Free Software Foundation, Inc.
;; This file is part of Mule (MULtilingual Enhancement of GNU Emacs).

;; Mule is free software distributed in the form of patches to GNU Emacs.
;; You can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 1, or (at your option)
;; any later version.

;; Mule is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;; 92.3.23  modified for Mule Ver.0.9.1 by K.Handa <handa@etl.go.jp>
;;	defrule -> its-defrule, define-its-mode -> its-define-mode
;;; 93.11.19 modified for Mule Ver.1.1 by K.Handa <handa@etl.go.jp>
;;;	"P" produces "ㅖ".

;(load "its-hanhira" t t)

(its-define-mode "hangul" "HG" t)
;(reset-its-mode  "hangul")
;
;(setq its:*current-map* (its:get-mode-map "hangul"))
;
;(defun hangul ()
;  (interactive)
;  (its:select-mode "hangul")
;  (x-set-kfont "16x16hangulg")
;  (redraw-display)
;  (setq default-kanji-fileio-code 3)	;;EUC
;  (setq kanji-fileio-code 3)	;;EUC
;  (setq default-kanji-fileio-code-find-file-not-found 3)
;)

;(defun japanese ()
;  (interactive)
;  (its:select-mode "roma-kana")
;  (x-set-kfont "16x16kanji")
;  (redraw-display)
;  (setq default-kanji-fileio-code 2)	;;JIS
;  (setq kanji-fileio-code 2)	;;JIS
;  (setq default-kanji-fileio-code-find-file-not-found 2)
;)

(let ((its:make-terminal-state 'its:make-terminal-state-hangul))

;(its-defrule* "J" ''(its:select-mode "hangul-kana"))

;(its-defrule "G"	"ㄱ")
;(its-defrule "N"	"ㄴ")
;(its-defrule "D"	"ㄷ")
;(its-defrule "R"	"ㄹ")
;(its-defrule "M"	"ㅁ")
;(its-defrule "B"	"ㅂ")
;(its-defrule "S"	"ㅅ")
;(its-defrule "Z"	"ㅇ")
;(its-defrule "J"	"ㅈ")
;(its-defrule "C"	"ㅊ")
;(its-defrule "K"	"ㅋ")
;(its-defrule "T"	"ㅌ")
;(its-defrule "P"	"ㅍ")
;(its-defrule "H"	"ㅎ")
;;ㄲ
;;ㄸ
;;ㅃ
;;ㅉ
;;ㅆ
;;(its-defrule "A"	"ㅏ")
;;(its-defrule "JA"	"ㅑ")
;;(its-defrule "E"	"ㅓ")
;;(its-defrule "JE"	"ㅕ")
;;(its-defrule "O"	"ㅗ")
;;(its-defrule "JO"	"ㅛ")
;;(its-defrule "U"	"ㅜ")
;;(its-defrule "JU"	"ㅠ")
;;(its-defrule "EU"	"ㅡ")
;;(its-defrule "I"	"ㅣ")

(its-defrule "r"	"ㄱ")
(its-defrule "s"	"ㄴ")
(its-defrule "e"	"ㄷ")
(its-defrule "f"	"ㄹ")
(its-defrule "a"	"ㅁ")
(its-defrule "q"	"ㅂ")
(its-defrule "t"	"ㅅ")
(its-defrule "d"	"ㅇ")
(its-defrule "w"	"ㅈ")
(its-defrule "c"	"ㅊ")
(its-defrule "z"	"ㅋ")
(its-defrule "x"	"ㅌ")
(its-defrule "v"	"ㅍ")
(its-defrule "g"	"ㅎ")

(its-defrule "R"	"ㄲ")
(its-defrule "E"	"ㄸ")
(its-defrule "Q"	"ㅃ")
(its-defrule "W"	"ㅉ")
(its-defrule "T"	"ㅆ")
(its-defrule "P"	"ㅖ")		;93.11.19 by K.Handa

;(its-defrule "k"	"ㅏ")
;(its-defrule "i"	"ㅑ")
;(its-defrule "j"	"ㅓ")
;(its-defrule "u"	"ㅕ")
;(its-defrule "h"	"ㅗ")
;(its-defrule "y"	"ㅛ")
;(its-defrule "n"	"ㅜ")
;(its-defrule "b"	"ㅠ")
;(its-defrule "m"	"ㅡ")
;(its-defrule "l"	"ㅣ")




(its-defrule "rk"	"가")
(its-defrule "rkr"	"각")
(its-defrule "rks"	"간")
(its-defrule "rke"	"갇")
(its-defrule "rkf"	"갈")
(its-defrule "rkfr"	"갉")
(its-defrule "rkfa"	"갊")
(its-defrule "rka"	"감")
(its-defrule "rkq"	"갑")
(its-defrule "rkqt"	"값")
(its-defrule "rkt"	"갓")
(its-defrule "rkT"	"갔")
(its-defrule "rkd"	"강")
(its-defrule "rkw"	"갖")
(its-defrule "rkc"	"갗")
(its-defrule "rkx"	"같")
(its-defrule "rkv"	"갚")
(its-defrule "rkg"	"갛")
(its-defrule "ro"	"개")
(its-defrule "ror"	"객")
(its-defrule "ros"	"갠")
(its-defrule "rof"	"갤")
(its-defrule "roa"	"갬")
(its-defrule "roq"	"갭")
(its-defrule "rot"	"갯")
(its-defrule "roT"	"갰")
(its-defrule "rod"	"갱")
(its-defrule "ri"	"갸")
(its-defrule "rir"	"갹")
(its-defrule "ris"	"갼")
(its-defrule "rif"	"걀")
(its-defrule "rit"	"걋")
(its-defrule "rid"	"걍")
(its-defrule "rO"	"걔")
(its-defrule "rOs"	"걘")
(its-defrule "rOf"	"걜")
(its-defrule "rj"	"거")
(its-defrule "rjr"	"걱")
(its-defrule "rjs"	"건")
(its-defrule "rje"	"걷")
(its-defrule "rjf"	"걸")
(its-defrule "rjfa"	"걺")
(its-defrule "rja"	"검")
(its-defrule "rjq"	"겁")
(its-defrule "rjt"	"것")
(its-defrule "rjT"	"겄")
(its-defrule "rjd"	"겅")
(its-defrule "rjw"	"겆")
(its-defrule "rjx"	"겉")
(its-defrule "rjv"	"겊")
(its-defrule "rjg"	"겋")
(its-defrule "rp"	"게")
(its-defrule "rps"	"겐")
(its-defrule "rpf"	"겔")
(its-defrule "rpa"	"겜")
(its-defrule "rpq"	"겝")
(its-defrule "rpt"	"겟")
(its-defrule "rpd"	"겡")
(its-defrule "rpT"	"겠")
(its-defrule "ru"	"겨")
(its-defrule "rur"	"격")
(its-defrule "ruR"	"겪")
(its-defrule "rus"	"견")
(its-defrule "rue"	"겯")
(its-defrule "ruf"	"결")
(its-defrule "rua"	"겸")
(its-defrule "ruq"	"겹")
(its-defrule "rut"	"겻")
(its-defrule "ruT"	"겼")
(its-defrule "rud"	"경")
(its-defrule "rux"	"곁")
(its-defrule "rP"	"계")
(its-defrule "rPs"	"곈")
(its-defrule "rPf"	"곌")
(its-defrule "rPq"	"곕")
(its-defrule "rPt"	"곗")
(its-defrule "rh"	"고")
(its-defrule "rhr"	"곡")
(its-defrule "rhs"	"곤")
(its-defrule "rhe"	"곧")
(its-defrule "rhf"	"골")
(its-defrule "rhfa"	"곪")
(its-defrule "rhft"	"곬")
(its-defrule "rhfg"	"곯")
(its-defrule "rha"	"곰")
(its-defrule "rhq"	"곱")
(its-defrule "rht"	"곳")
(its-defrule "rhd"	"공")
(its-defrule "rhw"	"곶")
(its-defrule "rhk"	"과")
(its-defrule "rhkr"	"곽")
(its-defrule "rhks"	"관")
(its-defrule "rhkf"	"괄")
(its-defrule "rhkfa"	"괆")
(its-defrule "rhka"	"괌")
(its-defrule "rhkq"	"괍")
(its-defrule "rhkt"	"괏")
(its-defrule "rhkd"	"광")
(its-defrule "rho"	"괘")
(its-defrule "rhos"	"괜")
(its-defrule "rhof"	"괠")
(its-defrule "rhoq"	"괩")
(its-defrule "rhoT"	"괬")
(its-defrule "rhod"	"괭")
(its-defrule "rhl"	"괴")
(its-defrule "rhlr"	"괵")
(its-defrule "rhls"	"괸")
(its-defrule "rhlf"	"괼")
(its-defrule "rhla"	"굄")
(its-defrule "rhlq"	"굅")
(its-defrule "rhlt"	"굇")
(its-defrule "rhld"	"굉")
(its-defrule "ry"	"교")
(its-defrule "rys"	"굔")
(its-defrule "ryf"	"굘")
(its-defrule "ryq"	"굡")
(its-defrule "ryt"	"굣")
(its-defrule "rn"	"구")
(its-defrule "rnr"	"국")
(its-defrule "rns"	"군")
(its-defrule "rne"	"굳")
(its-defrule "rnf"	"굴")
(its-defrule "rnfr"	"굵")
(its-defrule "rnfa"	"굶")
(its-defrule "rnfg"	"굻")
(its-defrule "rna"	"굼")
(its-defrule "rnq"	"굽")
(its-defrule "rnt"	"굿")
(its-defrule "rnd"	"궁")
(its-defrule "rnw"	"궂")
(its-defrule "rnj"	"궈")
(its-defrule "rnjr"	"궉")
(its-defrule "rnjs"	"권")
(its-defrule "rnjf"	"궐")
(its-defrule "rnjT"	"궜")
(its-defrule "rnjd"	"궝")
(its-defrule "rnp"	"궤")
(its-defrule "rnpt"	"궷")
(its-defrule "rnl"	"귀")
(its-defrule "rnlr"	"귁")
(its-defrule "rnls"	"귄")
(its-defrule "rnlf"	"귈")
(its-defrule "rnla"	"귐")
(its-defrule "rnlq"	"귑")
(its-defrule "rnlt"	"귓")
(its-defrule "rb"	"규")
(its-defrule "rbs"	"균")
(its-defrule "rbf"	"귤")
(its-defrule "rm"	"그")
(its-defrule "rmr"	"극")
(its-defrule "rms"	"근")
(its-defrule "rme"	"귿")
(its-defrule "rmf"	"글")
(its-defrule "rmfr"	"긁")
(its-defrule "rma"	"금")
(its-defrule "rmq"	"급")
(its-defrule "rmt"	"긋")
(its-defrule "rmd"	"긍")
(its-defrule "rml"	"긔")
(its-defrule "rl"	"기")
(its-defrule "rlr"	"긱")
(its-defrule "rls"	"긴")
(its-defrule "rle"	"긷")
(its-defrule "rlf"	"길")
(its-defrule "rlfa"	"긺")
(its-defrule "rla"	"김")
(its-defrule "rlq"	"깁")
(its-defrule "rlt"	"깃")
(its-defrule "rld"	"깅")
(its-defrule "rlw"	"깆")
(its-defrule "rlv"	"깊")
(its-defrule "Rk"	"까")
(its-defrule "Rkr"	"깍")
(its-defrule "RkR"	"깎")
(its-defrule "Rks"	"깐")
(its-defrule "Rkf"	"깔")
(its-defrule "Rkfa"	"깖")
(its-defrule "Rka"	"깜")
(its-defrule "Rkq"	"깝")
(its-defrule "Rkt"	"깟")
(its-defrule "RkT"	"깠")
(its-defrule "Rkd"	"깡")
(its-defrule "Rkx"	"깥")
(its-defrule "Ro"	"깨")
(its-defrule "Ror"	"깩")
(its-defrule "Ros"	"깬")
(its-defrule "Rof"	"깰")
(its-defrule "Roa"	"깸")
(its-defrule "Roq"	"깹")
(its-defrule "Rot"	"깻")
(its-defrule "RoT"	"깼")
(its-defrule "Rod"	"깽")
(its-defrule "Ri"	"꺄")
(its-defrule "Rir"	"꺅")
(its-defrule "Rif"	"꺌")
(its-defrule "Rj"	"꺼")
(its-defrule "Rjr"	"꺽")
(its-defrule "RjR"	"꺾")
(its-defrule "Rjs"	"껀")
(its-defrule "Rjf"	"껄")
(its-defrule "Rja"	"껌")
(its-defrule "Rjq"	"껍")
(its-defrule "Rjt"	"껏")
(its-defrule "RjT"	"껐")
(its-defrule "Rjd"	"껑")
(its-defrule "Rp"	"께")
(its-defrule "Rpr"	"껙")
(its-defrule "Rps"	"껜")
(its-defrule "Rpa"	"껨")
(its-defrule "Rpt"	"껫")
(its-defrule "Rpd"	"껭")
(its-defrule "Ru"	"껴")
(its-defrule "Rus"	"껸")
(its-defrule "Ruf"	"껼")
(its-defrule "Rut"	"꼇")
(its-defrule "RuT"	"꼈")
(its-defrule "Rux"	"꼍")
(its-defrule "RP"	"꼐")
(its-defrule "Rh"	"꼬")
(its-defrule "Rhr"	"꼭")
(its-defrule "Rhs"	"꼰")
(its-defrule "Rhsg"	"꼲")
(its-defrule "Rhf"	"꼴")
(its-defrule "Rha"	"꼼")
(its-defrule "Rhq"	"꼽")
(its-defrule "Rht"	"꼿")
(its-defrule "Rhd"	"꽁")
(its-defrule "Rhw"	"꽂")
(its-defrule "Rhc"	"꽃")
(its-defrule "Rhk"	"꽈")
(its-defrule "Rhkr"	"꽉")
(its-defrule "Rhkf"	"꽐")
(its-defrule "RhkT"	"꽜")
(its-defrule "Rhkd"	"꽝")
(its-defrule "Rho"	"꽤")
(its-defrule "Rhor"	"꽥")
(its-defrule "Rhod"	"꽹")
(its-defrule "Rhl"	"꾀")
(its-defrule "Rhls"	"꾄")
(its-defrule "Rhlf"	"꾈")
(its-defrule "Rhla"	"꾐")
(its-defrule "Rhlq"	"꾑")
(its-defrule "Rhld"	"꾕")
(its-defrule "Ry"	"꾜")
(its-defrule "Rn"	"꾸")
(its-defrule "Rnr"	"꾹")
(its-defrule "Rns"	"꾼")
(its-defrule "Rnf"	"꿀")
(its-defrule "Rnfg"	"꿇")
(its-defrule "Rna"	"꿈")
(its-defrule "Rnq"	"꿉")
(its-defrule "Rnt"	"꿋")
(its-defrule "Rnd"	"꿍")
(its-defrule "Rnw"	"꿎")
(its-defrule "Rnj"	"꿔")
(its-defrule "Rnjf"	"꿜")
(its-defrule "RnjT"	"꿨")
(its-defrule "Rnjd"	"꿩")
(its-defrule "Rnp"	"꿰")
(its-defrule "Rnpr"	"꿱")
(its-defrule "Rnps"	"꿴")
(its-defrule "Rnpf"	"꿸")
(its-defrule "Rnpa"	"뀀")
(its-defrule "Rnpq"	"뀁")
(its-defrule "RnpT"	"뀄")
(its-defrule "Rnl"	"뀌")
(its-defrule "Rnls"	"뀐")
(its-defrule "Rnlf"	"뀔")
(its-defrule "Rnla"	"뀜")
(its-defrule "Rnlq"	"뀝")
(its-defrule "Rb"	"뀨")
(its-defrule "Rm"	"끄")
(its-defrule "Rmr"	"끅")
(its-defrule "Rms"	"끈")
(its-defrule "Rmsg"	"끊")
(its-defrule "Rmf"	"끌")
(its-defrule "Rmfa"	"끎")
(its-defrule "Rmfg"	"끓")
(its-defrule "Rma"	"끔")
(its-defrule "Rmq"	"끕")
(its-defrule "Rmt"	"끗")
(its-defrule "Rmd"	"끙")
(its-defrule "Rmx"	"끝")
(its-defrule "Rl"	"끼")
(its-defrule "Rlr"	"끽")
(its-defrule "Rls"	"낀")
(its-defrule "Rlf"	"낄")
(its-defrule "Rla"	"낌")
(its-defrule "Rlq"	"낍")
(its-defrule "Rlt"	"낏")
(its-defrule "Rld"	"낑")
(its-defrule "sk"	"나")
(its-defrule "skr"	"낙")
(its-defrule "skR"	"낚")
(its-defrule "sks"	"난")
(its-defrule "ske"	"낟")
(its-defrule "skf"	"날")
(its-defrule "skfr"	"낡")
(its-defrule "skfa"	"낢")
(its-defrule "ska"	"남")
(its-defrule "skq"	"납")
(its-defrule "skt"	"낫")
(its-defrule "skT"	"났")
(its-defrule "skd"	"낭")
(its-defrule "skw"	"낮")
(its-defrule "skc"	"낯")
(its-defrule "skx"	"낱")
(its-defrule "skg"	"낳")
(its-defrule "so"	"내")
(its-defrule "sor"	"낵")
(its-defrule "sos"	"낸")
(its-defrule "sof"	"낼")
(its-defrule "soa"	"냄")
(its-defrule "soq"	"냅")
(its-defrule "sot"	"냇")
(its-defrule "soT"	"냈")
(its-defrule "sod"	"냉")
(its-defrule "si"	"냐")
(its-defrule "sir"	"냑")
(its-defrule "sis"	"냔")
(its-defrule "sif"	"냘")
(its-defrule "sia"	"냠")
(its-defrule "sid"	"냥")
(its-defrule "sj"	"너")
(its-defrule "sjr"	"넉")
(its-defrule "sjrt"	"넋")
(its-defrule "sjs"	"넌")
(its-defrule "sjf"	"널")
(its-defrule "sjfa"	"넒")
(its-defrule "sjfq"	"넓")
(its-defrule "sja"	"넘")
(its-defrule "sjq"	"넙")
(its-defrule "sjt"	"넛")
(its-defrule "sjT"	"넜")
(its-defrule "sjd"	"넝")
(its-defrule "sjg"	"넣")
(its-defrule "sp"	"네")
(its-defrule "spr"	"넥")
(its-defrule "sps"	"넨")
(its-defrule "spf"	"넬")
(its-defrule "spa"	"넴")
(its-defrule "spq"	"넵")
(its-defrule "spt"	"넷")
(its-defrule "spT"	"넸")
(its-defrule "spd"	"넹")
(its-defrule "su"	"녀")
(its-defrule "sur"	"녁")
(its-defrule "sus"	"년")
(its-defrule "suf"	"녈")
(its-defrule "sua"	"념")
(its-defrule "suq"	"녑")
(its-defrule "suT"	"녔")
(its-defrule "sud"	"녕")
(its-defrule "suz"	"녘")
(its-defrule "sP"	"녜")
(its-defrule "sPs"	"녠")
(its-defrule "sh"	"노")
(its-defrule "shr"	"녹")
(its-defrule "shs"	"논")
(its-defrule "shf"	"놀")
(its-defrule "shfa"	"놂")
(its-defrule "sha"	"놈")
(its-defrule "shq"	"놉")
(its-defrule "sht"	"놋")
(its-defrule "shd"	"농")
(its-defrule "shv"	"높")
(its-defrule "shg"	"놓")
(its-defrule "shk"	"놔")
(its-defrule "shks"	"놘")
(its-defrule "shkf"	"놜")
(its-defrule "shkT"	"놨")
(its-defrule "shl"	"뇌")
(its-defrule "shls"	"뇐")
(its-defrule "shlf"	"뇔")
(its-defrule "shla"	"뇜")
(its-defrule "shlq"	"뇝")
(its-defrule "shlt"	"뇟")
(its-defrule "sy"	"뇨")
(its-defrule "syr"	"뇩")
(its-defrule "sys"	"뇬")
(its-defrule "syf"	"뇰")
(its-defrule "syq"	"뇹")
(its-defrule "syt"	"뇻")
(its-defrule "syd"	"뇽")
(its-defrule "sn"	"누")
(its-defrule "snr"	"눅")
(its-defrule "sns"	"눈")
(its-defrule "sne"	"눋")
(its-defrule "snf"	"눌")
(its-defrule "sna"	"눔")
(its-defrule "snq"	"눕")
(its-defrule "snt"	"눗")
(its-defrule "snd"	"눙")
(its-defrule "snj"	"눠")
(its-defrule "snjT"	"눴")
(its-defrule "snp"	"눼")
(its-defrule "snl"	"뉘")
(its-defrule "snls"	"뉜")
(its-defrule "snlf"	"뉠")
(its-defrule "snla"	"뉨")
(its-defrule "snlq"	"뉩")
(its-defrule "sb"	"뉴")
(its-defrule "sbr"	"뉵")
(its-defrule "sbf"	"뉼")
(its-defrule "sba"	"늄")
(its-defrule "sbq"	"늅")
(its-defrule "sbd"	"늉")
(its-defrule "sm"	"느")
(its-defrule "smr"	"늑")
(its-defrule "sms"	"는")
(its-defrule "smf"	"늘")
(its-defrule "smfr"	"늙")
(its-defrule "smfa"	"늚")
(its-defrule "sma"	"늠")
(its-defrule "smq"	"늡")
(its-defrule "smt"	"늣")
(its-defrule "smd"	"능")
(its-defrule "smw"	"늦")
(its-defrule "smv"	"늪")
(its-defrule "sml"	"늬")
(its-defrule "smls"	"늰")
(its-defrule "smlf"	"늴")
(its-defrule "sl"	"니")
(its-defrule "slr"	"닉")
(its-defrule "sls"	"닌")
(its-defrule "slf"	"닐")
(its-defrule "slfa"	"닒")
(its-defrule "sla"	"님")
(its-defrule "slq"	"닙")
(its-defrule "slt"	"닛")
(its-defrule "sld"	"닝")
(its-defrule "slv"	"닢")
(its-defrule "ek"	"다")
(its-defrule "ekr"	"닥")
(its-defrule "ekR"	"닦")
(its-defrule "eks"	"단")
(its-defrule "eke"	"닫")
(its-defrule "ekf"	"달")
(its-defrule "ekfr"	"닭")
(its-defrule "ekfa"	"닮")
(its-defrule "ekfq"	"닯")
(its-defrule "ekfg"	"닳")
(its-defrule "eka"	"담")
(its-defrule "ekq"	"답")
(its-defrule "ekt"	"닷")
(its-defrule "ekT"	"닸")
(its-defrule "ekd"	"당")
(its-defrule "ekw"	"닺")
(its-defrule "ekc"	"닻")
(its-defrule "ekg"	"닿")
(its-defrule "eo"	"대")
(its-defrule "eor"	"댁")
(its-defrule "eos"	"댄")
(its-defrule "eof"	"댈")
(its-defrule "eoa"	"댐")
(its-defrule "eoq"	"댑")
(its-defrule "eot"	"댓")
(its-defrule "eoT"	"댔")
(its-defrule "eod"	"댕")
(its-defrule "ei"	"댜")
(its-defrule "ej"	"더")
(its-defrule "ejr"	"덕")
(its-defrule "ejR"	"덖")
(its-defrule "ejs"	"던")
(its-defrule "eje"	"덛")
(its-defrule "ejf"	"덜")
(its-defrule "ejfa"	"덞")
(its-defrule "ejfq"	"덟")
(its-defrule "eja"	"덤")
(its-defrule "ejq"	"덥")
(its-defrule "ejt"	"덧")
(its-defrule "ejd"	"덩")
(its-defrule "ejc"	"덫")
(its-defrule "ejv"	"덮")
(its-defrule "ep"	"데")
(its-defrule "epr"	"덱")
(its-defrule "eps"	"덴")
(its-defrule "epf"	"델")
(its-defrule "epa"	"뎀")
(its-defrule "epq"	"뎁")
(its-defrule "ept"	"뎃")
(its-defrule "epT"	"뎄")
(its-defrule "epd"	"뎅")
(its-defrule "eu"	"뎌")
(its-defrule "eus"	"뎐")
(its-defrule "euf"	"뎔")
(its-defrule "euT"	"뎠")
(its-defrule "eud"	"뎡")
(its-defrule "eP"	"뎨")
(its-defrule "ePs"	"뎬")
(its-defrule "eh"	"도")
(its-defrule "ehr"	"독")
(its-defrule "ehs"	"돈")
(its-defrule "ehe"	"돋")
(its-defrule "ehf"	"돌")
(its-defrule "ehfa"	"돎")
(its-defrule "ehft"	"돐")
(its-defrule "eha"	"돔")
(its-defrule "ehq"	"돕")
(its-defrule "eht"	"돗")
(its-defrule "ehd"	"동")
(its-defrule "ehc"	"돛")
(its-defrule "ehx"	"돝")
(its-defrule "ehk"	"돠")
(its-defrule "ehks"	"돤")
(its-defrule "ehkf"	"돨")
(its-defrule "eho"	"돼")
(its-defrule "ehoT"	"됐")
(its-defrule "ehl"	"되")
(its-defrule "ehls"	"된")
(its-defrule "ehlf"	"될")
(its-defrule "ehla"	"됨")
(its-defrule "ehlq"	"됩")
(its-defrule "ehlt"	"됫")
(its-defrule "ey"	"됴")
(its-defrule "en"	"두")
(its-defrule "enr"	"둑")
(its-defrule "ens"	"둔")
(its-defrule "enf"	"둘")
(its-defrule "ena"	"둠")
(its-defrule "enq"	"둡")
(its-defrule "ent"	"둣")
(its-defrule "end"	"둥")
(its-defrule "enj"	"둬")
(its-defrule "enjT"	"뒀")
(its-defrule "enp"	"뒈")
(its-defrule "enpd"	"뒝")
(its-defrule "enl"	"뒤")
(its-defrule "enls"	"뒨")
(its-defrule "enlf"	"뒬")
(its-defrule "enlq"	"뒵")
(its-defrule "enlt"	"뒷")
(its-defrule "enld"	"뒹")
(its-defrule "eb"	"듀")
(its-defrule "ebs"	"듄")
(its-defrule "ebf"	"듈")
(its-defrule "eba"	"듐")
(its-defrule "ebd"	"듕")
(its-defrule "em"	"드")
(its-defrule "emr"	"득")
(its-defrule "ems"	"든")
(its-defrule "eme"	"듣")
(its-defrule "emf"	"들")
(its-defrule "emfa"	"듦")
(its-defrule "ema"	"듬")
(its-defrule "emq"	"듭")
(its-defrule "emt"	"듯")
(its-defrule "emd"	"등")
(its-defrule "eml"	"듸")
(its-defrule "el"	"디")
(its-defrule "elr"	"딕")
(its-defrule "els"	"딘")
(its-defrule "ele"	"딛")
(its-defrule "elf"	"딜")
(its-defrule "ela"	"딤")
(its-defrule "elq"	"딥")
(its-defrule "elt"	"딧")
(its-defrule "elT"	"딨")
(its-defrule "eld"	"딩")
(its-defrule "elw"	"딪")
(its-defrule "Ek"	"따")
(its-defrule "Ekr"	"딱")
(its-defrule "Eks"	"딴")
(its-defrule "Ekf"	"딸")
(its-defrule "Eka"	"땀")
(its-defrule "Ekq"	"땁")
(its-defrule "Ekt"	"땃")
(its-defrule "EkT"	"땄")
(its-defrule "Ekd"	"땅")
(its-defrule "Ekg"	"땋")
(its-defrule "Eo"	"때")
(its-defrule "Eor"	"땍")
(its-defrule "Eos"	"땐")
(its-defrule "Eof"	"땔")
(its-defrule "Eoa"	"땜")
(its-defrule "Eoq"	"땝")
(its-defrule "Eot"	"땟")
(its-defrule "EoT"	"땠")
(its-defrule "Eod"	"땡")
(its-defrule "Ej"	"떠")
(its-defrule "Ejr"	"떡")
(its-defrule "Ejs"	"떤")
(its-defrule "Ejf"	"떨")
(its-defrule "Ejfa"	"떪")
(its-defrule "Ejfq"	"떫")
(its-defrule "Eja"	"떰")
(its-defrule "Ejq"	"떱")
(its-defrule "Ejt"	"떳")
(its-defrule "EjT"	"떴")
(its-defrule "Ejd"	"떵")
(its-defrule "Ejg"	"떻")
(its-defrule "Ep"	"떼")
(its-defrule "Epr"	"떽")
(its-defrule "Eps"	"뗀")
(its-defrule "Epf"	"뗄")
(its-defrule "Epa"	"뗌")
(its-defrule "Epq"	"뗍")
(its-defrule "Ept"	"뗏")
(its-defrule "EpT"	"뗐")
(its-defrule "Epd"	"뗑")
(its-defrule "Eu"	"뗘")
(its-defrule "EuT"	"뗬")
(its-defrule "Eh"	"또")
(its-defrule "Ehr"	"똑")
(its-defrule "Ehs"	"똔")
(its-defrule "Ehf"	"똘")
(its-defrule "Ehd"	"똥")
(its-defrule "Ehk"	"똬")
(its-defrule "Ehkf"	"똴")
(its-defrule "Eho"	"뙈")
(its-defrule "Ehl"	"뙤")
(its-defrule "Ehls"	"뙨")
(its-defrule "En"	"뚜")
(its-defrule "Enr"	"뚝")
(its-defrule "Ens"	"뚠")
(its-defrule "Enf"	"뚤")
(its-defrule "Enfg"	"뚫")
(its-defrule "Ena"	"뚬")
(its-defrule "End"	"뚱")
(its-defrule "Enp"	"뛔")
(its-defrule "Enl"	"뛰")
(its-defrule "Enls"	"뛴")
(its-defrule "Enlsf"	"뛸")
(its-defrule "Enlsa"	"뜀")
(its-defrule "Enlsq"	"뜁")
(its-defrule "Enlsd"	"뜅")
(its-defrule "Em"	"뜨")
(its-defrule "Emr"	"뜩")
(its-defrule "Ems"	"뜬")
(its-defrule "Eme"	"뜯")
(its-defrule "Emf"	"뜰")
(its-defrule "Ema"	"뜸")
(its-defrule "Emq"	"뜹")
(its-defrule "Emt"	"뜻")
(its-defrule "Eml"	"띄")
(its-defrule "Emls"	"띈")
(its-defrule "Emlf"	"띌")
(its-defrule "Emla"	"띔")
(its-defrule "Emlq"	"띕")
(its-defrule "El"	"띠")
(its-defrule "Els"	"띤")
(its-defrule "Elf"	"띨")
(its-defrule "Ela"	"띰")
(its-defrule "Elq"	"띱")
(its-defrule "Elt"	"띳")
(its-defrule "Eld"	"띵")
(its-defrule "fk"	"라")
(its-defrule "fkr"	"락")
(its-defrule "fks"	"란")
(its-defrule "fkf"	"랄")
(its-defrule "fka"	"람")
(its-defrule "fkq"	"랍")
(its-defrule "fkt"	"랏")
(its-defrule "fkT"	"랐")
(its-defrule "fkd"	"랑")
(its-defrule "fkw"	"랒")
(its-defrule "fkv"	"랖")
(its-defrule "fkg"	"랗")
(its-defrule "fo"	"래")
(its-defrule "for"	"랙")
(its-defrule "fos"	"랜")
(its-defrule "fof"	"랠")
(its-defrule "foa"	"램")
(its-defrule "foq"	"랩")
(its-defrule "fot"	"랫")
(its-defrule "foT"	"랬")
(its-defrule "fod"	"랭")
(its-defrule "fi"	"랴")
(its-defrule "fir"	"략")
(its-defrule "fis"	"랸")
(its-defrule "fit"	"럇")
(its-defrule "fid"	"량")
(its-defrule "fj"	"러")
(its-defrule "fjr"	"럭")
(its-defrule "fjs"	"런")
(its-defrule "fjf"	"럴")
(its-defrule "fja"	"럼")
(its-defrule "fjq"	"럽")
(its-defrule "fjt"	"럿")
(its-defrule "fjT"	"렀")
(its-defrule "fjd"	"렁")
(its-defrule "fjg"	"렇")
(its-defrule "fp"	"레")
(its-defrule "fpr"	"렉")
(its-defrule "fps"	"렌")
(its-defrule "fpf"	"렐")
(its-defrule "fpa"	"렘")
(its-defrule "fpq"	"렙")
(its-defrule "fpt"	"렛")
(its-defrule "fpd"	"렝")
(its-defrule "fu"	"려")
(its-defrule "fur"	"력")
(its-defrule "fus"	"련")
(its-defrule "fuf"	"렬")
(its-defrule "fua"	"렴")
(its-defrule "fuq"	"렵")
(its-defrule "fut"	"렷")
(its-defrule "fuT"	"렸")
(its-defrule "fud"	"령")
(its-defrule "fP"	"례")
(its-defrule "fPs"	"롄")
(its-defrule "fPq"	"롑")
(its-defrule "fPt"	"롓")
(its-defrule "fh"	"로")
(its-defrule "fhr"	"록")
(its-defrule "fhs"	"론")
(its-defrule "fhf"	"롤")
(its-defrule "fha"	"롬")
(its-defrule "fhq"	"롭")
(its-defrule "fht"	"롯")
(its-defrule "fhd"	"롱")
(its-defrule "fhk"	"롸")
(its-defrule "fhks"	"롼")
(its-defrule "fhkd"	"뢍")
(its-defrule "fhoT"	"뢨")
(its-defrule "fhl"	"뢰")
(its-defrule "fhls"	"뢴")
(its-defrule "fhlf"	"뢸")
(its-defrule "fhla"	"룀")
(its-defrule "fhlq"	"룁")
(its-defrule "fhlt"	"룃")
(its-defrule "fhld"	"룅")
(its-defrule "fy"	"료")
(its-defrule "fys"	"룐")
(its-defrule "fyf"	"룔")
(its-defrule "fyq"	"룝")
(its-defrule "fyt"	"룟")
(its-defrule "fyd"	"룡")
(its-defrule "fn"	"루")
(its-defrule "fnr"	"룩")
(its-defrule "fns"	"룬")
(its-defrule "fnf"	"룰")
(its-defrule "fna"	"룸")
(its-defrule "fnq"	"룹")
(its-defrule "fnt"	"룻")
(its-defrule "fnd"	"룽")
(its-defrule "fnj"	"뤄")
(its-defrule "fnjT"	"뤘")
(its-defrule "fnp"	"뤠")
(its-defrule "fnl"	"뤼")
(its-defrule "fnlr"	"뤽")
(its-defrule "fnls"	"륀")
(its-defrule "fnlf"	"륄")
(its-defrule "fnla"	"륌")
(its-defrule "fnlt"	"륏")
(its-defrule "fnld"	"륑")
(its-defrule "fb"	"류")
(its-defrule "fbr"	"륙")
(its-defrule "fbs"	"륜")
(its-defrule "fbf"	"률")
(its-defrule "fba"	"륨")
(its-defrule "fbq"	"륩")
(its-defrule "fbt"	"륫")
(its-defrule "fbd"	"륭")
(its-defrule "fm"	"르")
(its-defrule "fmr"	"륵")
(its-defrule "fms"	"른")
(its-defrule "fmf"	"를")
(its-defrule "fma"	"름")
(its-defrule "fmq"	"릅")
(its-defrule "fmt"	"릇")
(its-defrule "fmd"	"릉")
(its-defrule "fmw"	"릊")
(its-defrule "fmx"	"릍")
(its-defrule "fmv"	"릎")
(its-defrule "fl"	"리")
(its-defrule "flr"	"릭")
(its-defrule "fls"	"린")
(its-defrule "flf"	"릴")
(its-defrule "fla"	"림")
(its-defrule "flq"	"립")
(its-defrule "flt"	"릿")
(its-defrule "fld"	"링")
(its-defrule "ak"	"마")
(its-defrule "akr"	"막")
(its-defrule "aks"	"만")
(its-defrule "aksg"	"많")
(its-defrule "ake"	"맏")
(its-defrule "akf"	"말")
(its-defrule "akfr"	"맑")
(its-defrule "akfa"	"맒")
(its-defrule "aka"	"맘")
(its-defrule "akq"	"맙")
(its-defrule "akt"	"맛")
(its-defrule "akd"	"망")
(its-defrule "akw"	"맞")
(its-defrule "akx"	"맡")
(its-defrule "akg"	"맣")
(its-defrule "ao"	"매")
(its-defrule "aor"	"맥")
(its-defrule "aos"	"맨")
(its-defrule "aof"	"맬")
(its-defrule "aoa"	"맴")
(its-defrule "aoq"	"맵")
(its-defrule "aot"	"맷")
(its-defrule "aoT"	"맸")
(its-defrule "aod"	"맹")
(its-defrule "aow"	"맺")
(its-defrule "ai"	"먀")
(its-defrule "air"	"먁")
(its-defrule "aif"	"먈")
(its-defrule "aid"	"먕")
(its-defrule "aj"	"머")
(its-defrule "ajr"	"먹")
(its-defrule "ajs"	"먼")
(its-defrule "ajf"	"멀")
(its-defrule "ajfa"	"멂")
(its-defrule "aja"	"멈")
(its-defrule "ajq"	"멉")
(its-defrule "ajt"	"멋")
(its-defrule "ajd"	"멍")
(its-defrule "ajw"	"멎")
(its-defrule "ajg"	"멓")
(its-defrule "ap"	"메")
(its-defrule "apr"	"멕")
(its-defrule "aps"	"멘")
(its-defrule "apf"	"멜")
(its-defrule "apa"	"멤")
(its-defrule "apq"	"멥")
(its-defrule "apt"	"멧")
(its-defrule "apT"	"멨")
(its-defrule "apd"	"멩")
(its-defrule "au"	"며")
(its-defrule "aur"	"멱")
(its-defrule "aus"	"면")
(its-defrule "auf"	"멸")
(its-defrule "aut"	"몃")
(its-defrule "auT"	"몄")
(its-defrule "aud"	"명")
(its-defrule "auc"	"몇")
(its-defrule "aP"	"몌")
(its-defrule "ah"	"모")
(its-defrule "ahr"	"목")
(its-defrule "ahrt"	"몫")
(its-defrule "ahs"	"몬")
(its-defrule "ahf"	"몰")
(its-defrule "ahfa"	"몲")
(its-defrule "aha"	"몸")
(its-defrule "ahq"	"몹")
(its-defrule "aht"	"못")
(its-defrule "ahd"	"몽")
(its-defrule "ahk"	"뫄")
(its-defrule "ahks"	"뫈")
(its-defrule "ahkT"	"뫘")
(its-defrule "ahkd"	"뫙")
(its-defrule "ahl"	"뫼")

(its-defrule "ahls"	"묀")
(its-defrule "ahlf"	"묄")
(its-defrule "ahlq"	"묍")
(its-defrule "ahlt"	"묏")
(its-defrule "ahld"	"묑")
(its-defrule "ay"	"묘")
(its-defrule "ays"	"묜")
(its-defrule "ayf"	"묠")
(its-defrule "ayq"	"묩")
(its-defrule "ayt"	"묫")
(its-defrule "an"	"무")
(its-defrule "anr"	"묵")
(its-defrule "anR"	"묶")
(its-defrule "ans"	"문")
(its-defrule "ane"	"묻")
(its-defrule "anf"	"물")
(its-defrule "anfr"	"묽")
(its-defrule "anfa"	"묾")
(its-defrule "ana"	"뭄")
(its-defrule "anq"	"뭅")
(its-defrule "ant"	"뭇")
(its-defrule "and"	"뭉")
(its-defrule "anx"	"뭍")
(its-defrule "ang"	"뭏")
(its-defrule "anj"	"뭐")
(its-defrule "anjs"	"뭔")
(its-defrule "anjf"	"뭘")
(its-defrule "anjq"	"뭡")
(its-defrule "anjt"	"뭣")
(its-defrule "anp"	"뭬")
(its-defrule "anl"	"뮈")
(its-defrule "anls"	"뮌")
(its-defrule "anlf"	"뮐")
(its-defrule "ab"	"뮤")

(its-defrule "abs"	"뮨")
(its-defrule "abf"	"뮬")
(its-defrule "aba"	"뮴")
(its-defrule "abt"	"뮷")
(its-defrule "am"	"므")
(its-defrule "ams"	"믄")
(its-defrule "amf"	"믈")
(its-defrule "ama"	"믐")
(its-defrule "amt"	"믓")
(its-defrule "al"	"미")
(its-defrule "alr"	"믹")
(its-defrule "als"	"민")
(its-defrule "ale"	"믿")
(its-defrule "alf"	"밀")
(its-defrule "alfa"	"밂")
(its-defrule "ala"	"밈")
(its-defrule "alq"	"밉")
(its-defrule "alt"	"밋")
(its-defrule "alT"	"밌")
(its-defrule "ald"	"밍")
(its-defrule "alc"	"및")
(its-defrule "alx"	"밑")
(its-defrule "qk"	"바")
(its-defrule "qkr"	"박")
(its-defrule "qkR"	"밖")
(its-defrule "qkrt"	"밗")
(its-defrule "qks"	"반")
(its-defrule "qke"	"받")
(its-defrule "qkf"	"발")
(its-defrule "qkfr"	"밝")
(its-defrule "qkfa"	"밞")
(its-defrule "qkfq"	"밟")
(its-defrule "qka"	"밤")
(its-defrule "qkq"	"밥")
(its-defrule "qkt"	"밧")
(its-defrule "qkd"	"방")
(its-defrule "qkx"	"밭")
(its-defrule "qo"	"배")
(its-defrule "qor"	"백")
(its-defrule "qos"	"밴")
(its-defrule "qof"	"밸")
(its-defrule "qoa"	"뱀")
(its-defrule "qoq"	"뱁")
(its-defrule "qot"	"뱃")
(its-defrule "qoT"	"뱄")
(its-defrule "qod"	"뱅")
(its-defrule "qox"	"뱉")
(its-defrule "qi"	"뱌")
(its-defrule "qir"	"뱍")
(its-defrule "qis"	"뱐")
(its-defrule "qiq"	"뱝")
(its-defrule "qj"	"버")
(its-defrule "qjr"	"벅")
(its-defrule "qjs"	"번")
(its-defrule "qje"	"벋")
(its-defrule "qjf"	"벌")
(its-defrule "qjfa"	"벎")
(its-defrule "qja"	"범")
(its-defrule "qjq"	"법")
(its-defrule "qjt"	"벗")
(its-defrule "qjd"	"벙")
(its-defrule "qjw"	"벚")
(its-defrule "qp"	"베")
(its-defrule "qpr"	"벡")
(its-defrule "qps"	"벤")
(its-defrule "qpe"	"벧")
(its-defrule "qpf"	"벨")
(its-defrule "qpa"	"벰")
(its-defrule "qpq"	"벱")
(its-defrule "qpt"	"벳")
(its-defrule "qpT"	"벴")
(its-defrule "qpd"	"벵")
(its-defrule "qu"	"벼")
(its-defrule "qur"	"벽")
(its-defrule "qus"	"변")
(its-defrule "quf"	"별")
(its-defrule "quq"	"볍")
(its-defrule "qut"	"볏")
(its-defrule "quT"	"볐")
(its-defrule "qud"	"병")
(its-defrule "qux"	"볕")
(its-defrule "qP"	"볘")
(its-defrule "qPs"	"볜")
(its-defrule "qh"	"보")
(its-defrule "qhr"	"복")
(its-defrule "qhR"	"볶")
(its-defrule "qhs"	"본")
(its-defrule "qhf"	"볼")
(its-defrule "qha"	"봄")
(its-defrule "qhq"	"봅")
(its-defrule "qht"	"봇")
(its-defrule "qhd"	"봉")
(its-defrule "qhk"	"봐")
(its-defrule "qhks"	"봔")
(its-defrule "qhkT"	"봤")
(its-defrule "qho"	"봬")
(its-defrule "qhoT"	"뵀")
(its-defrule "qhl"	"뵈")
(its-defrule "qhlr"	"뵉")
(its-defrule "qhls"	"뵌")
(its-defrule "qhlf"	"뵐")
(its-defrule "qhla"	"뵘")
(its-defrule "qhlq"	"뵙")
(its-defrule "qy"	"뵤")
(its-defrule "qys"	"뵨")
(its-defrule "qn"	"부")
(its-defrule "qnr"	"북")
(its-defrule "qns"	"분")
(its-defrule "qne"	"붇")
(its-defrule "qnf"	"불")
(its-defrule "qnfr"	"붉")
(its-defrule "qnfa"	"붊")
(its-defrule "qna"	"붐")
(its-defrule "qnq"	"붑")
(its-defrule "qnt"	"붓")
(its-defrule "qnd"	"붕")
(its-defrule "qnx"	"붙")
(its-defrule "qnv"	"붚")
(its-defrule "qnj"	"붜")
(its-defrule "qnjf"	"붤")
(its-defrule "qnjT"	"붰")
(its-defrule "qnp"	"붸")
(its-defrule "qnl"	"뷔")
(its-defrule "qnlr"	"뷕")
(its-defrule "qnls"	"뷘")
(its-defrule "qnlf"	"뷜")
(its-defrule "qnld"	"뷩")
(its-defrule "qb"	"뷰")
(its-defrule "qbs"	"뷴")
(its-defrule "qbf"	"뷸")
(its-defrule "qba"	"븀")
(its-defrule "qbt"	"븃")
(its-defrule "qbd"	"븅")
(its-defrule "qm"	"브")
(its-defrule "qmr"	"븍")
(its-defrule "qms"	"븐")
(its-defrule "qmf"	"블")
(its-defrule "qma"	"븜")
(its-defrule "qmq"	"븝")
(its-defrule "qmt"	"븟")
(its-defrule "ql"	"비")
(its-defrule "qlr"	"빅")
(its-defrule "qls"	"빈")
(its-defrule "qlf"	"빌")
(its-defrule "qlfa"	"빎")
(its-defrule "qla"	"빔")
(its-defrule "qlq"	"빕")
(its-defrule "qlt"	"빗")
(its-defrule "qld"	"빙")
(its-defrule "qlw"	"빚")
(its-defrule "qlc"	"빛")
(its-defrule "Qk"	"빠")
(its-defrule "Qkr"	"빡")
(its-defrule "Qks"	"빤")
(its-defrule "Qkf"	"빨")
(its-defrule "Qkfa"	"빪")
(its-defrule "Qka"	"빰")
(its-defrule "Qkq"	"빱")
(its-defrule "Qkt"	"빳")
(its-defrule "QkT"	"빴")
(its-defrule "Qkd"	"빵")
(its-defrule "Qkg"	"빻")
(its-defrule "Qo"	"빼")
(its-defrule "Qor"	"빽")
(its-defrule "Qos"	"뺀")
(its-defrule "Qof"	"뺄")
(its-defrule "Qoa"	"뺌")
(its-defrule "Qoq"	"뺍")
(its-defrule "Qot"	"뺏")
(its-defrule "QoT"	"뺐")
(its-defrule "Qod"	"뺑")
(its-defrule "Qi"	"뺘")
(its-defrule "Qir"	"뺙")
(its-defrule "Qia"	"뺨")
(its-defrule "Qj"	"뻐")
(its-defrule "Qjr"	"뻑")
(its-defrule "Qjs"	"뻔")
(its-defrule "Qje"	"뻗")
(its-defrule "Qjf"	"뻘")
(its-defrule "Qja"	"뻠")
(its-defrule "Qjt"	"뻣")
(its-defrule "QjT"	"뻤")
(its-defrule "Qjd"	"뻥")
(its-defrule "Qp"	"뻬")
(its-defrule "Qpd"	"뼁")
(its-defrule "Qu"	"뼈")
(its-defrule "Qur"	"뼉")
(its-defrule "Qua"	"뼘")
(its-defrule "Quq"	"뼙")
(its-defrule "Qut"	"뼛")
(its-defrule "QuT"	"뼜")
(its-defrule "Qud"	"뼝")
(its-defrule "Qh"	"뽀")
(its-defrule "Qhr"	"뽁")
(its-defrule "Qhs"	"뽄")
(its-defrule "Qhf"	"뽈")
(its-defrule "Qha"	"뽐")
(its-defrule "Qhq"	"뽑")
(its-defrule "Qhd"	"뽕")
(its-defrule "Qhl"	"뾔")
(its-defrule "Qy"	"뾰")
(its-defrule "Qyd"	"뿅")
(its-defrule "Qn"	"뿌")
(its-defrule "Qnr"	"뿍")
(its-defrule "Qns"	"뿐")
(its-defrule "Qnf"	"뿔")
(its-defrule "Qna"	"뿜")
(its-defrule "Qnt"	"뿟")
(its-defrule "Qnd"	"뿡")
(its-defrule "Qb"	"쀼")
(its-defrule "Qbd"	"쁑")
(its-defrule "Qm"	"쁘")
(its-defrule "Qms"	"쁜")
(its-defrule "Qmf"	"쁠")
(its-defrule "Qma"	"쁨")
(its-defrule "Qmq"	"쁩")
(its-defrule "Ql"	"삐")
(its-defrule "Qlr"	"삑")
(its-defrule "Qls"	"삔")
(its-defrule "Qlf"	"삘")
(its-defrule "Qla"	"삠")
(its-defrule "Qlq"	"삡")
(its-defrule "Qlt"	"삣")
(its-defrule "Qld"	"삥")
(its-defrule "tk"	"사")
(its-defrule "tkr"	"삭")
(its-defrule "tkrt"	"삯")
(its-defrule "tks"	"산")
(its-defrule "tke"	"삳")
(its-defrule "tkf"	"살")
(its-defrule "tkfr"	"삵")
(its-defrule "tkfa"	"삶")
(its-defrule "tka"	"삼")
(its-defrule "tkq"	"삽")
(its-defrule "tkt"	"삿")
(its-defrule "tkT"	"샀")
(its-defrule "tkd"	"상")
(its-defrule "tkx"	"샅")
(its-defrule "to"	"새")
(its-defrule "tor"	"색")
(its-defrule "tos"	"샌")
(its-defrule "tof"	"샐")
(its-defrule "toa"	"샘")
(its-defrule "toq"	"샙")
(its-defrule "tot"	"샛")
(its-defrule "toT"	"샜")
(its-defrule "tod"	"생")
(its-defrule "ti"	"샤")
(its-defrule "tir"	"샥")
(its-defrule "tis"	"샨")
(its-defrule "tif"	"샬")
(its-defrule "tia"	"샴")
(its-defrule "tiq"	"샵")
(its-defrule "tit"	"샷")
(its-defrule "tid"	"샹")
(its-defrule "tO"	"섀")
(its-defrule "tOs"	"섄")
(its-defrule "tOf"	"섈")
(its-defrule "tOa"	"섐")
(its-defrule "tOd"	"섕")
(its-defrule "tj"	"서")
(its-defrule "tjr"	"석")
(its-defrule "tjR"	"섞")
(its-defrule "tjrt"	"섟")
(its-defrule "tjs"	"선")
(its-defrule "tje"	"섣")
(its-defrule "tjf"	"설")
(its-defrule "tjfa"	"섦")
(its-defrule "tjfq"	"섧")
(its-defrule "tja"	"섬")
(its-defrule "tjq"	"섭")
(its-defrule "tjt"	"섯")
(its-defrule "tjT"	"섰")
(its-defrule "tjd"	"성")
(its-defrule "tjv"	"섶")
(its-defrule "tp"	"세")
(its-defrule "tpr"	"섹")
(its-defrule "tps"	"센")
(its-defrule "tpf"	"셀")
(its-defrule "tpa"	"셈")
(its-defrule "tpq"	"셉")
(its-defrule "tpt"	"셋")
(its-defrule "tpT"	"셌")
(its-defrule "tpd"	"셍")
(its-defrule "tu"	"셔")
(its-defrule "tur"	"셕")
(its-defrule "tus"	"션")
(its-defrule "tuf"	"셜")
(its-defrule "tua"	"셤")
(its-defrule "tuq"	"셥")
(its-defrule "tut"	"셧")
(its-defrule "tuT"	"셨")
(its-defrule "tud"	"셩")
(its-defrule "tP"	"셰")
(its-defrule "tPs"	"셴")
(its-defrule "tPf"	"셸")
(its-defrule "tPd"	"솅")
(its-defrule "th"	"소")
(its-defrule "thr"	"속")
(its-defrule "thR"	"솎")
(its-defrule "ths"	"손")
(its-defrule "thf"	"솔")
(its-defrule "thfa"	"솖")
(its-defrule "tha"	"솜")
(its-defrule "thq"	"솝")
(its-defrule "tht"	"솟")
(its-defrule "thd"	"송")
(its-defrule "thx"	"솥")
(its-defrule "thk"	"솨")
(its-defrule "thkr"	"솩")
(its-defrule "thks"	"솬")
(its-defrule "thkf"	"솰")
(its-defrule "thkd"	"솽")
(its-defrule "tho"	"쇄")
(its-defrule "thos"	"쇈")
(its-defrule "thof"	"쇌")
(its-defrule "thoa"	"쇔")
(its-defrule "thot"	"쇗")
(its-defrule "thoT"	"쇘")
(its-defrule "thl"	"쇠")
(its-defrule "thls"	"쇤")
(its-defrule "thlf"	"쇨")
(its-defrule "thla"	"쇰")
(its-defrule "thlq"	"쇱")
(its-defrule "thlt"	"쇳")
(its-defrule "ty"	"쇼")
(its-defrule "tyr"	"쇽")
(its-defrule "tys"	"숀")
(its-defrule "tyf"	"숄")
(its-defrule "tya"	"숌")
(its-defrule "tyq"	"숍")
(its-defrule "tyt"	"숏")
(its-defrule "tyd"	"숑")
(its-defrule "tn"	"수")
(its-defrule "tnr"	"숙")
(its-defrule "tns"	"순")
(its-defrule "tne"	"숟")
(its-defrule "tnf"	"술")
(its-defrule "tna"	"숨")
(its-defrule "tnq"	"숩")
(its-defrule "tnt"	"숫")
(its-defrule "tnd"	"숭")
(its-defrule "tnc"	"숯")
(its-defrule "tnx"	"숱")
(its-defrule "tnv"	"숲")
(its-defrule "tnj"	"숴")
(its-defrule "tnjT"	"쉈")
(its-defrule "tnp"	"쉐")
(its-defrule "tnpr"	"쉑")
(its-defrule "tnps"	"쉔")
(its-defrule "tnpf"	"쉘")
(its-defrule "tnpa"	"쉠")
(its-defrule "tnpd"	"쉥")
(its-defrule "tnl"	"쉬")
(its-defrule "tnlr"	"쉭")
(its-defrule "tnls"	"쉰")
(its-defrule "tnlf"	"쉴")
(its-defrule "tnla"	"쉼")
(its-defrule "tnlq"	"쉽")
(its-defrule "tnlt"	"쉿")
(its-defrule "tnld"	"슁")
(its-defrule "tb"	"슈")
(its-defrule "tbr"	"슉")
(its-defrule "tbf"	"슐")
(its-defrule "tba"	"슘")
(its-defrule "tbt"	"슛")
(its-defrule "tbd"	"슝")
(its-defrule "tm"	"스")
(its-defrule "tmr"	"슥")
(its-defrule "tms"	"슨")
(its-defrule "tmf"	"슬")
(its-defrule "tmfr"	"슭")
(its-defrule "tma"	"슴")
(its-defrule "tmq"	"습")
(its-defrule "tmt"	"슷")
(its-defrule "tmd"	"승")
(its-defrule "tl"	"시")
(its-defrule "tlr"	"식")
(its-defrule "tls"	"신")
(its-defrule "tle"	"싣")
(its-defrule "tlf"	"실")
(its-defrule "tlfg"	"싫")
(its-defrule "tla"	"심")
(its-defrule "tlq"	"십")
(its-defrule "tlt"	"싯")
(its-defrule "tld"	"싱")
(its-defrule "tlv"	"싶")
(its-defrule "Tk"	"싸")
(its-defrule "Tkr"	"싹")
(its-defrule "Tkrt"	"싻")
(its-defrule "Tks"	"싼")
(its-defrule "Tkf"	"쌀")
(its-defrule "Tka"	"쌈")
(its-defrule "Tkq"	"쌉")
(its-defrule "TkT"	"쌌")
(its-defrule "Tkd"	"쌍")
(its-defrule "Tkg"	"쌓")
(its-defrule "To"	"쌔")
(its-defrule "Tor"	"쌕")
(its-defrule "Tos"	"쌘")
(its-defrule "Tof"	"쌜")
(its-defrule "Toa"	"쌤")
(its-defrule "Toq"	"쌥")
(its-defrule "ToT"	"쌨")
(its-defrule "Tod"	"쌩")
(its-defrule "Tid"	"썅")
(its-defrule "Tj"	"써")
(its-defrule "Tjr"	"썩")
(its-defrule "Tjs"	"썬")
(its-defrule "Tjf"	"썰")
(its-defrule "Tjfa"	"썲")
(its-defrule "Tja"	"썸")
(its-defrule "Tjq"	"썹")
(its-defrule "TjT"	"썼")
(its-defrule "Tjd"	"썽")
(its-defrule "Tp"	"쎄")
(its-defrule "Tps"	"쎈")
(its-defrule "Tpf"	"쎌")
(its-defrule "TPs"	"쏀")
(its-defrule "Th"	"쏘")
(its-defrule "Thr"	"쏙")
(its-defrule "Ths"	"쏜")
(its-defrule "The"	"쏟")
(its-defrule "Thf"	"쏠")
(its-defrule "Thfa"	"쏢")
(its-defrule "Tha"	"쏨")
(its-defrule "Thq"	"쏩")
(its-defrule "Thd"	"쏭")
(its-defrule "Thk"	"쏴")
(its-defrule "Thkr"	"쏵")
(its-defrule "Thks"	"쏸")
(its-defrule "ThkT"	"쐈")
(its-defrule "Tho"	"쐐")
(its-defrule "ThoT"	"쐤")
(its-defrule "Thl"	"쐬")
(its-defrule "Thls"	"쐰")
(its-defrule "Thlf"	"쐴")
(its-defrule "Thla"	"쐼")
(its-defrule "Thlq"	"쐽")
(its-defrule "Ty"	"쑈")
(its-defrule "Tn"	"쑤")
(its-defrule "Tnr"	"쑥")
(its-defrule "Tns"	"쑨")
(its-defrule "Tnf"	"쑬")
(its-defrule "Tna"	"쑴")
(its-defrule "Tnq"	"쑵")
(its-defrule "Tnd"	"쑹")
(its-defrule "Tnj"	"쒀")
(its-defrule "TnjT"	"쒔")
(its-defrule "Tnp"	"쒜")
(its-defrule "Tnl"	"쒸")
(its-defrule "Tnls"	"쒼")
(its-defrule "Tbd"	"쓩")
(its-defrule "Tm"	"쓰")
(its-defrule "Tmr"	"쓱")
(its-defrule "Tms"	"쓴")
(its-defrule "Tmf"	"쓸")
(its-defrule "Tmfa"	"쓺")
(its-defrule "Tmfg"	"쓿")
(its-defrule "Tma"	"씀")
(its-defrule "Tmq"	"씁")
(its-defrule "Tml"	"씌")
(its-defrule "Tmls"	"씐")
(its-defrule "Tmlf"	"씔")
(its-defrule "Tmla"	"씜")
(its-defrule "Tl"	"씨")
(its-defrule "Tlr"	"씩")
(its-defrule "Tls"	"씬")
(its-defrule "Tlf"	"씰")
(its-defrule "Tla"	"씸")
(its-defrule "Tlq"	"씹")
(its-defrule "Tlt"	"씻")
(its-defrule "Tld"	"씽")
(its-defrule "dk"	"아")
(its-defrule "dkr"	"악")
(its-defrule "dks"	"안")
(its-defrule "dksw"	"앉")
(its-defrule "dksg"	"않")
(its-defrule "dkf"	"알")
(its-defrule "dkfr"	"앍")
(its-defrule "dkfa"	"앎")
(its-defrule "dkfg"	"앓")
(its-defrule "dka"	"암")
(its-defrule "dkq"	"압")
(its-defrule "dkt"	"앗")
(its-defrule "dkT"	"았")
(its-defrule "dkd"	"앙")
(its-defrule "dkx"	"앝")
(its-defrule "dkv"	"앞")
(its-defrule "do"	"애")
(its-defrule "dor"	"액")
(its-defrule "dos"	"앤")
(its-defrule "dof"	"앨")
(its-defrule "doa"	"앰")
(its-defrule "doq"	"앱")
(its-defrule "dot"	"앳")
(its-defrule "doT"	"앴")
(its-defrule "dod"	"앵")
(its-defrule "di"	"야")
(its-defrule "dir"	"약")
(its-defrule "dis"	"얀")
(its-defrule "dif"	"얄")
(its-defrule "difq"	"얇")
(its-defrule "dia"	"얌")
(its-defrule "diq"	"얍")
(its-defrule "dit"	"얏")
(its-defrule "did"	"양")
(its-defrule "dix"	"얕")
(its-defrule "dig"	"얗")
(its-defrule "dO"	"얘")
(its-defrule "dOs"	"얜")
(its-defrule "dOf"	"얠")
(its-defrule "dOq"	"얩")
(its-defrule "dj"	"어")
(its-defrule "djr"	"억")
(its-defrule "djs"	"언")
(its-defrule "djsw"	"얹")
(its-defrule "dje"	"얻")
(its-defrule "djf"	"얼")
(its-defrule "djfr"	"얽")
(its-defrule "djfa"	"얾")
(its-defrule "dja"	"엄")
(its-defrule "djq"	"업")
(its-defrule "djqt"	"없")
(its-defrule "djt"	"엇")
(its-defrule "djT"	"었")
(its-defrule "djd"	"엉")
(its-defrule "djw"	"엊")
(its-defrule "djz"	"엌")
(its-defrule "djv"	"엎")
(its-defrule "dp"	"에")
(its-defrule "dpr"	"엑")
(its-defrule "dps"	"엔")
(its-defrule "dpf"	"엘")
(its-defrule "dpa"	"엠")
(its-defrule "dpq"	"엡")
(its-defrule "dpt"	"엣")
(its-defrule "dpd"	"엥")
(its-defrule "du"	"여")
(its-defrule "dur"	"역")
(its-defrule "duR"	"엮")
(its-defrule "dus"	"연")
(its-defrule "duf"	"열")
(its-defrule "dufa"	"엶")
(its-defrule "dufq"	"엷")
(its-defrule "dua"	"염")
(its-defrule "duq"	"엽")
(its-defrule "duqt"	"엾")
(its-defrule "dut"	"엿")
(its-defrule "duT"	"였")
(its-defrule "dud"	"영")
(its-defrule "dux"	"옅")
(its-defrule "duv"	"옆")
(its-defrule "dug"	"옇")
(its-defrule "dP"	"예")
(its-defrule "dPs"	"옌")
(its-defrule "dPf"	"옐")
(its-defrule "dPa"	"옘")
(its-defrule "dPq"	"옙")
(its-defrule "dPt"	"옛")
(its-defrule "dPT"	"옜")
(its-defrule "dh"	"오")
(its-defrule "dhr"	"옥")
(its-defrule "dhs"	"온")
(its-defrule "dhf"	"올")
(its-defrule "dhfr"	"옭")
(its-defrule "dhfa"	"옮")
(its-defrule "dhft"	"옰")
(its-defrule "dhfg"	"옳")
(its-defrule "dha"	"옴")
(its-defrule "dhq"	"옵")
(its-defrule "dht"	"옷")
(its-defrule "dhd"	"옹")
(its-defrule "dhc"	"옻")
(its-defrule "dhk"	"와")
(its-defrule "dhkr"	"왁")
(its-defrule "dhks"	"완")
(its-defrule "dhkf"	"왈")
(its-defrule "dhka"	"왐")
(its-defrule "dhkq"	"왑")
(its-defrule "dhkt"	"왓")
(its-defrule "dhkT"	"왔")
(its-defrule "dhkd"	"왕")
(its-defrule "dho"	"왜")
(its-defrule "dhor"	"왝")
(its-defrule "dhos"	"왠")
(its-defrule "dhoa"	"왬")
(its-defrule "dhot"	"왯")
(its-defrule "dhod"	"왱")
(its-defrule "dhl"	"외")
(its-defrule "dhlr"	"왹")
(its-defrule "dhls"	"왼")
(its-defrule "dhlf"	"욀")
(its-defrule "dhla"	"욈")
(its-defrule "dhlq"	"욉")
(its-defrule "dhlt"	"욋")
(its-defrule "dhld"	"욍")
(its-defrule "dy"	"요")
(its-defrule "dyr"	"욕")
(its-defrule "dys"	"욘")
(its-defrule "dyf"	"욜")
(its-defrule "dya"	"욤")
(its-defrule "dyq"	"욥")
(its-defrule "dyt"	"욧")
(its-defrule "dyd"	"용")
(its-defrule "dn"	"우")
(its-defrule "dnr"	"욱")
(its-defrule "dns"	"운")
(its-defrule "dnf"	"울")
(its-defrule "dnfr"	"욹")
(its-defrule "dnfa"	"욺")
(its-defrule "dna"	"움")
(its-defrule "dnq"	"웁")
(its-defrule "dnt"	"웃")
(its-defrule "dnd"	"웅")
(its-defrule "dnj"	"워")
(its-defrule "dnjr"	"웍")
(its-defrule "dnjs"	"원")
(its-defrule "dnjf"	"월")
(its-defrule "dnja"	"웜")
(its-defrule "dnjq"	"웝")
(its-defrule "dnjT"	"웠")
(its-defrule "dnjd"	"웡")
(its-defrule "dnp"	"웨")
(its-defrule "dnpr"	"웩")
(its-defrule "dnps"	"웬")
(its-defrule "dnpf"	"웰")
(its-defrule "dnpa"	"웸")
(its-defrule "dnpq"	"웹")
(its-defrule "dnpd"	"웽")
(its-defrule "dnl"	"위")
(its-defrule "dnlr"	"윅")
(its-defrule "dnls"	"윈")
(its-defrule "dnlf"	"윌")
(its-defrule "dnla"	"윔")
(its-defrule "dnlq"	"윕")
(its-defrule "dnlt"	"윗")
(its-defrule "dnld"	"윙")
(its-defrule "db"	"유")
(its-defrule "dbr"	"육")
(its-defrule "dbs"	"윤")
(its-defrule "dbf"	"율")
(its-defrule "dba"	"윰")
(its-defrule "dbq"	"윱")
(its-defrule "dbt"	"윳")
(its-defrule "dbd"	"융")
(its-defrule "dbg"	"윷")
(its-defrule "dm"	"으")
(its-defrule "dmr"	"윽")
(its-defrule "dms"	"은")
(its-defrule "dmf"	"을")
(its-defrule "dmfv"	"읊")
(its-defrule "dma"	"음")
(its-defrule "dmq"	"읍")
(its-defrule "dmt"	"읏")
(its-defrule "dmd"	"응")
(its-defrule "dmw"	"읒")
(its-defrule "dmc"	"읓")
(its-defrule "dmz"	"읔")
(its-defrule "dmx"	"읕")
(its-defrule "dmv"	"읖")
(its-defrule "dmg"	"읗")
(its-defrule "dml"	"의")
(its-defrule "dmls"	"읜")
(its-defrule "dmlf"	"읠")
(its-defrule "dmla"	"읨")
(its-defrule "dmlt"	"읫")
(its-defrule "dl"	"이")
(its-defrule "dlr"	"익")
(its-defrule "dls"	"인")
(its-defrule "dlf"	"일")
(its-defrule "dlfr"	"읽")
(its-defrule "dlfa"	"읾")
(its-defrule "dlfg"	"잃")
(its-defrule "dla"	"임")
(its-defrule "dlq"	"입")
(its-defrule "dlt"	"잇")
(its-defrule "dlT"	"있")
(its-defrule "dld"	"잉")
(its-defrule "dlw"	"잊")
(its-defrule "dlv"	"잎")
(its-defrule "wk"	"자")
(its-defrule "wkr"	"작")
(its-defrule "wks"	"잔")
(its-defrule "wksg"	"잖")
(its-defrule "wke"	"잗")
(its-defrule "wkf"	"잘")
(its-defrule "wkfa"	"잚")
(its-defrule "wka"	"잠")
(its-defrule "wkq"	"잡")
(its-defrule "wkt"	"잣")
(its-defrule "wkT"	"잤")
(its-defrule "wkd"	"장")
(its-defrule "wkw"	"잦")
(its-defrule "O"	"재")
(its-defrule "Or"	"잭")
(its-defrule "Os"	"잰")
(its-defrule "Of"	"잴")
(its-defrule "Oa"	"잼")
(its-defrule "Oq"	"잽")
(its-defrule "Ot"	"잿")
(its-defrule "OT"	"쟀")
(its-defrule "Od"	"쟁")
(its-defrule "wi"	"쟈")
(its-defrule "wir"	"쟉")
(its-defrule "wis"	"쟌")
(its-defrule "wisg"	"쟎")
(its-defrule "wif"	"쟐")
(its-defrule "wia"	"쟘")
(its-defrule "wid"	"쟝")
(its-defrule "wO"	"쟤")
(its-defrule "wOs"	"쟨")
(its-defrule "wOf"	"쟬")
(its-defrule "wj"	"저")
(its-defrule "wjr"	"적")
(its-defrule "wjs"	"전")
(its-defrule "wjf"	"절")
(its-defrule "wjfa"	"젊")
(its-defrule "wja"	"점")
(its-defrule "wjq"	"접")
(its-defrule "wjt"	"젓")
(its-defrule "wjd"	"정")
(its-defrule "wjw"	"젖")
(its-defrule "wp"	"제")
(its-defrule "wpr"	"젝")
(its-defrule "wps"	"젠")
(its-defrule "wpf"	"젤")
(its-defrule "wpa"	"젬")
(its-defrule "wpq"	"젭")
(its-defrule "wpt"	"젯")
(its-defrule "wpd"	"젱")
(its-defrule "wu"	"져")
(its-defrule "wus"	"젼")
(its-defrule "wuf"	"졀")
(its-defrule "wua"	"졈")
(its-defrule "wuq"	"졉")
(its-defrule "wuT"	"졌")
(its-defrule "wud"	"졍")
(its-defrule "wP"	"졔")
(its-defrule "wh"	"조")
(its-defrule "whr"	"족")
(its-defrule "whs"	"존")
(its-defrule "whf"	"졸")
(its-defrule "whfa"	"졺")
(its-defrule "wha"	"좀")
(its-defrule "whq"	"좁")
(its-defrule "wht"	"좃")
(its-defrule "whd"	"종")
(its-defrule "whw"	"좆")
(its-defrule "whc"	"좇")
(its-defrule "whg"	"좋")
(its-defrule "whk"	"좌")
(its-defrule "whkr"	"좍")
(its-defrule "whkf"	"좔")
(its-defrule "whkq"	"좝")
(its-defrule "whkt"	"좟")
(its-defrule "whkd"	"좡")
(its-defrule "who"	"좨")
(its-defrule "whoT"	"좼")
(its-defrule "whod"	"좽")
(its-defrule "whl"	"죄")
(its-defrule "whls"	"죈")
(its-defrule "whlf"	"죌")
(its-defrule "whla"	"죔")
(its-defrule "whlaq"	"죕")
(its-defrule "whlat"	"죗")
(its-defrule "whlad"	"죙")
(its-defrule "wy"	"죠")
(its-defrule "wyr"	"죡")
(its-defrule "wys"	"죤")
(its-defrule "wyd"	"죵")
(its-defrule "wn"	"주")
(its-defrule "wnr"	"죽")
(its-defrule "wns"	"준")
(its-defrule "wnf"	"줄")
(its-defrule "wnfr"	"줅")
(its-defrule "wnfa"	"줆")
(its-defrule "wna"	"줌")
(its-defrule "wnq"	"줍")
(its-defrule "wnt"	"줏")
(its-defrule "wnd"	"중")
(its-defrule "wnj"	"줘")
(its-defrule "wnjT"	"줬")
(its-defrule "wnp"	"줴")
(its-defrule "wnl"	"쥐")
(its-defrule "wnlr"	"쥑")
(its-defrule "wnls"	"쥔")
(its-defrule "wnlf"	"쥘")
(its-defrule "wnla"	"쥠")
(its-defrule "wnlq"	"쥡")
(its-defrule "wnlt"	"쥣")
(its-defrule "wb"	"쥬")
(its-defrule "wbs"	"쥰")
(its-defrule "wbf"	"쥴")
(its-defrule "wba"	"쥼")
(its-defrule "wm"	"즈")
(its-defrule "wmr"	"즉")
(its-defrule "wms"	"즌")
(its-defrule "wmf"	"즐")
(its-defrule "wma"	"즘")
(its-defrule "wmq"	"즙")
(its-defrule "wmt"	"즛")
(its-defrule "wmd"	"증")
(its-defrule "wl"	"지")
(its-defrule "wlr"	"직")
(its-defrule "wls"	"진")
(its-defrule "wle"	"짇")
(its-defrule "wlf"	"질")
(its-defrule "wlfa"	"짊")
(its-defrule "wla"	"짐")
(its-defrule "wlq"	"집")
(its-defrule "wlt"	"짓")
(its-defrule "wld"	"징")
(its-defrule "wlw"	"짖")
(its-defrule "wlx"	"짙")
(its-defrule "wlv"	"짚")
(its-defrule "Wk"	"짜")
(its-defrule "Wkr"	"짝")
(its-defrule "Wks"	"짠")
(its-defrule "Wksg"	"짢")
(its-defrule "Wkf"	"짤")
(its-defrule "Wkfq"	"짧")
(its-defrule "Wka"	"짬")
(its-defrule "Wkq"	"짭")
(its-defrule "Wkt"	"짯")
(its-defrule "WkT"	"짰")
(its-defrule "Wkd"	"짱")
(its-defrule "Wo"	"째")
(its-defrule "Wor"	"짹")
(its-defrule "Wos"	"짼")
(its-defrule "Wof"	"쨀")
(its-defrule "Woa"	"쨈")
(its-defrule "Woq"	"쨉")
(its-defrule "Wot"	"쨋")
(its-defrule "WoT"	"쨌")
(its-defrule "Wod"	"쨍")
(its-defrule "Wi"	"쨔")
(its-defrule "Wis"	"쨘")
(its-defrule "Wid"	"쨩")
(its-defrule "Wj"	"쩌")
(its-defrule "Wjr"	"쩍")
(its-defrule "Wjs"	"쩐")
(its-defrule "Wjf"	"쩔")
(its-defrule "Wja"	"쩜")
(its-defrule "Wjq"	"쩝")
(its-defrule "Wjt"	"쩟")
(its-defrule "WjT"	"쩠")
(its-defrule "Wjd"	"쩡")
(its-defrule "Wp"	"쩨")
(its-defrule "Wpd"	"쩽")
(its-defrule "Wu"	"쪄")
(its-defrule "WuT"	"쪘")
(its-defrule "Wh"	"쪼")
(its-defrule "Whr"	"쪽")
(its-defrule "Whs"	"쫀")
(its-defrule "Whf"	"쫄")
(its-defrule "Wha"	"쫌")
(its-defrule "Whq"	"쫍")
(its-defrule "Wht"	"쫏")
(its-defrule "Whd"	"쫑")
(its-defrule "Whc"	"쫓")
(its-defrule "Whk"	"쫘")
(its-defrule "Whkr"	"쫙")
(its-defrule "Whkf"	"쫠")
(its-defrule "WhkT"	"쫬")
(its-defrule "Who"	"쫴")
(its-defrule "WhoT"	"쬈")
(its-defrule "Whl"	"쬐")
(its-defrule "Whls"	"쬔")
(its-defrule "Whlf"	"쬘")
(its-defrule "Whla"	"쬠")
(its-defrule "Whlq"	"쬡")
(its-defrule "Wyd"	"쭁")
(its-defrule "Wn"	"쭈")
(its-defrule "Wnr"	"쭉")
(its-defrule "Wns"	"쭌")
(its-defrule "Wnf"	"쭐")
(its-defrule "Wna"	"쭘")
(its-defrule "Wnq"	"쭙")
(its-defrule "Wnd"	"쭝")
(its-defrule "Wnj"	"쭤")
(its-defrule "WnjT"	"쭸")
(its-defrule "Wnjd"	"쭹")
(its-defrule "Wnl"	"쮜")
(its-defrule "Wb"	"쮸")
(its-defrule "Wm"	"쯔")
(its-defrule "Wma"	"쯤")
(its-defrule "Wmt"	"쯧")
(its-defrule "Wmd"	"쯩")
(its-defrule "Wl"	"찌")
(its-defrule "Wlr"	"찍")
(its-defrule "Wls"	"찐")
(its-defrule "Wlf"	"찔")
(its-defrule "Wla"	"찜")
(its-defrule "Wlq"	"찝")
(its-defrule "Wld"	"찡")
(its-defrule "Wlw"	"찢")
(its-defrule "Wlg"	"찧")
(its-defrule "ck"	"차")
(its-defrule "ckr"	"착")
(its-defrule "cks"	"찬")
(its-defrule "cksg"	"찮")
(its-defrule "ckf"	"찰")
(its-defrule "cka"	"참")
(its-defrule "ckq"	"찹")
(its-defrule "ckt"	"찻")
(its-defrule "ckT"	"찼")
(its-defrule "ckd"	"창")
(its-defrule "ckw"	"찾")
(its-defrule "co"	"채")
(its-defrule "cor"	"책")
(its-defrule "cos"	"챈")
(its-defrule "cof"	"챌")
(its-defrule "coa"	"챔")
(its-defrule "coq"	"챕")
(its-defrule "cot"	"챗")
(its-defrule "coT"	"챘")
(its-defrule "cod"	"챙")
(its-defrule "ci"	"챠")
(its-defrule "cis"	"챤")
(its-defrule "cisg"	"챦")
(its-defrule "cif"	"챨")
(its-defrule "cia"	"챰")
(its-defrule "cid"	"챵")
(its-defrule "cj"	"처")
(its-defrule "cjr"	"척")
(its-defrule "cjs"	"천")
(its-defrule "cjf"	"철")
(its-defrule "cja"	"첨")
(its-defrule "cjq"	"첩")
(its-defrule "cjt"	"첫")
(its-defrule "cjT"	"첬")
(its-defrule "cjd"	"청")
(its-defrule "cp"	"체")
(its-defrule "cpr"	"첵")
(its-defrule "cps"	"첸")
(its-defrule "cpf"	"첼")
(its-defrule "cpa"	"쳄")
(its-defrule "cpq"	"쳅")
(its-defrule "cpt"	"쳇")
(its-defrule "cpd"	"쳉")
(its-defrule "cu"	"쳐")
(its-defrule "cus"	"쳔")
(its-defrule "cuT"	"쳤")
(its-defrule "cP"	"쳬")
(its-defrule "cPs"	"쳰")
(its-defrule "cPd"	"촁")
(its-defrule "ch"	"초")
(its-defrule "chr"	"촉")
(its-defrule "chs"	"촌")
(its-defrule "chf"	"촐")
(its-defrule "cha"	"촘")
(its-defrule "chq"	"촙")
(its-defrule "cht"	"촛")
(its-defrule "chd"	"총")
(its-defrule "chk"	"촤")
(its-defrule "chks"	"촨")
(its-defrule "chkf"	"촬")
(its-defrule "chkd"	"촹")
(its-defrule "chl"	"최")
(its-defrule "chls"	"쵠")
(its-defrule "chlf"	"쵤")
(its-defrule "chla"	"쵬")
(its-defrule "chlq"	"쵭")
(its-defrule "chlt"	"쵯")
(its-defrule "chld"	"쵱")
(its-defrule "cy"	"쵸")
(its-defrule "cya"	"춈")
(its-defrule "cn"	"추")
(its-defrule "cnr"	"축")
(its-defrule "cns"	"춘")
(its-defrule "cnf"	"출")
(its-defrule "cna"	"춤")
(its-defrule "cnq"	"춥")
(its-defrule "cnt"	"춧")
(its-defrule "cnd"	"충")
(its-defrule "cnj"	"춰")
(its-defrule "cnjT"	"췄")
(its-defrule "cnp"	"췌")
(its-defrule "cnps"	"췐")
(its-defrule "cnl"	"취")
(its-defrule "cnls"	"췬")
(its-defrule "cnlf"	"췰")
(its-defrule "cnla"	"췸")
(its-defrule "cnlq"	"췹")
(its-defrule "cnlt"	"췻")
(its-defrule "cnld"	"췽")
(its-defrule "cb"	"츄")
(its-defrule "cbs"	"츈")
(its-defrule "cbf"	"츌")
(its-defrule "cba"	"츔")
(its-defrule "cbd"	"츙")
(its-defrule "cm"	"츠")
(its-defrule "cmr"	"측")
(its-defrule "cms"	"츤")
(its-defrule "cmf"	"츨")
(its-defrule "cma"	"츰")
(its-defrule "cmq"	"츱")
(its-defrule "cmt"	"츳")
(its-defrule "cmd"	"층")
(its-defrule "cl"	"치")
(its-defrule "clr"	"칙")
(its-defrule "cls"	"친")
(its-defrule "cle"	"칟")
(its-defrule "clf"	"칠")
(its-defrule "clfr"	"칡")
(its-defrule "cla"	"침")
(its-defrule "clq"	"칩")
(its-defrule "clt"	"칫")
(its-defrule "cld"	"칭")
(its-defrule "zk"	"카")
(its-defrule "zkr"	"칵")
(its-defrule "zks"	"칸")
(its-defrule "zkf"	"칼")
(its-defrule "zka"	"캄")
(its-defrule "zkq"	"캅")
(its-defrule "zkt"	"캇")
(its-defrule "zkd"	"캉")
(its-defrule "zo"	"캐")
(its-defrule "zor"	"캑")
(its-defrule "zos"	"캔")
(its-defrule "zof"	"캘")
(its-defrule "zoa"	"캠")
(its-defrule "zoq"	"캡")
(its-defrule "zot"	"캣")
(its-defrule "zoT"	"캤")
(its-defrule "zod"	"캥")
(its-defrule "zi"	"캬")
(its-defrule "zir"	"캭")
(its-defrule "zid"	"컁")
(its-defrule "zj"	"커")
(its-defrule "zjr"	"컥")
(its-defrule "zjs"	"컨")
(its-defrule "zje"	"컫")
(its-defrule "zjf"	"컬")
(its-defrule "zja"	"컴")
(its-defrule "zjq"	"컵")
(its-defrule "zjt"	"컷")
(its-defrule "zjT"	"컸")
(its-defrule "zjd"	"컹")
(its-defrule "zp"	"케")
(its-defrule "zpr"	"켁")
(its-defrule "zps"	"켄")
(its-defrule "zpf"	"켈")
(its-defrule "zpa"	"켐")
(its-defrule "zpq"	"켑")
(its-defrule "zpt"	"켓")
(its-defrule "zpd"	"켕")
(its-defrule "zu"	"켜")
(its-defrule "zus"	"켠")
(its-defrule "zuf"	"켤")
(its-defrule "zua"	"켬")
(its-defrule "zuq"	"켭")
(its-defrule "zut"	"켯")
(its-defrule "zuT"	"켰")
(its-defrule "zud"	"켱")
(its-defrule "zP"	"켸")
(its-defrule "zh"	"코")
(its-defrule "zhr"	"콕")
(its-defrule "zhs"	"콘")
(its-defrule "zhf"	"콜")
(its-defrule "zha"	"콤")
(its-defrule "zhq"	"콥")
(its-defrule "zht"	"콧")
(its-defrule "zhd"	"콩")
(its-defrule "zhk"	"콰")
(its-defrule "zhkr"	"콱")
(its-defrule "zhks"	"콴")
(its-defrule "zhkf"	"콸")
(its-defrule "zhka"	"쾀")
(its-defrule "zhkd"	"쾅")
(its-defrule "zho"	"쾌")
(its-defrule "zhod"	"쾡")
(its-defrule "zhl"	"쾨")
(its-defrule "zhlf"	"쾰")
(its-defrule "zy"	"쿄")
(its-defrule "zn"	"쿠")
(its-defrule "znr"	"쿡")
(its-defrule "znf"	"쿨")
(its-defrule "zna"	"쿰")
(its-defrule "znq"	"쿱")
(its-defrule "znt"	"쿳")
(its-defrule "znd"	"쿵")
(its-defrule "znj"	"쿼")
(its-defrule "znjs"	"퀀")
(its-defrule "znjf"	"퀄")
(its-defrule "znjd"	"퀑")
(its-defrule "znp"	"퀘")
(its-defrule "znpd"	"퀭")
(its-defrule "znl"	"퀴")
(its-defrule "znlr"	"퀵")
(its-defrule "znls"	"퀸")
(its-defrule "znlf"	"퀼")
(its-defrule "znla"	"큄")
(its-defrule "znlq"	"큅")
(its-defrule "znlt"	"큇")
(its-defrule "znld"	"큉")
(its-defrule "zb"	"큐")
(its-defrule "zbs"	"큔")
(its-defrule "zbf"	"큘")
(its-defrule "zba"	"큠")
(its-defrule "zm"	"크")
(its-defrule "zmr"	"큭")
(its-defrule "zms"	"큰")
(its-defrule "zmf"	"클")
(its-defrule "zma"	"큼")
(its-defrule "zmq"	"큽")
(its-defrule "zmd"	"킁")
(its-defrule "zl"	"키")
(its-defrule "zlr"	"킥")
(its-defrule "zls"	"킨")
(its-defrule "zlf"	"킬")
(its-defrule "zla"	"킴")
(its-defrule "zlq"	"킵")
(its-defrule "zlt"	"킷")
(its-defrule "zld"	"킹")
(its-defrule "xk"	"타")
(its-defrule "xkr"	"탁")
(its-defrule "xks"	"탄")
(its-defrule "xkf"	"탈")
(its-defrule "xkfr"	"탉")
(its-defrule "xka"	"탐")
(its-defrule "xkq"	"탑")
(its-defrule "xkt"	"탓")
(its-defrule "xkT"	"탔")
(its-defrule "xkd"	"탕")
(its-defrule "xo"	"태")
(its-defrule "xor"	"택")
(its-defrule "xos"	"탠")
(its-defrule "xof"	"탤")
(its-defrule "xoa"	"탬")
(its-defrule "xoq"	"탭")
(its-defrule "xot"	"탯")
(its-defrule "xoT"	"탰")
(its-defrule "xod"	"탱")
(its-defrule "xi"	"탸")
(its-defrule "xid"	"턍")
(its-defrule "xj"	"터")
(its-defrule "xjr"	"턱")
(its-defrule "xjs"	"턴")
(its-defrule "xjf"	"털")
(its-defrule "xjfa"	"턺")
(its-defrule "xja"	"텀")
(its-defrule "xjq"	"텁")
(its-defrule "xjt"	"텃")
(its-defrule "xjT"	"텄")
(its-defrule "xjd"	"텅")
(its-defrule "xp"	"테")
(its-defrule "xpr"	"텍")
(its-defrule "xps"	"텐")
(its-defrule "xpf"	"텔")
(its-defrule "xpa"	"템")
(its-defrule "xpq"	"텝")
(its-defrule "xpt"	"텟")
(its-defrule "xpd"	"텡")
(its-defrule "xu"	"텨")
(its-defrule "xus"	"텬")
(its-defrule "xuT"	"텼")
(its-defrule "xP"	"톄")
(its-defrule "xPs"	"톈")
(its-defrule "xh"	"토")
(its-defrule "xhr"	"톡")
(its-defrule "xhs"	"톤")
(its-defrule "xhf"	"톨")
(its-defrule "xha"	"톰")
(its-defrule "xhq"	"톱")
(its-defrule "xht"	"톳")
(its-defrule "xhd"	"통")
(its-defrule "xhv"	"톺")
(its-defrule "xhk"	"톼")
(its-defrule "xhks"	"퇀")
(its-defrule "xho"	"퇘")
(its-defrule "xhl"	"퇴")
(its-defrule "xhls"	"퇸")
(its-defrule "xhlt"	"툇")
(its-defrule "xhld"	"툉")
(its-defrule "xy"	"툐")
(its-defrule "xn"	"투")
(its-defrule "xnr"	"툭")
(its-defrule "xns"	"툰")
(its-defrule "xnf"	"툴")
(its-defrule "xna"	"툼")
(its-defrule "xnq"	"툽")
(its-defrule "xnt"	"툿")
(its-defrule "xnd"	"퉁")
(its-defrule "xnj"	"퉈")
(its-defrule "xnjT"	"퉜")
(its-defrule "xnp"	"퉤")
(its-defrule "xnl"	"튀")
(its-defrule "xnlr"	"튁")
(its-defrule "xnls"	"튄")
(its-defrule "xnlf"	"튈")
(its-defrule "xnla"	"튐")
(its-defrule "xnlq"	"튑")
(its-defrule "xnld"	"튕")
(its-defrule "xb"	"튜")
(its-defrule "xbs"	"튠")
(its-defrule "xbf"	"튤")
(its-defrule "xba"	"튬")
(its-defrule "xbd"	"튱")
(its-defrule "xm"	"트")
(its-defrule "xmr"	"특")
(its-defrule "xms"	"튼")
(its-defrule "xme"	"튿")
(its-defrule "xmf"	"틀")
(its-defrule "xmfa"	"틂")
(its-defrule "xma"	"틈")
(its-defrule "xmq"	"틉")
(its-defrule "xmt"	"틋")
(its-defrule "xml"	"틔")
(its-defrule "xmls"	"틘")
(its-defrule "xmlf"	"틜")
(its-defrule "xmla"	"틤")
(its-defrule "xmlq"	"틥")
(its-defrule "xl"	"티")
(its-defrule "xlr"	"틱")
(its-defrule "xls"	"틴")
(its-defrule "xlf"	"틸")
(its-defrule "xla"	"팀")
(its-defrule "xlq"	"팁")
(its-defrule "xlt"	"팃")
(its-defrule "xld"	"팅")
(its-defrule "vk"	"파")
(its-defrule "vkr"	"팍")
(its-defrule "vkR"	"팎")
(its-defrule "vkrs"	"판")
(its-defrule "vkf"	"팔")
(its-defrule "vkfa"	"팖")
(its-defrule "vka"	"팜")
(its-defrule "vkq"	"팝")
(its-defrule "vkt"	"팟")
(its-defrule "vkT"	"팠")
(its-defrule "vkd"	"팡")
(its-defrule "vkx"	"팥")
(its-defrule "vo"	"패")
(its-defrule "vor"	"팩")
(its-defrule "vos"	"팬")
(its-defrule "vof"	"팰")
(its-defrule "voa"	"팸")
(its-defrule "voq"	"팹")
(its-defrule "vot"	"팻")
(its-defrule "voT"	"팼")
(its-defrule "vod"	"팽")
(its-defrule "vi"	"퍄")
(its-defrule "vir"	"퍅")
(its-defrule "vj"	"퍼")
(its-defrule "vjr"	"퍽")
(its-defrule "vjs"	"펀")
(its-defrule "vja"	"펌")
(its-defrule "vjq"	"펍")
(its-defrule "vjt"	"펏")
(its-defrule "vjtT"	"펐")
(its-defrule "vjd"	"펑")
(its-defrule "vp"	"페")
(its-defrule "vpr"	"펙")
(its-defrule "vps"	"펜")
(its-defrule "vpf"	"펠")
(its-defrule "vpa"	"펨")
(its-defrule "vpq"	"펩")
(its-defrule "vpt"	"펫")
(its-defrule "vpd"	"펭")
(its-defrule "vu"	"펴")
(its-defrule "vus"	"편")
(its-defrule "vuf"	"펼")
(its-defrule "vua"	"폄")
(its-defrule "vuq"	"폅")
(its-defrule "vuT"	"폈")
(its-defrule "vud"	"평")
(its-defrule "vP"	"폐")
(its-defrule "vPf"	"폘")
(its-defrule "vPq"	"폡")
(its-defrule "vPt"	"폣")
(its-defrule "vh"	"포")
(its-defrule "vhr"	"폭")
(its-defrule "vhs"	"폰")
(its-defrule "vhf"	"폴")
(its-defrule "vha"	"폼")
(its-defrule "vhq"	"폽")
(its-defrule "vht"	"폿")
(its-defrule "vhd"	"퐁")
(its-defrule "vhk"	"퐈")
(its-defrule "vhkd"	"퐝")
(its-defrule "vhl"	"푀")
(its-defrule "vhls"	"푄")
(its-defrule "vy"	"표")
(its-defrule "vys"	"푠")
(its-defrule "vyf"	"푤")
(its-defrule "vyq"	"푭")
(its-defrule "vyt"	"푯")
(its-defrule "vn"	"푸")
(its-defrule "vnr"	"푹")
(its-defrule "vns"	"푼")
(its-defrule "vne"	"푿")
(its-defrule "vnf"	"풀")
(its-defrule "vnfa"	"풂")
(its-defrule "vna"	"품")
(its-defrule "vnq"	"풉")
(its-defrule "vnt"	"풋")
(its-defrule "vnd"	"풍")
(its-defrule "vnj"	"풔")
(its-defrule "vnjd"	"풩")
(its-defrule "vnl"	"퓌")
(its-defrule "vnls"	"퓐")
(its-defrule "vnlf"	"퓔")
(its-defrule "vnla"	"퓜")
(its-defrule "vnlt"	"퓟")
(its-defrule "vb"	"퓨")
(its-defrule "vbs"	"퓬")
(its-defrule "vbf"	"퓰")
(its-defrule "vba"	"퓸")
(its-defrule "vbt"	"퓻")
(its-defrule "vbd"	"퓽")
(its-defrule "vm"	"프")
(its-defrule "vms"	"픈")
(its-defrule "vmf"	"플")
(its-defrule "vma"	"픔")
(its-defrule "vmq"	"픕")
(its-defrule "vmt"	"픗")
(its-defrule "vl"	"피")
(its-defrule "vlr"	"픽")
(its-defrule "vls"	"핀")
(its-defrule "vlf"	"필")
(its-defrule "vla"	"핌")
(its-defrule "vlq"	"핍")
(its-defrule "vlt"	"핏")
(its-defrule "vld"	"핑")
(its-defrule "gk"	"하")
(its-defrule "gkr"	"학")
(its-defrule "gks"	"한")
(its-defrule "gkf"	"할")
(its-defrule "gkfx"	"핥")
(its-defrule "gka"	"함")
(its-defrule "gkq"	"합")
(its-defrule "gkt"	"핫")
(its-defrule "gkd"	"항")
(its-defrule "go"	"해")
(its-defrule "gor"	"핵")
(its-defrule "gos"	"핸")
(its-defrule "gof"	"핼")
(its-defrule "goa"	"햄")
(its-defrule "goq"	"햅")
(its-defrule "got"	"햇")
(its-defrule "goT"	"했")
(its-defrule "god"	"행")
(its-defrule "gi"	"햐")
(its-defrule "gid"	"향")
(its-defrule "gj"	"허")
(its-defrule "gjr"	"헉")
(its-defrule "gjs"	"헌")
(its-defrule "gjf"	"헐")
(its-defrule "gjfa"	"헒")
(its-defrule "gja"	"험")
(its-defrule "gjq"	"헙")
(its-defrule "gjt"	"헛")
(its-defrule "gjd"	"헝")
(its-defrule "gp"	"헤")
(its-defrule "gpr"	"헥")
(its-defrule "gps"	"헨")
(its-defrule "gpf"	"헬")
(its-defrule "gpa"	"헴")
(its-defrule "gpq"	"헵")
(its-defrule "gpt"	"헷")
(its-defrule "gpd"	"헹")
(its-defrule "gu"	"혀")
(its-defrule "gur"	"혁")
(its-defrule "gus"	"현")
(its-defrule "guf"	"혈")
(its-defrule "gua"	"혐")
(its-defrule "guq"	"협")
(its-defrule "gut"	"혓")
(its-defrule "guT"	"혔")
(its-defrule "gud"	"형")
(its-defrule "gP"	"혜")
(its-defrule "gPs"	"혠")
(its-defrule "gPf"	"혤")
(its-defrule "gPq"	"혭")
(its-defrule "gh"	"호")
(its-defrule "ghr"	"혹")
(its-defrule "ghs"	"혼")
(its-defrule "ghf"	"홀")
(its-defrule "ghfx"	"홅")
(its-defrule "gha"	"홈")
(its-defrule "ghq"	"홉")
(its-defrule "ght"	"홋")
(its-defrule "ghd"	"홍")
(its-defrule "ghx"	"홑")
(its-defrule "ghk"	"화")
(its-defrule "ghkr"	"확")
(its-defrule "ghks"	"환")
(its-defrule "ghkf"	"활")
(its-defrule "ghkt"	"홧")
(its-defrule "ghkd"	"황")
(its-defrule "gho"	"홰")
(its-defrule "ghor"	"홱")
(its-defrule "ghos"	"홴")
(its-defrule "ghot"	"횃")
(its-defrule "ghod"	"횅")
(its-defrule "ghl"	"회")
(its-defrule "ghlr"	"획")
(its-defrule "ghls"	"횐")
(its-defrule "ghlf"	"횔")
(its-defrule "ghlq"	"횝")
(its-defrule "ghlt"	"횟")
(its-defrule "ghld"	"횡")
(its-defrule "gy"	"효")
(its-defrule "gys"	"횬")
(its-defrule "gyf"	"횰")
(its-defrule "gyq"	"횹")
(its-defrule "gyt"	"횻")
(its-defrule "gn"	"후")
(its-defrule "gnr"	"훅")
(its-defrule "gns"	"훈")
(its-defrule "gnf"	"훌")
(its-defrule "gnfx"	"훑")
(its-defrule "gna"	"훔")
(its-defrule "gnt"	"훗")
(its-defrule "gnd"	"훙")
(its-defrule "gnj"	"훠")
(its-defrule "gnjs"	"훤")
(its-defrule "gnjf"	"훨")
(its-defrule "gnja"	"훰")
(its-defrule "gnjd"	"훵")
(its-defrule "gnp"	"훼")
(its-defrule "gnpr"	"훽")
(its-defrule "gnps"	"휀")
(its-defrule "gnpf"	"휄")
(its-defrule "gnpd"	"휑")
(its-defrule "gnl"	"휘")
(its-defrule "gnlr"	"휙")
(its-defrule "gnls"	"휜")
(its-defrule "gnlf"	"휠")
(its-defrule "gnla"	"휨")
(its-defrule "gnlq"	"휩")
(its-defrule "gnlt"	"휫")
(its-defrule "gnld"	"휭")
(its-defrule "gb"	"휴")
(its-defrule "gbr"	"휵")
(its-defrule "gbs"	"휸")
(its-defrule "gbf"	"휼")
(its-defrule "gba"	"흄")
(its-defrule "gbt"	"흇")
(its-defrule "gbd"	"흉")
(its-defrule "gm"	"흐")
(its-defrule "gmr"	"흑")
(its-defrule "gms"	"흔")
(its-defrule "gmsg"	"흖")
(its-defrule "gme"	"흗")
(its-defrule "gmf"	"흘")
(its-defrule "gmfr"	"흙")
(its-defrule "gma"	"흠")
(its-defrule "gmq"	"흡")
(its-defrule "gmt"	"흣")
(its-defrule "gmd"	"흥")
(its-defrule "gmx"	"흩")
(its-defrule "gml"	"희")
(its-defrule "gmls"	"흰")
(its-defrule "gmlf"	"흴")
(its-defrule "gmla"	"흼")
(its-defrule "gmlq"	"흽")
(its-defrule "gmld"	"힁")
(its-defrule "gl"	"히")
(its-defrule "glr"	"힉")
(its-defrule "gls"	"힌")
(its-defrule "glf"	"힐")
(its-defrule "gla"	"힘")
(its-defrule "glq"	"힙")
(its-defrule "glt"	"힛")
(its-defrule "gld"	"힝"))
