;;; custom-load.el --- automatically extracted custom dependencies


;;; Code:

(custom-add-loads 'extensions '("iswitchb" "page-ext" "time-stamp"))
(custom-add-loads 'change-log '("add-log"))
(custom-add-loads 'filladapt '("filladapt"))
(custom-add-loads 'copyright '("upd-copyr"))
(custom-add-loads 'info-faces '("info"))
(custom-add-loads 'recent-files-menu '("recent-files"))
(custom-add-loads 'mouse '("avoid"))
(custom-add-loads 'tex '("texnfo-tex"))
(custom-add-loads 'tar '("tar-mode"))
(custom-add-loads 'etags '("etags"))
(custom-add-loads 'igrep '("igrep"))
(custom-add-loads 'menu '("recent-files"))
(custom-add-loads 'minibuffer '("icomplete"))
(custom-add-loads 'environment '("gnuserv"))
(custom-add-loads 'texinfo '("texnfo-tex"))
(custom-add-loads 'terminals '("gnuserv"))
(custom-add-loads 'auto-save '("auto-save"))
(custom-add-loads 'ispell '("auto-autoloads" "ispell"))
(custom-add-loads 'mail '("feedmail" "metamail" "supercite"))
(custom-add-loads 'ps-print-face '("ps-print"))
(custom-add-loads 'crypt '("crypt"))
(custom-add-loads 'object '("gopher"))
(custom-add-loads 'man '("man"))
(custom-add-loads 'lpr '("lpr"))
(custom-add-loads 'ps-print-header '("ps-print"))
(custom-add-loads 'docs '("hyper-apropos" "info" "makeinfo"))
(custom-add-loads 'completion '("completion"))
(custom-add-loads 'tools '("add-log" "autoinsert" "compile" "diff" "etags" "func-menu" "generic-sc" "hyper-apropos" "rcompile"))
(custom-add-loads 'recent-files '("recent-files"))
(custom-add-loads 'display-time-balloon '("time"))
(custom-add-loads 'dabbrev '("dabbrev"))
(custom-add-loads 'display-time '("time"))
(custom-add-loads 'hypermedia '("gopher" "metamail"))
(custom-add-loads 'save-place '("saveplace"))
(custom-add-loads 'lisp '("func-menu" "hyper-apropos"))
(custom-add-loads 'jka-compr '("jka-compr"))
(custom-add-loads 'diff '("diff"))
(custom-add-loads 'supercite-cite '("supercite"))
(custom-add-loads 'applications '("time"))
(custom-add-loads 'paren-matching '("paren"))
(custom-add-loads 'time-stamp '("time-stamp"))
(custom-add-loads 'avoid '("avoid"))
(custom-add-loads 'help '("hyper-apropos" "info" "man"))
(custom-add-loads 'supercite '("supercite"))
(custom-add-loads 'generic-sc '("generic-sc"))
(custom-add-loads 'local '("gopher"))
(custom-add-loads 'keyboard '("pending-del"))
(custom-add-loads 'hyper-apropos-faces '("hyper-apropos"))
(custom-add-loads 'data '("auto-save" "jka-compr" "saveplace" "tar-mode" "time-stamp"))
(custom-add-loads 'ps-print '("ps-print"))
(custom-add-loads 'compression '("jka-compr"))
(custom-add-loads 'comm '("gopher"))
(custom-add-loads 'ps-print-font '("ps-print"))
(custom-add-loads 'backup '("backup-dir"))
(custom-add-loads 'frames '("gnuserv" "balloon-help" "desktop"))
(custom-add-loads 'supercite-attr '("supercite"))
(custom-add-loads 'bookmarks '("bookmark"))
(custom-add-loads 'desktop '("desktop"))
(custom-add-loads 'abbrev '("dabbrev"))
(custom-add-loads 'remote-compile '("rcompile"))
(custom-add-loads 'programming '("compile"))
(custom-add-loads 'metamail '("metamail"))
(custom-add-loads 'icomplete '("icomplete"))
(custom-add-loads 'compilation '("compile" "auto-autoloads"))
(custom-add-loads 'iswitchb '("iswitchb"))
(custom-add-loads 'makeinfo '("makeinfo"))
(custom-add-loads 'fume '("func-menu"))
(custom-add-loads 'auto-insert '("autoinsert"))
(custom-add-loads 'files '("recent-files"))
(custom-add-loads 'fast-lock '("fast-lock"))
(custom-add-loads 'gnuserv '("gnuserv"))
(custom-add-loads 'ps-print-horizontal '("ps-print"))
(custom-add-loads 'maint '("add-log" "upd-copyr"))
(custom-add-loads 'fill '("filladapt"))
(custom-add-loads 'balloon-help '("balloon-help"))
(custom-add-loads 'supercite-hooks '("supercite"))
(custom-add-loads 'texinfo-tex '("texnfo-tex"))
(custom-add-loads 'faces '("fast-lock" "hyper-apropos" "info" "ps-print"))
(custom-add-loads 'pages '("page-ext"))
(custom-add-loads 'supercite-frames '("supercite"))
(custom-add-loads 'feedmail '("feedmail"))
(custom-add-loads 'processes '("compile" "gnuserv" "igrep" "ispell" "metamail" "rcompile"))
(custom-add-loads 'news '("supercite"))
(custom-add-loads 'hyper-apropos '("hyper-apropos"))
(custom-add-loads 'wp '("lpr" "ps-print"))
(custom-add-loads 'ps-print-vertical '("ps-print"))
(custom-add-loads 'gopher '("gopher"))
(custom-add-loads 'editing '("bookmark"))
(custom-add-loads 'matching '("paren" "completion" "hyper-apropos"))
(custom-add-loads 'ps-print-color '("ps-print"))
(custom-add-loads 'info '("info"))
(custom-add-loads 'unix '("tar-mode"))
(custom-add-loads 'c '("func-menu"))

;;; custom-load.el ends here
