;;; custom-load.el --- automatically extracted custom dependencies


;;; Code:

(custom-add-loads 'ssh '("ssh"))
(custom-add-loads 'telnet '("telnet"))
(custom-add-loads 'shell '("shell"))
(custom-add-loads 'comint-completion '("comint"))
(custom-add-loads 'comint '("comint-xemacs" "comint" "telnet"))
(custom-add-loads 'rlogin '("rlogin"))
(custom-add-loads 'shell-faces '("shell"))
(custom-add-loads 'shell-directories '("shell"))
(custom-add-loads 'comint-source '("comint"))
(custom-add-loads 'processes '("background" "comint" "rlogin" "shell" "ssh"))
(custom-add-loads 'background '("background"))
(custom-add-loads 'unix '("rlogin" "shell" "ssh"))

;;; custom-load.el ends here
