;; This file is part of Egg on Mule (Multilingual Environment)

;; Egg is distributed in the forms of patches to GNU
;; Emacs under the terms of the GNU EMACS GENERAL PUBLIC
;; LICENSE which is distributed along with GNU Emacs by the
;; Free Software Foundation.

;; Egg is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied
;; warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;; PURPOSE.  See the GNU EMACS GENERAL PUBLIC LICENSE for
;; more details.

;; You should have received a copy of the GNU EMACS GENERAL
;; PUBLIC LICENSE along with Nemacs; see the file COPYING.
;; If not, write to the Free Software Foundation, 675 Mass
;; Ave, Cambridge, MA 02139, USA.

;;;
;;;  busyu.el
;;;

;;;  Written by Toshiaki Shingu (shingu@cpr.canon.co.jp)
;;; 92.7.24 modified by T.Shingu
;;;	busyu-input and kakusuu-input is re-written.
;;; 92.8.24 modified by T.Shingu
;;;	Bub busyu-input fixed.
;;; 92.9.17 modified by K.Handa <handa@etl.go.jp>
;;;	All tables are defined directly (not by marcro).
;;; 92.9.20 modified by T.Enami <enami@sys.ptg.sony.co.jp>
;;;	In busyu-input, unnecessary mapcar avoided.
;;; 92.9.30  modified for Mule Ver.0.9.6 by T.Mitani <mit@huie.hokudai.ac.jp>
;;;	In busyu-input and kakusuu-input, bugs in kakusuu check fixed.

;;;  The tables of bushu (部首) and kakusuu (画数) are copied from:
;;;	『漢字辞書　「ｋａｎ君」　Ｖｅｒ１．０』
;;;  with slight modifications at:
;;;	("ぼう（个,中など）" . "ぼう")
;;;	("やまいだれ（疔,疚など）" . "やまいだれ")
;;;	("しんにゅう（辷,辺）" . "しんにゅう")
;;;	("ぐうのあし（禹,禺,禽）" . "ぐうのあし")
;;;  In addition to the notice at the head of this file (bushu.el),
;;;  you should also obey the document below to copy or destribute
;;;  these tables.  The document is the one attached to the original.
;;;
;**************************************************************************
;
;　　漢字辞書　「ｋａｎ君」　Ｖｅｒ１．０
;                                                                H3,10,20
;**************************************************************************
;
;／１．はじめに／
;
;　現在の日本語ＦＥＰで部首変換といているのは、第二水準のもののみといったものが
;多く、完全にカバーしているものは見あたりません。そこで、第一水準、第二水準とも
;部首変換と総画数の変換ができれば便利ではない思い作り上げました。
;このデータは、多くの日本語フロントエンドプロセッサに対応しています。
;このデータを作成するには、ＪＧＡＷＫが必要です。
;
;
;／２．出力形式／
;
;・日本語ＦＥＰ　一括で辞書登録できるテキストファイルを出力します。
;
;　　ＡＴＯＫ７          （株）ジャストシステム
;　　松茸Ｖ３            （株）管理工学研究所
;　　ＷＸ２＋（ＷＸＰ）   エーアイソフト（株）
;　　ＶＪＥ−β          （株）バックス
;　　ＤＦＪ              （株）デジタル・ファーム
;
;　　注意）
;　　　　・ＷＸ２＋のタイプはフリーウェア版のＷＸＰにも対応しております。
;　　　　・ＶＪＥ−βは単体では、登録できません。
;　　　　　同社から販売されている「ＶＪＥ−Ｔｏｏｌｓ」をお使い下さい。
;
;
;／３．ファイルの内容／
;
;README.DOC      このドキメント
;KAN   .COM      データ作成用プログラム
;KANMAN.TXT      TEL.BATのコマンドリファレンスシート
;KANADV.TXT      アドバイスシート（実行前に必ず読んで下さい）
;BUSYU .TXT      部首読み対応シート
;KANJI .DAT      漢字辞書データ
;REPORT.TXT      ユーザー登録用紙
;
;
;／４．使用方法／
;
;・用意する物
;
;このデータ作成するには、ＪＧＡＷＫが必要です。雑誌にも時々付いていますので
;お持ちの方は多いと思われます。大きなネットでも大体アップロードされています。
;91年10月現在、主なネットの所在は次のとおりです。
;
;NIFTY-Serve     FGALAP LIB:7    53  JGAWK29.LZH GNUawk2.11.1+2.9MSDOS漢字版
;ASCII-NET pcs   pool   msdos  2174  JGA_EXE.ISH GNU awk 2.11.1+2 MSDOS版の
;                                                漢字対応版の.exeとマニュアル
;                       msdos  3655  JGA_29.ISH  jgawk 2.11.1 + 2.0 → +2.9
;                                                (JGA_EXE.LZH の差分ファイル)
;・使い方
;
;A>KAN  (リターン)
;
;で、簡単なパラメータの説明がでてきます。
;詳しい説明は、付属の KANMAN.TXT　に書いてあります。
;
;たとえば、ＡＴＯＫ７場合は次のようになります。
;
;A>KAN ATOK B 1 @
;
;ATOK    　　ＡＴＯＫ７用
;B     　　総画数変換辞書を出力
;1       　　第一水準のみを出力
;@       　　読みに@が付けられる
;
;以上を実行すると、KAN.TXTというファイルができます。
;各ＦＥＰ、データベースの登録方法は、各マニュアルを参考下さい。
;辞書登録ならば、「一括辞書登録」「辞書併合」などの箇所に書いてあります。
;
;
;／５．ユーザ登録につて（無償）／
;
;  いろいろの方に質問や指摘、アイデイアいただきありがとうございました。データが
;間違っていた場合は、こちらまでお知らせくだされば幸いです。質問なども付属の登録
;用紙をお使いください。ユーザ登録のメールを受け取った場合は、１，２週間以内に折
;り返しご返事のメールを差し上げます。万一メールがこない場合は、お問い合わせ下さ
;い。
;
;また、次の事項に付いてご了解願います。
;
;・この登録は、一般的なユーザ登録と違い、お互いに権利や義務は存在いたしません。
;・私が、皆様方にデータ作成のお手伝いをお願いした場合は、お忙しい方は仕方があり
;　ませんがなるべくご協力下さるようお願いいたします。
;・このデータによって生じたいかなる事故については責任を負いかねます。
;・データの更新に努めますが、諸処の事情により滞る場合があります。
;・問い合わせなどにはなるべくご返事いたします。返事が１週間以上無い場合は、忙し
;　いか忘れている場合です。申し訳ございませんが再度お送り下さい。
;
;
;／６．転載、紹介、販売などについて／
;
;＜転載＞
;　他のネットへの転載は許可しますが、転載者は必ずユーザ登録を行なって下さい。
;データの更新時には、最新版をアップロードするように心がけて下さい。
;　データの変更は混乱を招きますので、なるべくこのままの形でアップして下さい。
;もし、変更する場合は一度ご連絡下さい。
;　同時にアップロードされている下記のファイルも一緒に登録してください。
;
;・郵便番号簿　ZIP91A.LZH ・市外局番簿　TEL91A.LZH ・漢字辞書　KANJIDIC.LZH
;
;＜紹介＞
;　雑誌等の紹介や収録に付いては、更新などで新たに公開しているものが紹介されない
;可能性がありますので一度ご連絡下さい。フロッピーなどの収録時には、現時点での、
;修正版を提供いたします。
;
;＜販売＞
;営利目的でこのデータを取り込んだソフトの販売については一度ご連絡下さい。
;基本的に了解いたします。詳しいことは、改めて問い合わせ下さい。
;
;
;／７．連絡先／
;
;メールは下記までお送りください。
;
;ＡＳＣＩＩ−ＮＥＴ（ＰＣＳ）    pcs35011
;ＮＩＦＴＹ−Ｓｅｒｖｅ          NAG01423
;ＰＣ−ＶＡＮ                    GLG93462
;日経ＭＩＸ                      tadashi
;ＭＡＳＴＥＲ−ＮＥＴ            CAG741
;**************************************************************************

(defvar busyu-table
  '[0					; ignore
    (("一" . "いち")			; 1
     ("ぼう（个,中など）" . "ぼう")
     ("丶" . "てん")
     ("丿" . "の")
     ("乙" . "おつ")
     ("亅" . "はねぼう")
     )
    (("二" . "に")			; 2
     ("亠" . "なべふた")
     ("人" . "ひと")
     ("儿" . "ひとあし")
     ("入" . "いる")
     ("八" . "はち")
     ("冂" . "えんがまえ")
     ("冖" . "わかんむり")
     ("冫" . "にすい")
     ("几" . "つくえ")
     ("凵" . "うけばこ")
     ("刀" . "かたな")
     ("力" . "ちから")
     ("勹" . "つつみがまえ")
     ("匕" . "さじのひ")
     ("匚" . "はこがまえ")
     ("匸" . "かくしがまえ")
     ("十" . "じゅう")
     ("卜" . "ぼくのと")
     ("卩" . "ふしづくり")
     ("厂" . "がんだれ")
     ("厶" . "む")
     ("又" . "また")
     )
    (("口（くち）" . "くち")		; 3
     ("囗（くにがまえ）" . "くにがまえ")
     ("土" . "つち")
     ("士" . "さむらい")
     ("夂" . "ふゆがしら")
     ("夊" . "すいにょう")
     ("夕" . "ゆうべ")
     ("大" . "だい")
     ("女" . "おんな")
     ("子" . "こ")
     ("宀" . "うかんむり")
     ("寸" . "すん")
     ("小" . "ちいさい")
     ("尢" . "まげあし")
     ("尸" . "しかばね")
     ("屮" . "てつ")
     ("山" . "やま")
     ("巛" . "まがりがわ")
     ("工" . "たくみ")
     ("己" . "おのれ")
     ("巾" . "はば")
     ("干" . "ほす")
     ("幺" . "いとがしら")
     ("广" . "まだれ")
     ("廴" . "えんにょう")
     ("廾" . "にじゅうあし")
     ("弋" . "しきがまえ")
     ("弓" . "ゆみ")
     ("彑" . "けいがしら")
     ("彡" . "さんづくり")
     ("彳" . "ぎょうにんべん")
     )
    (("心" . "こころ")			; 4
     ("戈" . "かのほこ")
     ("戸" . "とびらのと")
     ("手" . "て")
     ("支" . "じゅうまた")
     ("攴" . "とまた")
     ("文" . "ぶん")
     ("斗" . "とます")
     ("斤" . "きん")
     ("方" . "ほう")
     ("旡" . "すでのつくり")
     ("日" . "にち")
     ("曰" . "ひらび")
     ("月" . "つき")
     ("木" . "き")
     ("欠" . "かける")
     ("止" . "とめる")
     ("歹" . "いちた")
     ("殳" . "るまた")
     ("毋" . "なかれ")
     ("比" . "くらべるひ")
     ("毛" . "け")
     ("氏" . "うじ")
     ("气" . "きがまえ")
     ("水" . "みず")
     ("火" . "ひ")
     ("爪" . "つめ")
     ("父" . "ちち")
     ("爻" . "めめ")
     ("爿" . "しょうへん")
     ("片" . "かた")
     ("牙" . "きば")
     ("牛" . "うし")
     ("犬" . "いぬ")
     )
    (("玄" . "げん")			; 5
     ("玉" . "たま")
     ("瓜" . "うり")
     ("瓦" . "かわら")
     ("甘" . "あまい")
     ("生" . "うまれる")
     ("用" . "もちいる")
     ("田" . "た")
     ("疋" . "ひき")
     ("やまいだれ（疔,疚など）" . "やまいだれ")
     ("癶" . "はつがしら")
     ("白" . "しろ")
     ("皮" . "ひのかわら")
     ("皿" . "さら")
     ("目" . "め")
     ("矛" . "むのほこ")
     ("矢" . "や")
     ("石" . "いし")
     ("示" . "しめす")
     ("ぐうのあし（禹,禺,禽）" . "ぐうのあし")
     ("禾" . "のぎ")
     ("穴" . "あな")
     ("立" . "たつ")
     )
    (("竹" . "たけ")			; 6
     ("米" . "こめ")
     ("糸" . "いと")
     ("缶" . "ほとぎ")
     ("网" . "あみがしら")
     ("羊" . "ひつじ")
     ("羽" . "はね")
     ("老" . "おい")
     ("而" . "しかして")
     ("耒" . "らいすき")
     ("耳" . "みみ")
     ("聿" . "ふでづくり")
     ("肉" . "にく")
     ("臣" . "しん")
     ("自" . "みずから")
     ("至" . "いたる")
     ("臼" . "うす")
     ("舌" . "した")
     ("舛" . "ます")
     ("舟" . "ふね")
     ("艮" . "こん")
     ("色" . "いろ")
     ("艸" . "くさ")
     ("虍" . "とらがしら")
     ("虫" . "むし")
     ("血" . "ち")
     ("行" . "ぎょう")
     ("衣" . "ころも")
     ("襾" . "にし")
     )
    (("見" . "みる")			; 7
     ("角" . "つの")
     ("言" . "ことば")
     ("谷" . "たに")
     ("豆" . "まめ")
     ("豕" . "いのこ")
     ("豸" . "むじな")
     ("貝" . "かい")
     ("赤" . "あか")
     ("走" . "はしる")
     ("足" . "あし")
     ("身" . "み")
     ("車" . "くるま")
     ("辛" . "からい")
     ("辰" . "しんのたつ")
     ("しんにゅう（辷,辺）" . "しんにゅう")
     ("邑" . "むら")
     ("酉" . "さけのとり")
     ("釆" . "のごめ")
     ("里" . "さと")
     )
    (("金" . "かね")			; 8
     ("長" . "ながい")
     ("門" . "もん")
     ("阜" . "ぎふのふ")
     ("隶" . "れいづくり")
     ("隹" . "ふるとり")
     ("雨" . "あめ")
     ("青" . "あお")
     ("非" . "あらず")
     )
    (("面" . "めん")			; 9
     ("革" . "かくのかわ")
     ("韋" . "なめしがわ")
     ("韭" . "にら")
     ("音" . "おと")
     ("頁" . "おおがい")
     ("風" . "かぜ")
     ("飛" . "とぶ")
     ("食" . "しょく")
     ("首" . "くび")
     ("香" . "においこう")
     )
    (("馬" . "うま")			; 10
     ("骨" . "ほね")
     ("高" . "たかい")
     ("髟" . "かみがしら")
     ("鬥" . "とうがまえ")
     ("鬯" . "ちょう")
     ("鬲" . "かく")
     ("鬼" . "おに")
     )
    (("魚" . "うお")			; 11
     ("鳥" . "とり")
     ("鹵" . "ろ")
     ("鹿" . "しか")
     ("麦" . "むぎ")
     ("麻" . "あさ")
     )
    (("黄" . "きいろ")			; 12
     ("黍" . "きび")
     ("黒" . "くろ")
     ("黹" . "ふつ")
     )
    (("黽" . "べん")			; 13
     ("鼎" . "かなえ")
     ("鼓" . "つづみ")
     ("鼠" . "ねずみ")
     )
    (("鼻" . "はな")			; 14
     ("齊" . "せい")
     )
    (("齒" . "は")			; 15
     )
    (("龍" . "りゅう")			; 16
     ("龜" . "かめ")
     )
    (("龠" . "やく")			; 17
     )
    ])

(defvar busyu-kaku-alist
  '(("あお"
     (8 . "青")
     (13 . "靖")
     (14 . "静")
     (16 . "靜"))
    ("あか"
     (7 . "赤")
     (12 . "赧")
     (14 . "赫")
     (16 . "赭"))
    ("あさ"
     (11 . "麻")
     (14 . "麼")
     (15 . "麾")
     (18 . "麿"))
    ("あし"
     (7 . "足")
     (11 . "跂趾趺")
     (12 . "跏距跚跖跌跛跋")
     (13 . "跪跫跨跟跡践跣跳路")
     (14 . "踈跼踊踉跿")
     (15 . "踐踝踞踟踏踪")
     (16 . "踴蹂踵蹄踰")
     (17 . "蹊蹇蹉蹌蹐蹈")
     (18 . "蹠蹟蹙蹤蹣蹕")
     (19 . "蹶蹴蹲蹼")
     (20 . "躁躇躄躅")
     (21 . "躋躊躍")
     (22 . "躓躑躔")
     (23 . "躙")
     (25 . "躡")
     (27 . "躪"))
    ("あな"
     (5 . "穴")
     (7 . "究")
     (8 . "穹空突")
     (9 . "穽窃穿")
     (10 . "窈")
     (11 . "窄窓窒窕")
     (12 . "窗窘窖")
     (13 . "窟")
     (14 . "窪窩")
     (15 . "窮窯窰")
     (16 . "窶窺")
     (17 . "窿竃")
     (18 . "竅竄")
     (20 . "竇")
     (21 . "竈")
     (22 . "竊"))
    ("あまい"
     (5 . "甘")
     (9 . "甚")
     (11 . "甜")
     (13 . "甞"))
    ("あみがしら"
     (6 . "网")
     (7 . "罕")
     (8 . "罔")
     (9 . "罘")
     (10 . "罟罠")
     (13 . "罨罫罪署置罩罧")
     (14 . "罰")
     (15 . "罸罵罷")
     (16 . "罹")
     (18 . "羂")
     (19 . "羃羆羅")
     (22 . "羇")
     (24 . "羈"))
    ("あめ"
     (8 . "雨")
     (11 . "雪雫")
     (12 . "雲雰")
     (13 . "電雹雷零")
     (14 . "需")
     (15 . "霄震霆霈霊")
     (16 . "霍霓霎霑霏霖")
     (17 . "霙霞霜")
     (18 . "霤")
     (19 . "霪霧")
     (20 . "霰露")
     (21 . "霸霹")
     (22 . "霽霾")
     (24 . "靈靄靆靂")
     (25 . "靉"))
    ("あらず"
     (8 . "非")
     (15 . "靠")
     (19 . "靡"))
    ("いし"
     (5 . "石")
     (8 . "矼")
     (9 . "研砂砕砌砒")
     (10 . "砦砥砠砧破砲砺砿")
     (11 . "硅")
     (12 . "硯硬硝硫硲硴")
     (13 . "碚碗碎碍碕碁碓碇硼碌碆")
     (14 . "碪碣磁碑碧碯碵碩")
     (15 . "磆磑確磋磔碾碼磐磅磊")
     (16 . "磚磬磧磨")
     (17 . "磯磽礁磴")
     (18 . "礒礎礑礇")
     (19 . "礙")
     (20 . "礬礪礫礦"))
    ("いたる"
     (6 . "至")
     (10 . "致")
     (14 . "臺")
     (16 . "臻"))
    ("いちた"
     (4 . "歹")
     (6 . "死")
     (8 . "歿殀")
     (9 . "殆殄殃")
     (10 . "残殊殉")
     (11 . "殍")
     (12 . "殘殖殕")
     (14 . "殞")
     (15 . "殤")
     (16 . "殪殫")
     (18 . "殯")
     (19 . "殱")
     (21 . "殲"))
    ("いち"
     (1 . "一")
     (2 . "七丁")
     (3 . "下三上丈万与")
     (4 . "丐丑不")
     (5 . "丘且世丗丕丙")
     (6 . "丞両")
     (8 . "並"))
    ("いとがしら"
     (3 . "幺")
     (4 . "幻")
     (5 . "幼")
     (9 . "幽")
     (12 . "幾"))
    ("いと"
     (6 . "糸")
     (7 . "系糺")
     (9 . "紆紀級糾紅紂約")
     (10 . "紜紘紗索紙純素紐納紕紊紛紡紋")
     (11 . "経絅絃紺細紮紫終紹紳紲組紿紬紵絆累絋")
     (12 . "絲絵給結絢絞絳絖絎絨絮絶統絣絡絏")
     (13 . "經継絹絛綏続絽綛綉")
     (14 . "維綺綮綣綱綵緇綽綬緒総綜綻綢綴綯緋綿網綾緑綸綟練綰綫緕")
     (15 . "緲緜縁緩緘緊緝縄緤線緞緻締緡編緬縅")
     (16 . "緯縊縣縞縡縒縦縟縉縋縢縛繁縫")
     (17 . "縲總縱繆繦縮績繊縵縻縹繃縷縺")
     (18 . "繪繝繞織繕繙繚繧繖繍")
     (19 . "繩繭繹繋繰")
     (20 . "繼纂繻繽辮纃繿")
     (21 . "續纎纈纏纐纉")
     (22 . "纒")
     (23 . "纖纓纔")
     (25 . "纛")
     (27 . "纜"))
    ("いぬ"
     (4 . "犬")
     (5 . "犯")
     (6 . "犲")
     (7 . "状狂狃狆狄犹")
     (8 . "狐狎狗狙狛狒")
     (9 . "狠狭狡狩独狢")
     (10 . "狹狷狸狼狽")
     (11 . "倏猗猊猜猖猝猪猛猟")
     (12 . "猫猴猩猶猥猯")
     (13 . "献猷猿猾獅")
     (14 . "獄獏")
     (15 . "獎獗")
     (16 . "獣獨獪獲")
     (17 . "獰")
     (18 . "獵")
     (19 . "獸獺")
     (20 . "獻"))
    ("いのこ"
     (7 . "豕")
     (11 . "豚")
     (12 . "象")
     (13 . "豢")
     (14 . "豪")
     (16 . "豫豬"))
    ("いる"
     (2 . "入")
     (6 . "全")
     (8 . "兩")
     (9 . "兪"))
    ("いろ"
     (6 . "色")
     (19 . "艶")
     (24 . "艷"))
    ("うお"
     (11 . "魚")
     (15 . "魯魴")
     (16 . "鮓鮎鮃鮒鮑鮗鮖")
     (17 . "鮟鮪鮠鮭鮫鮨鮮鮴")
     (18 . "鯀鯊鮹鯉鯆鯏鯑鯒")
     (19 . "鯣鯨鯢鯤鯔鯖鯛鯡鯱鯰鯲鯵")
     (20 . "鰕鰉鰓鰌鰍鰆鰈鰒鰊鰔鰄鰛鰐")
     (21 . "鰮鰥鰭鰤鰯鰡鰰")
     (22 . "鰹鰺鰻鰾鱈鱆鱇鰲")
     (23 . "鱒鱗鱚")
     (24 . "鱠鱧")
     (26 . "鱶")
     (27 . "鱸"))
    ("うかんむり"
     (3 . "宀")
     (5 . "它")
     (6 . "安宇守宅")
     (7 . "完宏宋宍")
     (8 . "宛官宜実宗宙定宕宝")
     (9 . "客宦室宣宥")
     (10 . "宴家害宮宰宵宸容")
     (11 . "寃寅寄寂宿密寇")
     (12 . "寒寓寔寐富")
     (13 . "寛寝")
     (14 . "實寢寡寤察寧寞寥寨")
     (15 . "寫審寮")
     (16 . "寰")
     (19 . "寳寵")
     (20 . "寶"))
    ("うけばこ"
     (2 . "凵")
     (4 . "凶")
     (5 . "凹出凸")
     (8 . "函")
     (9 . "凾"))
    ("うし"
     (4 . "牛")
     (6 . "牝牟")
     (7 . "牡牢")
     (8 . "牧物")
     (9 . "牲牴")
     (10 . "特")
     (11 . "牾牽犁")
     (12 . "犀犇犂")
     (14 . "犒犖")
     (17 . "犠")
     (19 . "犢")
     (20 . "犧"))
    ("うじ"
     (4 . "氏")
     (5 . "民")
     (8 . "氓"))
    ("うす"
     (6 . "臼")
     (9 . "臾舁")
     (11 . "舂")
     (13 . "舅")
     (14 . "與")
     (16 . "興")
     (17 . "舉")
     (18 . "舊"))
    ("うまれる"
     (5 . "生")
     (11 . "産")
     (12 . "甥甦"))
    ("うま"
     (10 . "馬")
     (12 . "馭馮")
     (13 . "馴馳")
     (14 . "駅駆駄駁馼")
     (15 . "駈駕駒駟駛駝駘駐駑")
     (16 . "駭駮駱駲")
     (17 . "駻駿駸騁")
     (18 . "騎騏験騅騒駢")
     (19 . "騙騨")
     (20 . "騷騫騰")
     (21 . "驅驂驀驃騾")
     (22 . "驚驕驍")
     (23 . "驛驗")
     (24 . "驟")
     (26 . "驢")
     (27 . "驥驤")
     (28 . "驩")
     (29 . "驪")
     (30 . "驫"))
    ("うり"
     (6 . "瓜")
     (11 . "瓠")
     (16 . "瓢")
     (19 . "瓣"))
    ("えんがまえ"
     (2 . "冂")
     (4 . "内円")
     (5 . "冊册冉囘")
     (6 . "再")
     (7 . "冏")
     (9 . "冑冒")
     (10 . "冓")
     (11 . "冕"))
    ("えんにょう"
     (3 . "廴")
     (7 . "廷")
     (8 . "延廸")
     (9 . "廻建廼"))
    ("おい"
     (6 . "考老")
     (8 . "者")
     (10 . "耆耄")
     (12 . "耋"))
    ("おおがい"
     (9 . "頁")
     (11 . "頃頂")
     (12 . "項須順")
     (13 . "頑頏頌頓頒預")
     (14 . "頗領頚")
     (15 . "頤頡")
     (16 . "頼頷頬頸頽頭")
     (17 . "顆頻")
     (18 . "額顎顔顏顕顋題類")
     (19 . "願顛")
     (21 . "顧")
     (22 . "顫")
     (23 . "顯")
     (24 . "顰")
     (25 . "顱")
     (27 . "顴顳"))
    ("おつ"
     (1 . "乙")
     (2 . "九")
     (3 . "乞也")
     (7 . "乱")
     (8 . "乳")
     (11 . "乾亀")
     (13 . "亂"))
    ("おと"
     (9 . "音")
     (13 . "韵")
     (14 . "韶")
     (19 . "韻")
     (20 . "響"))
    ("おに"
     (10 . "鬼")
     (14 . "魁魂")
     (15 . "魄魃魅")
     (17 . "醜")
     (18 . "魏魍魎")
     (21 . "魑魔")
     (24 . "魘"))
    ("おのれ"
     (3 . "已己巳")
     (4 . "巴")
     (7 . "巵")
     (9 . "巻巷")
     (12 . "巽"))
    ("おんな"
     (3 . "女")
     (5 . "奴")
     (6 . "如奸好妁妃妄")
     (7 . "妓妝妥妊妣妨妙妖妍")
     (8 . "侫委姑妻始姉妾姓姐妲妬姆妹")
     (9 . "姙娃威姨姻姦姜姿姪姥姚姶")
     (10 . "娚娥姫娟娯娑娘娠娜娉娩")
     (11 . "婀婉婚娵娶娼婆婢婦婁婪婬")
     (12 . "媛婿媒媚")
     (13 . "媼嫁嫌媾嫉嫋嫂媽嫐")
     (14 . "嫦嫣嫗嫡嫩嫖")
     (15 . "嫺嫻嬉嬌嬋")
     (16 . "嬢嬖")
     (17 . "嬰嬬嬲嬪嬶")
     (19 . "嬾")
     (20 . "孃孅孀"))
    ("かい"
     (7 . "貝")
     (9 . "負")
     (10 . "貢財")
     (11 . "貨貫責貪販貧貮貭")
     (12 . "貽賀貴貲貰貸貯貼貳買費貶貿賁")
     (13 . "賊賈資賃賂賄賎賍")
     (14 . "賑")
     (15 . "賛賜賞賤質賣賠賓賦賚")
     (16 . "賢賭")
     (17 . "購賽賺賻")
     (18 . "贄贅贈贇")
     (19 . "贊贋")
     (20 . "贏贍")
     (21 . "贐贓贔")
     (22 . "贖"))
    ("かくしがまえ"
     (2 . "匸")
     (4 . "区匹")
     (7 . "医")
     (10 . "匿")
     (11 . "區"))
    ("かくのかわ"
     (9 . "革")
     (12 . "靫靱靭")
     (13 . "靴靹")
     (14 . "鞅靼鞄靺鞆鞁")
     (15 . "鞋鞍鞏鞐")
     (16 . "鞘")
     (17 . "鞠鞜")
     (18 . "鞫鞦鞣鞭鞨")
     (19 . "鞴鞳")
     (22 . "韃")
     (24 . "韈韆"))
    ("かく"
     (10 . "鬲")
     (22 . "鬻"))
    ("かける"
     (4 . "欠")
     (6 . "次")
     (8 . "欧欣")
     (11 . "欸欷欲")
     (12 . "欹款欺欽")
     (13 . "歇歃")
     (14 . "歌歉")
     (15 . "歐歓歎")
     (16 . "歙歔")
     (17 . "歛")
     (18 . "歟")
     (22 . "歡"))
    ("かぜ"
     (9 . "風")
     (12 . "颪")
     (14 . "颯颱")
     (17 . "颶")
     (20 . "飄飃")
     (21 . "飆"))
    ("かたな"
     (2 . "刀")
     (3 . "刃刄")
     (4 . "刈切分")
     (5 . "刊刋")
     (6 . "刑刎列刔")
     (7 . "刪初判別利刧")
     (8 . "刮券刳刻刷刺制刹到")
     (9 . "剏剄剋削前則剃剌")
     (10 . "剞剣剛剤剔剥剖")
     (11 . "剱剰剩剪副")
     (12 . "剴割創剳")
     (13 . "剿剽")
     (14 . "劃")
     (15 . "劍劇劈劉")
     (16 . "劒劔劑"))
    ("かた"
     (4 . "片")
     (8 . "版")
     (12 . "牌牋")
     (13 . "牒")
     (17 . "牆")
     (19 . "牘"))
    ("かなえ"
     (13 . "鼎"))
    ("かね"
     (8 . "金")
     (10 . "釖針釘釜釡釛釟")
     (11 . "釼釦釵釧釣釶")
     (12 . "鈞釿鈔鈍鈑鈎鈬鈩")
     (13 . "鉞鉛鉗鉅鉉鈷鉱鉤鉈鉦鉄鈿鉢鉋鈴鉐鉚")
     (14 . "銕銜銀銖銃銭銑銓銛銅鉾銘銚")
     (15 . "鋪鋭鋏銹鋤銷鋳鋒鋲鋩")
     (16 . "錦錢錏鋺鋸錮鋼錯錙錠錐錆錫錚錘錣錬録錺錵錻鍄")
     (17 . "鍋鍔鍵鍠鍬鍾鍼鍛鍮鍍錨鍖鍜")
     (18 . "鎰鎧鎬鎖鎗鎮鎭鎚鎔鎌鎹")
     (19 . "鏥鏖鏡鏗鏨鏘鏃鏑鏝鏐鏈鏤")
     (20 . "鐡鐚鐘鐔鐓鐙鐃鐐鐇")
     (21 . "鐵鐶鐫鐸鐺")
     (22 . "鑄鑓鑁鑒")
     (23 . "鑛鑑鑠鑢鑞鑚")
     (24 . "鑪")
     (25 . "鑰")
     (26 . "鑵鑷")
     (27 . "鑽鑼鑾鑿")
     (28 . "钁"))
    ("かのほこ"
     (4 . "戈")
     (5 . "戉戊")
     (6 . "戍戎戌成")
     (7 . "我戒")
     (8 . "或戔")
     (11 . "戛戚戝")
     (12 . "戞戟")
     (13 . "戡戦")
     (14 . "截")
     (15 . "戯戮")
     (16 . "戰")
     (17 . "戲")
     (18 . "戴戳"))
    ("かみがしら"
     (10 . "髟")
     (13 . "髢")
     (14 . "髪髣髦")
     (15 . "髴髯髮髭髫髱")
     (16 . "髷髻")
     (18 . "鬆")
     (21 . "鬘")
     (22 . "鬚")
     (23 . "鬟")
     (24 . "鬢")
     (25 . "鬣"))
    ("かめ"
     (16 . "龜"))
    ("からい"
     (7 . "辛")
     (12 . "辜")
     (13 . "辟")
     (14 . "辣")
     (16 . "辨辧")
     (19 . "辭")
     (21 . "辯"))
    ("かわら"
     (5 . "瓦")
     (7 . "瓧")
     (8 . "瓩")
     (9 . "瓮瓲瓱瓰")
     (11 . "瓷瓶瓸")
     (14 . "甄甃甅")
     (16 . "甌甎甍")
     (17 . "甑")
     (18 . "甕甓"))
    ("がんだれ"
     (2 . "厂")
     (4 . "厄")
     (9 . "厚厖厘")
     (10 . "原")
     (11 . "厠")
     (12 . "厥厦厩厨")
     (14 . "厭厮厰")
     (17 . "厳"))
    ("きいろ"
     (11 . "黄")
     (25 . "黌"))
    ("きがまえ"
     (4 . "气")
     (6 . "気")
     (8 . "氛")
     (10 . "氣氤"))
    ("きば"
     (4 . "牙"))
    ("きび"
     (12 . "黍")
     (15 . "黎")
     (17 . "黏")
     (23 . "黐"))
    ("きん"
     (4 . "斤")
     (5 . "斥")
     (8 . "斧")
     (9 . "斫")
     (11 . "斬断")
     (12 . "斯")
     (13 . "新")
     (18 . "斷"))
    ("き"
     (4 . "木")
     (5 . "札朮本末未")
     (6 . "机朽朿朱朶朴朸杁朷")
     (7 . "来杆杏杞材杉条杖束村杜杓杙李杣杢杤杠")
     (8 . "杲杳枉果杭枝杵松枩枢析杼枕東杯板枇杪枌枋枚林枡枠杰枅枦杷")
     (9 . "柆栄架枷柯柑柬枳柩枸枯査柴柵柞柿柘柊柁柝柱柢柔某柏柎柄枹柚柳\
栂栃柾柤柧柮枴")
     (10 . "栢案桜格核桓栞桔框栩桂桀校桁栲根栽桟桎株栓栴桑桃桐梅栗栫桍桙\
栖桧档")
     (11 . "桿條桝梳械桷梟梧梗梏梱梭梓梔梢梛梯梃桶梶桴梵梨梠梁梺桾梍梼梹")
     (12 . "椄棧椅椁棺棋棊棘検棍棕椒植森棲棗椎棣棟棹棠棚棒棉椋椀椚椙椛棆\
棔棡棯椈椌椏椡椪椣椥椨椦椢")
     (13 . "楳椶楹楷楽棄業極楸楫楢楯楔楚楮椿椹椽楠楓椰楡楊楝楼椴楙楜楪楴\
楾榁楕楞")
     (14 . "榮榔榎槐概構槁槓榾槎槊様榛榱槍槌槙槇榻槃榧榑模榜榕榴榊榠榲榿\
樮槝槞")
     (15 . "樞槨樂樓樣横槻樛槿権槲槧樟樅槽樗樋樊標樒樫槫槭槹樔樌樢")
     (16 . "樺橄機橘橋樹樵橡橇樽橢橙橦橈樸樶橲橸")
     (17 . "檢檐橿檠檄檎檣檀檗檍檪檜")
     (18 . "櫁檻櫃櫂檮檸檳檬")
     (19 . "櫛櫚櫟櫓櫞櫑")
     (20 . "欄櫪櫨")
     (21 . "櫻欅櫺")
     (22 . "權")
     (23 . "欒")
     (25 . "欖")
     (26 . "欝")
     (29 . "欟"))
    ("ぎふのふ"
     (6 . "阡")
     (7 . "阯阪阨阮防")
     (8 . "阜陂阿阻陀附")
     (9 . "限陌陋陏")
     (10 . "院陥降除陞陣陝陟陛陜陦")
     (11 . "陷陰険陲陬陳陶陪陸隆陵")
     (12 . "隋随階隅隍隊陽隈")
     (13 . "隘隕隗隔")
     (14 . "隙隠際障")
     (16 . "險隧隣")
     (17 . "隱隰隲")
     (19 . "隴"))
    ("ぎょうにんべん"
     (3 . "彳")
     (7 . "役彷")
     (8 . "彿往徃径征徂彼")
     (9 . "後很徇待律徊")
     (10 . "徑従徐徒")
     (11 . "徠從徙得徘")
     (12 . "御徨循復")
     (13 . "微徭")
     (14 . "徴徳")
     (15 . "徹")
     (16 . "徼")
     (17 . "徽"))
    ("ぎょう"
     (6 . "行")
     (9 . "衍")
     (11 . "衒術")
     (12 . "街")
     (13 . "衙")
     (15 . "衝")
     (16 . "衛衞衡")
     (24 . "衢"))
    ("くさ"
     (6 . "艸艾芋")
     (7 . "芍芒花芸芝芙芳")
     (8 . "苅芥芹芫芟芯芭芬芦若英苑芽苦茎苗茉茂")
     (9 . "苛茄苣苟苫苒苴苔苧苳范苻苹苞茅茆苜苓苙苡苺荒茜草荘茶")
     (10 . "芻茵茴荊茨茲茱荀茹茸荏荐荅茯茫茗茘茖荷華莉")
     (11 . "莅莖莊莚莪莞莟莢莎荵荻荼莫莓莠莨莇茣莽莵荳菓菊菌菜著菩萌")
     (12 . "菷菴萎菅菫菰菎菽菖萃菘萋菁萇菟菠菲萍萠莱菱菻萄萓萢萬葵萩葬葉落")
     (13 . "葦葭萼葛葷萱葫葺葱董葩葡葆葯葎萪蒄萵萸葮葹葢蒂蒸蓄蓉")
     (14 . "蒭蓊蓋蒹蒿蒟蓑蒜蓍蒔蒻蒐蓐蓁蓆蒼蓖蒲蒙蓙蒡蓚蔦蔵")
     (15 . "蔔蔆蔭蔚蔡蓴蔗蒋蔘蔬蔟蔕蔓蔑蓬蔀蓼蓮蓿")
     (16 . "蕩蕚蕎蕀蕨蕣蕉蕘蕈蕁蕊蕋蕃蕪蔽蕕蕭蕗薫薪薦薄薬")
     (17 . "薗薤薈薑薊薨薔薛薙薇薜蕾蕷薐薮薀")
     (18 . "藁藏薩藉薯薺薹藐藤藩藍")
     (19 . "藝藥藕藪藜藻蘭")
     (20 . "蘂藷藹蘊蘇蘓蘋藺蘆蘢藾")
     (21 . "蘗蘖蘯蘚蘰")
     (23 . "蘿"))
    ("くち"
     (3 . "口")
     (5 . "叶叩可句古号司史只叱召台叮叨右叺叭")
     (6 . "吉吃叫吁向后合吊吐吋同名吏各")
     (7 . "吸吽呀含听吟君呉吾吭吼告吹吮呈吶呑吠否吻吩呆呂吝呎")
     (8 . "咏呵咎呟呼呱呷咋呰呪咒周呻咀知呶咄咆味命和咐")
     (9 . "哀哇咽咳咢咸咥咬哄哈哉咨咫咲哂咤品咼咾哘咯")
     (10 . "唔員哥唏哽哮哭唆哨唇哲唐唄哺哩哢哦唖")
     (11 . "啀啓售商唱啜唾啅啄啖啗唸問唯唳唹啌啝啣")
     (12 . "喝営喙喀喚喊喜喟喫喬喧喉啻啾善喘喪喞單喋啼喃喩喇喰喨")
     (13 . "嗚嗅嗟嗄嗣嗜嗤嗔嘆")
     (14 . "嘔嘉嗷嘖嘗嗾嗽嗹嘛")
     (15 . "噎器噐嘘嘴嘱嘶噌噂嘲嘸噴噛嘩")
     (16 . "噫噤嘯噬噸噺噪")
     (17 . "嚇嚆嚔嚏嚊嚀")
     (18 . "嚠")
     (19 . "嚥嚮")
     (20 . "嚶嚴")
     (21 . "囂嚼囁囃囀")
     (22 . "囈嚢囎")
     (24 . "囑囓"))
    ("くにがまえ"
     (3 . "囗")
     (5 . "囚四")
     (6 . "因回団")
     (7 . "囲囮困図")
     (8 . "固国囹")
     (9 . "圀囿")
     (10 . "圄圃")
     (11 . "國圉圈")
     (12 . "圍圏")
     (13 . "圓園嗇")
     (14 . "團圖")
     (16 . "圜"))
    ("くび"
     (9 . "首")
     (11 . "馗")
     (17 . "馘"))
    ("くらべるひ"
     (4 . "比")
     (9 . "毘"))
    ("くるま"
     (7 . "車")
     (8 . "軋")
     (9 . "軌軍")
     (10 . "軒")
     (11 . "軛転軟軣")
     (12 . "軼軻軽軸軫")
     (13 . "較載軾輊輅輌")
     (14 . "輕輒輓輔")
     (15 . "輙輝輜輟輩輛輪輦")
     (16 . "輯輳輻輸輹")
     (17 . "轅轄轂輾輿")
     (18 . "轉轆轌")
     (19 . "轎轍")
     (20 . "轗")
     (21 . "轟轜")
     (22 . "轡轢")
     (23 . "轣轤"))
    ("くろ"
     (11 . "黒")
     (15 . "黙")
     (16 . "默黔")
     (17 . "黛黜點黝")
     (18 . "黠")
     (20 . "黨黥")
     (21 . "黯")
     (23 . "黴")
     (26 . "黶")
     (27 . "黷"))
    ("ぐうのあし"
     (9 . "禹禺")
     (13 . "禽"))
    ("けいがしら"
     (3 . "彑")
     (6 . "当")
     (9 . "彖")
     (11 . "彗")
     (13 . "彙")
     (16 . "彜")
     (18 . "彝"))
    ("け"
     (4 . "毛")
     (8 . "毟")
     (11 . "毬毫")
     (12 . "毳毯")
     (17 . "氈"))
    ("げん"
     (5 . "玄")
     (11 . "率"))
    ("こころ"
     (4 . "心")
     (5 . "必")
     (6 . "忖忙")
     (7 . "応忌志忍忘快忻忤忸忱忰")
     (8 . "忽忠忝念忿怐怡怪怯怙性怛怕怖怫怦怏怜怺怩")
     (9 . "怨急思怎怠怒怱恊恠悔恢恪恟恒恆恰恍恨恃恤恂恬恫")
     (10 . "恚恁恩恐恭恵恣恕息恥恙恋恷悧悦悁悍悟悃悚悄悛悌悩悖悒悋悗")
     (11 . "悪患悉悠惟悸惓惚惨情悴悽惜惆悵悼惇惘惧")
     (12 . "悳惠惣悲悶惑惱愕慌惶愀惴惺惻惰愎愉愃愡")
     (13 . "惡愛意感愚愆慈惹愁惷想愍愈慍慨愧愾慊慎愼愴慄愽戀")
     (14 . "慇愨愿愬態慕慂慘慣慳慷慚慴慥慟慓慢慵慯慱")
     (15 . "慰慶慧慙慫慝憂慾慮憇憎憬憔憚憧憫憮憤憐")
     (16 . "憙憖憩憲憊憑懌懊憶懐懈憾懆憺懍")
     (17 . "應懃懇懋懦")
     (18 . "懲懣懴")
     (19 . "懷懶")
     (20 . "懸懺")
     (21 . "懽懼懾")
     (22 . "懿"))
    ("ことば"
     (7 . "言")
     (9 . "計訂訃")
     (10 . "記訖訓訐訌訊託討")
     (11 . "設訛訝許訣訟訥訪訳")
     (12 . "詠訶詁詐詞証詔診訴詛詑詒註詆評詈")
     (13 . "詼該詭詰詣誇詬詩試詢詳誠詫誅誂誉誄話詮")
     (14 . "誨誡誑語誤誥誌誦誚誓説読認誣誘")
     (15 . "謁課誼諏諄諸誰請諍諾誕談調諂誹諒論諚諫")
     (16 . "諤諳謂諧諌諱謔諠諺諢諮諜諦諷諞謀諭諛謡諡")
     (17 . "謌謠謹謙謇講謚謝謖謄謐謗謎")
     (18 . "謳謦謫謾謬謨")
     (19 . "譌證譁譏譎警識譖譛譚譜")
     (20 . "譯議護譲譫譟譬")
     (21 . "譽譴")
     (22 . "讀讃")
     (23 . "讌讎讐")
     (24 . "讓讒讖")
     (25 . "讙")
     (26 . "讚"))
    ("こめ"
     (6 . "米")
     (8 . "籵")
     (9 . "粁粂籾")
     (10 . "粃粋粉粍粐")
     (11 . "粗粘粕粒")
     (12 . "粤粢粥粧粟粡粨粭粫")
     (13 . "粳粲粱粮")
     (14 . "粹精糀粽")
     (15 . "糊糅糎糂")
     (16 . "糖糒糘")
     (17 . "糢糠糟糜糞")
     (18 . "糧")
     (20 . "糯")
     (21 . "糲")
     (22 . "糴")
     (25 . "糶"))
    ("ころも"
     (6 . "衣")
     (8 . "衫表")
     (9 . "衿衵衽衷衲袂")
     (10 . "袁衾袞衰袖袗袒袙袢被袍")
     (11 . "袈袋袤袮袰袵袿袷袱裃裄裳")
     (12 . "裁装裂裙補裕裡")
     (13 . "裝裔裘裏裟褂裾裼裨裸裲褄")
     (14 . "裹製裴褐褌複褊褓褝")
     (15 . "褒褞褥褪褫")
     (16 . "褸襁褶")
     (17 . "襃襄褻襌襍")
     (18 . "襖襟襠")
     (19 . "襞襦襤")
     (20 . "襭襪")
     (21 . "襯")
     (22 . "襲襴襷"))
    ("こん"
     (6 . "艮")
     (7 . "良")
     (17 . "艱"))
    ("こ"
     (3 . "孑子")
     (4 . "孔")
     (5 . "孕")
     (6 . "字存")
     (7 . "孝孚孛")
     (8 . "学孥孟孤季")
     (9 . "孩")
     (10 . "孫")
     (11 . "孰")
     (12 . "孱")
     (13 . "孳")
     (14 . "孵")
     (16 . "學")
     (17 . "孺"))
    ("さけのとり"
     (7 . "酉")
     (9 . "酋酊")
     (10 . "酌酒酎配")
     (11 . "酔酖酘")
     (12 . "酣酢酥鈕")
     (13 . "酬酩酪")
     (14 . "酳酵酷酸酲")
     (15 . "醉醋醇醂")
     (16 . "醒醍醐醗")
     (17 . "醢")
     (18 . "醫醯醤醪")
     (20 . "醵醸醴")
     (21 . "醺")
     (24 . "釀")
     (25 . "釁"))
    ("さじのひ"
     (2 . "匕")
     (4 . "化")
     (5 . "北")
     (11 . "匙"))
    ("さと"
     (7 . "里")
     (9 . "重")
     (11 . "野")
     (12 . "量")
     (18 . "釐"))
    ("さむらい"
     (3 . "士")
     (4 . "壬")
     (6 . "壮")
     (7 . "壯壱声売")
     (11 . "壷")
     (12 . "壹壺壻")
     (13 . "壼")
     (14 . "壽"))
    ("さら"
     (5 . "皿")
     (8 . "盂")
     (9 . "盃盈盆")
     (10 . "益盍")
     (11 . "盗盒盛盖")
     (12 . "盜")
     (13 . "盞盟")
     (14 . "盡")
     (15 . "監盤")
     (16 . "盥盧")
     (17 . "盪"))
    ("さんづくり"
     (3 . "彡")
     (7 . "形")
     (9 . "彦")
     (11 . "彩彫彪彬")
     (12 . "彭")
     (14 . "彰")
     (15 . "影"))
    ("しかして"
     (6 . "而")
     (9 . "耐"))
    ("しかばね"
     (3 . "尸")
     (4 . "尹尺")
     (5 . "尻尼")
     (6 . "尽")
     (7 . "局尿屁尾")
     (8 . "届屆居屈")
     (9 . "屋屍屎屏昼")
     (10 . "屐屑展屓")
     (12 . "属屠屡")
     (14 . "層")
     (15 . "履")
     (21 . "屬"))
    ("しか"
     (11 . "鹿")
     (13 . "麁")
     (16 . "麈")
     (17 . "麋")
     (18 . "麌")
     (19 . "麓麕麒麑麗")
     (21 . "麝")
     (23 . "麟"))
    ("しきがまえ"
     (3 . "弋")
     (4 . "弌")
     (5 . "弍")
     (6 . "式弐")
     (12 . "弑"))
    ("した"
     (6 . "舌")
     (8 . "舎舍")
     (10 . "舐")
     (12 . "舒")
     (13 . "辞")
     (15 . "舗舖")
     (16 . "舘"))
    ("しめす"
     (5 . "示礼")
     (7 . "社")
     (8 . "祀祁祈祉")
     (9 . "祇祝神祖祐")
     (10 . "祠祗祟祚祕祓祢祥")
     (11 . "祭票祷袴")
     (12 . "禄")
     (13 . "祺禁祿禀禍禅禎福")
     (14 . "禊")
     (15 . "禝")
     (16 . "禦頴")
     (17 . "禪禧")
     (18 . "禮")
     (19 . "禰")
     (22 . "禳"))
    ("しょうへん"
     (4 . "爿")
     (8 . "牀"))
    ("しょく"
     (9 . "食")
     (10 . "飢")
     (12 . "飯")
     (13 . "飭飲飮飫飩飾飼飽")
     (14 . "飴")
     (15 . "餃餌餉養餅餓")
     (16 . "餘餐餒餔館")
     (17 . "餝餡餞餤餠")
     (18 . "餬餮")
     (19 . "餽餾饂")
     (20 . "饉饅")
     (21 . "饐饋饑饒饌")
     (22 . "饗饕"))
    ("しろ"
     (5 . "白")
     (6 . "百")
     (7 . "皀皃")
     (8 . "的")
     (9 . "皈皆皇")
     (10 . "皋")
     (11 . "皐皎")
     (12 . "皖皓")
     (13 . "皙")
     (15 . "皚"))
    ("しんにゅう"
     (5 . "辷辺込")
     (6 . "辻迅")
     (7 . "迂迄辿近迎返")
     (8 . "迚述迭迫")
     (9 . "迦迥迢迪迯迩邇逆送退追逃迷")
     (10 . "迹迴迺迸逅逝造速逐通逓途透連")
     (11 . "逕逑這逡逍逞逖逗逋逢逎逸週進逮")
     (12 . "逶逵逹運過遇遂達遅道遍遊遥")
     (13 . "遏遐遑遒遉遁逼逾遖違遠遣")
     (14 . "逧遡遞遙遘遜遭適")
     (15 . "遨遮遯遺遵選遷遼")
     (16 . "隨遲遶還避")
     (17 . "邂遽邁邀")
     (18 . "邃")
     (19 . "邊")
     (21 . "邉")
     (23 . "邏"))
    ("しんのたつ"
     (7 . "辰")
     (10 . "辱")
     (13 . "農"))
    ("しん"
     (7 . "臣")
     (8 . "臥")
     (14 . "臧")
     (18 . "臨"))
    ("じゅうまた"
     (4 . "支"))
    ("じゅう"
     (2 . "十")
     (3 . "千")
     (4 . "午升卅卆")
     (5 . "卉半")
     (6 . "卍")
     (8 . "協卒卓")
     (9 . "南卑単")
     (12 . "博"))
    ("すでのつくり"
     (4 . "旡无")
     (10 . "既"))
    ("すん"
     (3 . "寸")
     (6 . "寺")
     (7 . "寿対")
     (9 . "専封")
     (10 . "尅射将")
     (11 . "專將尉")
     (12 . "尋尊")
     (14 . "對")
     (15 . "導"))
    ("せい"
     (14 . "齊")
     (17 . "齋")
     (21 . "齎")
     (23 . "齏"))
    ("たかい"
     (10 . "高")
     (23 . "髞"))
    ("たくみ"
     (3 . "工")
     (5 . "巨巧左")
     (7 . "巫")
     (10 . "差"))
    ("たけ"
     (6 . "竹")
     (8 . "竺")
     (9 . "竿笂")
     (10 . "笈笏笑笊笆笋")
     (11 . "笳笥笘笙第笞笛符笨笠笹笶笵")
     (12 . "筈筐筋笄策筍筌筅筑等答筒筏筆筝")
     (13 . "筺筵筥筴筧筮節筬筰筱")
     (14 . "篏箇管箝箕箘箟箍箜箚算箋箒箏箔箙箆")
     (15 . "篋篁箱箴箭箸篆篇篌箪範")
     (16 . "築篝篩篦篥篭簑篤")
     (17 . "簒簀篠簇篳篷篶簍簓簗簔")
     (18 . "簡簣簧簪簟")
     (19 . "簷簫簽簸簿簾籀")
     (20 . "籍籌籃籏")
     (21 . "籐籖籔")
     (22 . "籘籟籠")
     (23 . "籤籥")
     (25 . "籬"))
    ("たつ"
     (5 . "立")
     (7 . "竍")
     (8 . "竏")
     (9 . "竒竓竕")
     (10 . "竚站竝竜")
     (11 . "章竡竟")
     (12 . "竢竣竦童")
     (13 . "竪")
     (14 . "竭端竰")
     (20 . "競")
     (22 . "竸"))
    ("たに"
     (7 . "谷")
     (11 . "谺")
     (17 . "豁谿"))
    ("たま"
     (4 . "王")
     (5 . "玉")
     (7 . "玖")
     (8 . "玩")
     (9 . "珂珈珊珍珎玻玲珀玳")
     (10 . "珪珥珠珮班琉珞珱")
     (11 . "球現理琅珸")
     (12 . "琢琴琥琵琺琳琲琶")
     (13 . "瑛瑕瑚瑟瑞瑁瑜瑶琿瑙")
     (14 . "瑯瑤瑰瑣瑳瑪瑠")
     (15 . "瑩瑾璋璃")
     (16 . "璢璞")
     (17 . "環")
     (18 . "璧")
     (19 . "璽瓊")
     (20 . "瓏")
     (21 . "瓔"))
    ("た"
     (5 . "甲申田由")
     (7 . "甸男町甼")
     (8 . "画畄")
     (9 . "畋畏界畍畑畉畆畊")
     (10 . "畛畜畔畝畚留畠")
     (11 . "異畦畤畢略畧畩")
     (12 . "畫畳番畭畴")
     (13 . "畸畷當")
     (15 . "畿")
     (16 . "疂")
     (19 . "疆疇")
     (22 . "疊疉"))
    ("だい"
     (3 . "大")
     (4 . "夬太天夫夭")
     (5 . "央失夲")
     (6 . "夷夸")
     (7 . "夾")
     (8 . "奄奇奈奉奔")
     (9 . "奕奐奎契奏")
     (10 . "奚奘套")
     (12 . "奥奢奠")
     (13 . "奧奨")
     (14 . "奩奬奪")
     (16 . "奮"))
    ("ちいさい"
     (3 . "小")
     (4 . "少")
     (5 . "尓")
     (6 . "尖")
     (8 . "尚")
     (13 . "尠"))
    ("ちから"
     (2 . "力")
     (5 . "加功")
     (6 . "劣")
     (7 . "劬劫助劭努励労")
     (8 . "劵劾効劼")
     (9 . "勁勅勃勇")
     (10 . "勍勉")
     (11 . "勘動勗務勒")
     (12 . "勞勤勝募")
     (13 . "勧勢勦勠勣")
     (15 . "勲")
     (16 . "勳")
     (17 . "勵")
     (20 . "勸"))
    ("ちち"
     (4 . "父")
     (13 . "爺"))
    ("ちょう"
     (10 . "鬯")
     (29 . "鬱"))
    ("ち"
     (6 . "血")
     (9 . "衂")
     (10 . "衄")
     (12 . "衆"))
    ("つき"
     (4 . "月")
     (6 . "有")
     (8 . "服朋")
     (9 . "朏")
     (10 . "朔朕朗")
     (11 . "朖望")
     (12 . "期朞朝")
     (18 . "朦")
     (20 . "朧"))
    ("つくえ"
     (2 . "几")
     (3 . "凡")
     (5 . "処凧")
     (6 . "凩凪")
     (8 . "凭")
     (11 . "凰")
     (12 . "凱"))
    ("つち"
     (3 . "土")
     (5 . "圧圦")
     (6 . "圭在地圷圸")
     (7 . "坎圻均坑坐址坏坂坊")
     (8 . "坩坤垂坦坡坿坪垉尭")
     (9 . "垈垣垓垠型垢城垤垪垰垳")
     (10 . "埖埀埃埋埒埓埔埆")
     (11 . "堊域基埼堀執埴堆堂培埠堋埣埜")
     (12 . "堙堰堝堪堅場堕塚堤堵塔報堡塁塀堽堺堯")
     (13 . "塋塊塙塒塑塞填塗塘墓塰塢塩")
     (14 . "塲境塹塾墅塵増墨")
     (15 . "墮墟墜墳墫墹墸")
     (16 . "墺壊墾壌壇壁壅")
     (17 . "墻壓壑壕壗")
     (18 . "壘壙")
     (19 . "壞壜壟")
     (20 . "壤壥"))
    ("つつみがまえ"
     (2 . "勹")
     (3 . "勺")
     (4 . "勾勿匂匁")
     (5 . "匆包")
     (6 . "匈")
     (9 . "匍")
     (11 . "匏匐"))
    ("つづみ"
     (13 . "鼓")
     (14 . "皷")
     (18 . "鼕"))
    ("つの"
     (7 . "角")
     (12 . "觚觜觝")
     (13 . "解触")
     (15 . "觧")
     (18 . "觴")
     (20 . "觸"))
    ("つめ"
     (4 . "爪")
     (8 . "爭爬")
     (9 . "爰")
     (12 . "爲")
     (17 . "爵"))
    ("てつ"
     (3 . "屮")
     (4 . "屯"))
    ("てん"
     (1 . "丶")
     (3 . "丸")
     (4 . "丹")
     (5 . "主丼"))
    ("て"
     (3 . "才")
     (4 . "手扎")
     (5 . "打払")
     (6 . "扞扣扛扠扱托扨")
     (7 . "扼技抉抗抒抄抓択投抖把抜批扶扮抃抔抑找折抂")
     (8 . "承抻拂拔押拗拐拡拑拒拠拘招拙拓拆担抽抵拈拝拍拌披拊拇抱抛抹拉抬")
     (9 . "拏拜按挌括拮挟拱挂拷拶指持拾拯拭拵挑挧")
     (10 . "拿挙挈拳捍挾挨捐挫振挺捜挿捉捏捌挽捕捗")
     (11 . "掖掩掛掎掬据掀掘掲捲控採捨授捷推捶接措掃探掏掉掟捺捻排捧捫掠\
捩掫掵掴")
     (12 . "掌掣插描握援掾揩換揮揆揣揉揃提揶揄揖揚揺揀")
     (13 . "搜搭搖携搆搓搦搶摂掻損搗搨搏搬搾")
     (14 . "搴摧摺摶摘摸摎")
     (15 . "撃摯摩撮撰撒撞撤撓撚播撥撫撲撩撈撕撹")
     (16 . "擇據擔撼擒擅操撻擁擂")
     (17 . "擘擱擬擦擠擡擢擣擯")
     (18 . "擧擴擾擲擺擽擶攅")
     (19 . "攀")
     (20 . "攘")
     (21 . "攜攝")
     (22 . "攤")
     (23 . "攣攪攫")
     (24 . "攬"))
    ("とうがまえ"
     (10 . "鬥")
     (15 . "鬧")
     (16 . "鬨")
     (18 . "鬩")
     (20 . "鬪")
     (26 . "鬮"))
    ("とびらのと"
     (4 . "戸")
     (7 . "戻")
     (8 . "房所")
     (9 . "扁")
     (10 . "扇")
     (11 . "扈")
     (12 . "扉"))
    ("とぶ"
     (9 . "飛")
     (21 . "飜"))
    ("とます"
     (4 . "斗")
     (10 . "料")
     (11 . "斛斜")
     (13 . "斟")
     (14 . "斡"))
    ("とまた"
     (4 . "攴攵")
     (6 . "收攷")
     (7 . "改攻孜攸")
     (8 . "放")
     (9 . "故政")
     (10 . "效敏")
     (11 . "赦敕敍敘救教敖敗")
     (12 . "敢敬散敞敦敝")
     (13 . "数")
     (14 . "敲")
     (15 . "數敵敷")
     (16 . "整")
     (17 . "斂")
     (18 . "斃")
     (22 . "變"))
    ("とめる"
     (4 . "止")
     (5 . "正此")
     (8 . "武歩")
     (9 . "歪")
     (12 . "歯")
     (13 . "歳")
     (14 . "歴")
     (18 . "歸"))
    ("とらがしら"
     (6 . "虍")
     (8 . "虎")
     (9 . "虐")
     (10 . "虔")
     (11 . "處虚")
     (13 . "號虞虜")
     (17 . "虧"))
    ("とり"
     (9 . "鳬")
     (11 . "鳥")
     (13 . "鳫鳩鳧鳰")
     (14 . "鳶鳳鳴")
     (15 . "鴈鴉鴃鴆鴇")
     (16 . "鴛鴨鴟鴕鴫鴒鴣鴪鴦鴬鴎鶯")
     (17 . "鵄鴻鵁鴿鴾鵆鵈")
     (18 . "鵝鵞鵑鵠鵜鵐鵡鵤鵙")
     (19 . "鶏鵲鶉鶇鵯鵬鵺")
     (20 . "鶚鶤鶩鶫")
     (21 . "鷄鶴鷁鶻鶸鶺鶲鷂鷆鷏")
     (22 . "鷙鷓")
     (23 . "鷸鷲鷦鷭鷯鷺")
     (24 . "鷽鷹")
     (28 . "鸚")
     (29 . "鸛")
     (30 . "鸞"))
    ("なかれ"
     (4 . "毋")
     (5 . "母")
     (6 . "毎")
     (8 . "毒")
     (14 . "毓"))
    ("ながい"
     (8 . "長"))
    ("なべふた"
     (2 . "亠")
     (3 . "亡")
     (4 . "亢")
     (6 . "亦亥交")
     (7 . "亨")
     (8 . "享京")
     (9 . "亰亭亮")
     (10 . "亳")
     (13 . "亶"))
    ("なめしがわ"
     (9 . "韋")
     (17 . "韓")
     (19 . "韜"))
    ("においこう"
     (9 . "香")
     (18 . "馥")
     (20 . "馨"))
    ("にく"
     (6 . "肉肌肋")
     (7 . "肝肛肓肖肘肚")
     (8 . "冐育肯肩股肴肱肢肥肪肬肭")
     (9 . "胃胤胡胛胥胙胎胆胝胄背肺胚胖胞脉")
     (10 . "胸脅脇胯脂脆脊胴能脈胱")
     (11 . "脣脚脛脩脱脳脯")
     (12 . "腋腔腎脹腆脾腓腑胼腕")
     (13 . "腦腱腫腥腸腹腴腰腺腟腮")
     (14 . "腐膃膈膏腿膊膀膜膂")
     (15 . "膓膠膝膣膚膕膤")
     (16 . "膩膳膰膨膵")
     (17 . "膽臆膾臀膿臂膺臉膸臈")
     (18 . "臍臑")
     (19 . "臓臘")
     (20 . "臙臚")
     (22 . "臟")
     (25 . "臠"))
    ("にし"
     (6 . "襾西")
     (9 . "要")
     (12 . "覃")
     (18 . "覆")
     (19 . "覈覇")
     (25 . "覊"))
    ("にじゅうあし"
     (3 . "廾")
     (4 . "廿")
     (5 . "弁")
     (7 . "弄弃")
     (10 . "弉")
     (15 . "弊"))
    ("にすい"
     (2 . "冫")
     (5 . "冬")
     (6 . "冱冰决冲")
     (7 . "冴冶冷况")
     (8 . "冽")
     (10 . "凅准凄凋凍凉")
     (11 . "凌")
     (12 . "凖")
     (15 . "凛")
     (16 . "凝"))
    ("にち"
     (4 . "日")
     (5 . "旦旧")
     (6 . "旭旨旬早")
     (7 . "旱")
     (8 . "易旺昊昆昏昇昌昔昃明旻")
     (9 . "昂映昨昵春昭是星昶昧昴昜昿")
     (10 . "晏晃晄時晋晉晁晒")
     (11 . "晝晦晞晤晧晨晟晢")
     (12 . "暁景暑晶晴晰智晩普暃")
     (13 . "暎暘暗暈暇暉暄暖")
     (14 . "暢暮暝暦")
     (15 . "暫暴")
     (16 . "曉曁暹暾曇曄暼暸")
     (17 . "曖")
     (18 . "曙曚曜")
     (19 . "曠曝")
     (20 . "曦")
     (21 . "曩"))
    ("にら"
     (9 . "韭")
     (13 . "韮")
     (19 . "韲"))
    ("に"
     (2 . "二")
     (3 . "于")
     (4 . "云五互井")
     (6 . "亙亘")
     (7 . "亜些")
     (8 . "亞")
     (9 . "亟"))
    ("ねずみ"
     (8 . "鼡")
     (13 . "鼠")
     (18 . "鼬"))
    ("のぎ"
     (5 . "禾")
     (7 . "私秀禿")
     (8 . "秉")
     (9 . "科秋秕秒")
     (10 . "秘秧秬称秤秦租秩秣秡")
     (11 . "移")
     (12 . "稈稀税稍程")
     (13 . "稔稚稠稗稜稟稘稙")
     (14 . "稱穀種稲")
     (15 . "稻稼稽稿稾稷穂穃")
     (16 . "穐穎穏積穆")
     (17 . "穉穗")
     (18 . "穢穫穣穡")
     (19 . "穩")
     (21 . "龝")
     (22 . "穰"))
    ("のごめ"
     (7 . "釆")
     (8 . "采")
     (11 . "釈")
     (12 . "釉")
     (20 . "釋"))
    ("の"
     (1 . "丿")
     (2 . "乂乃")
     (3 . "久")
     (4 . "之")
     (5 . "乎乍乏")
     (7 . "乕")
     (8 . "乖")
     (9 . "乗")
     (10 . "乘"))
    ("はこがまえ"
     (2 . "匚")
     (6 . "匡匠")
     (7 . "匣")
     (10 . "匪")
     (13 . "匯")
     (14 . "匱")
     (15 . "匳"))
    ("はしる"
     (5 . "赱")
     (7 . "走")
     (9 . "赴")
     (10 . "起赳")
     (12 . "越超趁")
     (14 . "趙")
     (15 . "趣")
     (17 . "趨"))
    ("はち"
     (2 . "八")
     (4 . "兮公六")
     (6 . "共")
     (7 . "兵")
     (8 . "其具典")
     (10 . "兼")
     (16 . "冀"))
    ("はつがしら"
     (5 . "癶")
     (9 . "癸発")
     (12 . "發登"))
    ("はな"
     (14 . "鼻")
     (17 . "鼾"))
    ("はねぼう"
     (1 . "亅")
     (2 . "了")
     (4 . "予")
     (6 . "争")
     (7 "亊"))
    ("はねぼう"
     (8 . "事"))
    ("はね"
     (6 . "羽")
     (10 . "翁翅翆")
     (11 . "習翌翊")
     (12 . "翕翔")
     (14 . "翠翡")
     (15 . "翫翦翩")
     (16 . "翰")
     (17 . "翳翼")
     (18 . "翹翻")
     (20 . "耀"))
    ("はば"
     (3 . "巾")
     (5 . "匝市布")
     (6 . "帆")
     (7 . "希帋")
     (8 . "帖帚帙帑帛")
     (9 . "帥帝")
     (10 . "師席帯帰")
     (11 . "帶帷常帳")
     (12 . "幄幃幀幅帽幇")
     (13 . "幌幕幎")
     (14 . "幗幔")
     (15 . "幟幢幡幣幤"))
    ("は"
     (15 . "齒")
     (17 . "齔齢")
     (20 . "齣齟齠齡")
     (21 . "齦齧")
     (22 . "齪齬")
     (24 . "齷齲齶"))
    ("ひき"
     (5 . "疋")
     (11 . "疏")
     (12 . "疎")
     (14 . "疑"))
    ("ひつじ"
     (6 . "羊")
     (8 . "羌")
     (9 . "美")
     (10 . "羔")
     (11 . "羞羝羚")
     (13 . "義群羣羨")
     (15 . "羯羮")
     (16 . "羲")
     (19 . "羹羶羸")
     (20 . "譱"))
    ("ひとあし"
     (2 . "儿")
     (3 . "兀")
     (4 . "允元")
     (5 . "兄")
     (6 . "兇光充先兆")
     (7 . "克児兌兎")
     (8 . "兒兔免")
     (10 . "党")
     (11 . "兜")
     (14 . "兢"))
    ("ひと"
     (2 . "人")
     (4 . "介仇今什仍仁仄仆仏仂从")
     (5 . "以仕仔仗仞仭仙仟他代付令")
     (6 . "伊仮会价企伎休仰件伍伉仲伝任伐伏伜")
     (7 . "佛位佚何伽估佝佐作伺似住伸佗但佇体低佃伯伴佑余伶佞")
     (8 . "依価佳侃佶供佼使侈侍侏侘佻佩佰侮併侑佯來例侖侠儘侭")
     (9 . "俄係俔侯俟俊俎信侵促俗俘便俛保俑俚俐侶俤俥俣")
     (10 . "倚倭俺倨倶倔倪倹倦個候倖倥倅借俶倡倩倬値倒俳倍俾俵俯倣俸們倆\
倫倉修")
     (11 . "假做偃偕偐偽偶偈健偲偖偬側偸停偵偏偉")
     (12 . "傀傚傘備傅傍傑償")
     (13 . "傳傴僅傾傲債催傷僉僊僧傭僂働")
     (14 . "僞僖僑僥僭僣像僮僕僚")
     (15 . "價儉億儀僵儁儂僻")
     (16 . "儒儕儔儚儖")
     (17 . "優儡")
     (18 . "儲")
     (21 . "儺儷")
     (22 . "儼儻"))
    ("ひのかわら"
     (5 . "皮")
     (10 . "皰")
     (12 . "皴")
     (14 . "皸皹")
     (15 . "皺"))
    ("ひらび"
     (4 . "曰")
     (6 . "曳曲")
     (7 . "曵更")
     (9 . "曷")
     (10 . "書")
     (11 . "曼曹曽")
     (12 . "最曾替")
     (13 . "會"))
    ("ひ"
     (4 . "火")
     (6 . "灰灯")
     (7 . "灸災灼")
     (8 . "炎炙炊炒炉")
     (9 . "炯炬炸炭炳炮為点熨")
     (10 . "烏烝烈烙烋烟")
     (11 . "烱焉烹烽")
     (12 . "焔焜煮焼焦然無焚焙")
     (13 . "煙煥煕煦煢煌照煎煖煤煬煉煩")
     (14 . "熈熏煽熄熊熕熔")
     (15 . "熬熟熱")
     (16 . "燈燒燕熹熾燉燃燔燎燐燗")
     (17 . "燮營燠燬燦燭燧燥燵")
     (18 . "燻燼燹燿")
     (19 . "爆爍")
     (20 . "爐")
     (21 . "爛")
     (29 . "爨"))
    ("ふいにょう"
     (3 . "夊")
     (10 . "夏")
     (14 . "夐"))
    ("ふしづくり"
     (2 . "卩")
     (5 . "卮卯夘")
     (6 . "印危")
     (7 . "却即卵")
     (8 . "卷")
     (9 . "卻卸")
     (10 . "卿"))
    ("ふつ"
     (12 . "黹")
     (17 . "黻")
     (19 . "黼"))
    ("ふでづくり"
     (6 . "聿")
     (11 . "粛")
     (13 . "肅肄肆")
     (14 . "肇"))
    ("ふね"
     (6 . "舟")
     (10 . "航般舫舩舮")
     (11 . "舵舸舷船舳舶")
     (13 . "艇艀")
     (16 . "艙艘")
     (17 . "艚艝")
     (18 . "艟")
     (19 . "艢艤")
     (20 . "艨")
     (21 . "艦艪")
     (22 . "艫"))
    ("ふゆがしら"
     (3 . "夂")
     (9 . "変"))
    ("ふるとり"
     (8 . "隹")
     (10 . "隻隼")
     (11 . "雀寉")
     (12 . "雁雇集雄")
     (13 . "雅雌雎雋雉雍")
     (14 . "雑")
     (16 . "雕")
     (17 . "雖")
     (18 . "雙雜雛難")
     (19 . "離"))
    ("ぶん"
     (4 . "文")
     (7 . "斈")
     (8 . "斉")
     (11 . "斎")
     (12 . "斑斐斌"))
    ("べん"
     (13 . "黽")
     (24 . "鼇")
     (25 . "鼈"))
    ("ほう"
     (4 . "方")
     (8 . "於")
     (9 . "施")
     (10 . "旃旆旁旄旅")
     (11 . "旌族旋")
     (12 . "旒")
     (14 . "旗")
     (16 . "旙")
     (18 . "旛"))
    ("ほす"
     (3 . "干")
     (5 . "平")
     (6 . "幵年并")
     (8 . "幸")
     (13 . "幹"))
    ("ほとぎ"
     (6 . "缶")
     (9 . "缸")
     (10 . "缺")
     (17 . "罅")
     (20 . "罌")
     (21 . "罍")
     (22 . "罎")
     (24 . "罐"))
    ("ほね"
     (10 . "骨")
     (13 . "骭")
     (14 . "骰")
     (16 . "骸骼")
     (18 . "髀")
     (19 . "髄")
     (21 . "髏")
     (23 . "體髓髑"))
    ("ぼう"
     (3 . "个")
     (4 . "中")
     (5 . "丱")
     (7 . "串"))
    ("ぼくのと"
     (2 . "卜")
     (4 . "卞")
     (5 . "占")
     (8 . "卦")
     (9 . "貞"))
    ("まがりがわ"
     (3 . "巛川")
     (6 . "州巡")
     (11 . "巣"))
    ("まげあし"
     (3 . "尢")
     (4 . "尤")
     (7 . "尨")
     (12 . "就"))
    ("ます"
     (6 . "舛")
     (12 . "舜")
     (15 . "舞"))
    ("また"
     (2 . "又")
     (3 . "叉")
     (4 . "及双反友")
     (5 . "収")
     (8 . "取受叔")
     (9 . "叙叛")
     (10 . "叟")
     (16 . "叡")
     (18 . "叢"))
    ("まだれ"
     (3 . "广")
     (5 . "広庁庄")
     (7 . "序床庇")
     (8 . "庚底店府庖")
     (9 . "度庠")
     (10 . "庫座庭")
     (11 . "庵康庸庶")
     (12 . "廁廂廃廊")
     (13 . "廈廉")
     (14 . "廓廏廐廖")
     (15 . "廣廢廝廠廚廛廟廡")
     (16 . "廨廩")
     (19 . "廬")
     (20 . "廰")
     (21 . "廱")
     (25 . "廳"))
    ("まめ"
     (7 . "豆")
     (10 . "豈")
     (13 . "豊")
     (15 . "豌豎")
     (18 . "豐"))
    ("みずから"
     (6 . "自")
     (9 . "臭"))
    ("みず"
     (4 . "水")
     (5 . "永氷汁汀氾")
     (6 . "汚汗江汕汝汐池汎汢")
     (7 . "求汞汪汽沂汲決沍沙沚沁汰沢沖沈沌沛汾汨汳沐没沒沃")
     (8 . "沓泳泄沿泱泓河泣況沽泗泅沮沼沱治注泥沾波泊泛泌泯沸法泡泙沫油沺泪泝")
     (9 . "泉洟洩海活洶洫洪洽洸洙洲洵洋浄津洒浅洗染洞派洛洌洳")
     (10 . "泰浤浣涓浩浚消浹浸浙涎涕涅浜浮浦浴浬流涙浪涌涛涜")
     (11 . "淨淺淫液淵渕渊淹涯渇涵淇渓涸淆混淬渋淑淳渉淞深清済淒淅淙淡添\
淀淘淤涼淪淋淮淌淕淦")
     (12 . "湶渚渥渭湮温渦渙渠減湖港渾渣滋湿湘湫渫湊測湛湍渟渡湯渺湎渤満\
渝游湧湾渮湃湟湲溂溌")
     (13 . "溪溢滑漢源溝溘滉溷滓準溽溯滄溲滞溺滔漠溥滂溟滅溶溜滝溏")
     (14 . "滿滯演漑滬滸滾漬漆滲漸漕漱漲滴滌漂漫漾漣漁漏漉滷潅漓")
     (15 . "滕潁漿澁澀澑潰澗澆潔潺潸潤潯潟潜潛潭澄澂潮潼潘澎潦潴")
     (16 . "澤澣濂澳激澡濁澹澱濃濆澪")
     (17 . "濱濟濕濠濡濬濯濤濘濮濛濔濶")
     (18 . "瀁瀉瀋濺瀑濫瀏濾")
     (19 . "瀞瀧濳瀛瀚瀦瀕瀬瀝瀘")
     (20 . "瀟瀰瀾瀲")
     (21 . "灌")
     (22 . "灑灘")
     (25 . "灣"))
    ("みみ"
     (6 . "耳")
     (9 . "耶耻")
     (10 . "耿耽")
     (11 . "聊聆")
     (12 . "聒")
     (13 . "聖聘")
     (14 . "聟聚聡聞聢")
     (15 . "聨")
     (17 . "聰聳聲聴聯")
     (18 . "聶職")
     (20 . "聹")
     (22 . "聽聾"))
    ("みる"
     (7 . "見")
     (11 . "規視覓")
     (12 . "覚覗覘")
     (14 . "覡")
     (16 . "覩親覦")
     (17 . "覬覯覧")
     (18 . "観覲")
     (20 . "覺")
     (21 . "覽")
     (22 . "覿")
     (25 . "觀"))
    ("み"
     (7 . "身")
     (10 . "躬")
     (11 . "躯")
     (12 . "躰")
     (13 . "躱")
     (16 . "躾")
     (19 . "軅")
     (20 . "軆")
     (24 . "軈"))
    ("むぎ"
     (7 . "麦")
     (11 . "麥麸")
     (15 . "麩麪")
     (16 . "麭")
     (19 . "麹")
     (20 . "麺"))
    ("むし"
     (6 . "虫")
     (8 . "虱")
     (9 . "虹虻")
     (10 . "蚓蚩蚤蚕蚊蚌蚣蚪蚋")
     (11 . "蚶蛍蛆蛇蛋蚰蚫蚯蛄蛉蛎")
     (12 . "蛙蛔蛞蛩蛬蛟蛤蛭蛛蛮蛯")
     (13 . "蜒蛾蜆蜈蛸蜀蜃蛻蜑蜉蜂蛹蜊蜍")
     (14 . "蜿蜷蜻蜥蜘蜩蜚蜜蜴蝋")
     (15 . "蝟蝦蝸蝌蝎蝴蝗蝨蝕蝶蝮蝙蝓蝠蝣蝪蝉蝿")
     (16 . "螢螟螂融")
     (17 . "螯蟋螽螫蟄螳螺螻蟀蟆蟇蟐")
     (18 . "蟲蟯蟠蟒")
     (19 . "蟷蠎蟹蠏蠍蟻蟾蟶蠅")
     (20 . "蠑蠖蠕")
     (21 . "蠢蠣蠡")
     (23 . "蠱蠧")
     (24 . "蠶蠹")
     (25 . "蠻"))
    ("むじな"
     (7 . "豸")
     (10 . "豺豹")
     (11 . "豼")
     (12 . "貂")
     (13 . "貉貅貊")
     (14 . "貍貌")
     (15 . "貎")
     (17 . "貔")
     (18 . "貘"))
    ("むのほこ"
     (5 . "矛")
     (9 . "矜"))
    ("むら"
     (7 . "邑邨那邦")
     (8 . "邯邱邵邸邪")
     (9 . "郁郊郎")
     (10 . "郢郡郤郛")
     (11 . "郭郷都部郵")
     (12 . "鄂")
     (13 . "鄒")
     (14 . "鄙")
     (15 . "鄭鄲鄰"))
    ("む"
     (2 . "厶")
     (5 . "去")
     (8 . "参")
     (11 . "參"))
    ("めめ"
     (4 . "爻")
     (9 . "爼")
     (11 . "爽")
     (14 . "爾"))
    ("めん"
     (9 . "面")
     (14 . "靤")
     (16 . "靦")
     (23 . "靨"))
    ("め"
     (5 . "目")
     (8 . "直盲")
     (9 . "看盻盾相省眈眉眇眄県")
     (10 . "眩真眞眥眦眛眠眤")
     (11 . "眼眷眺眸")
     (12 . "着睇")
     (13 . "睚睨睫睡睛督睥睦")
     (14 . "睿睾睹")
     (15 . "瞎瞋瞑")
     (16 . "瞠瞞")
     (17 . "瞰瞳瞥瞭瞶瞹")
     (18 . "瞿瞼瞽瞬瞻")
     (19 . "矇")
     (20 . "矍")
     (24 . "矗")
     (26 . "矚"))
    ("もちいる"
     (5 . "用")
     (7 . "甫甬"))
    ("もん"
     (8 . "門")
     (9 . "閂")
     (10 . "閃")
     (11 . "閉閇閊")
     (12 . "開間閑閏閔閖")
     (13 . "閘閠閙")
     (14 . "閣関閨閤閧閥")
     (15 . "閲閭")
     (16 . "閼閻閹閾")
     (17 . "闇闊闃闍闌")
     (18 . "闕闔闖闘")
     (19 . "關")
     (20 . "闡")
     (21 . "闥闢"))
    ("やく"
     (17 . "龠"))
    ("やまいだれ"
     (7 . "疔")
     (8 . "疚疝")
     (9 . "疫疥疣")
     (10 . "痂疳痃疵疾疽症疹疸疼疲病疱")
     (11 . "痍痕痔痊痒")
     (12 . "痙痣痛痘痞痢")
     (13 . "痾痿痼瘁痰痴痲痳痺")
     (14 . "瘋瘍瘉")
     (15 . "瘟瘧瘠瘡痩瘢瘤")
     (16 . "瘴瘰瘻")
     (17 . "癇癌癈療癆")
     (18 . "癜癖癒癘")
     (19 . "癡")
     (20 . "癢")
     (21 . "癨癩癪癧")
     (22 . "癬")
     (23 . "癰")
     (24 . "癲"))
    ("やま"
     (3 . "山")
     (4 . "乢")
     (5 . "屶")
     (6 . "屹")
     (7 . "岐岌岑妛岔")
     (8 . "岳岸岡岬岫岨岱岷岻岾峅岼岶岩")
     (9 . "峡峙峠峇")
     (10 . "峽峨峩峻峭島峰峯峪峺")
     (11 . "崗崋崖崕崎崟崛崑崔崇崢崩崘崙崚")
     (12 . "嵋嵜嵌嵎嵐嵒嵩")
     (13 . "嵬嵯嵳嵶")
     (14 . "嶌嶋嶇嶄嶂")
     (15 . "嶐嶢嶝")
     (16 . "嶮嶬")
     (17 . "嶽嶷嶼嶺")
     (20 . "巌巉")
     (21 . "巍")
     (22 . "巓巒")
     (23 . "巖"))
    ("や"
     (5 . "矢")
     (7 . "矣")
     (9 . "矧")
     (10 . "矩")
     (12 . "短")
     (13 . "矮")
     (17 . "矯"))
    ("ゆうべ"
     (3 . "夕")
     (5 . "外")
     (6 . "夙多夛")
     (8 . "夜")
     (11 . "梦")
     (13 . "夢")
     (14 . "夥"))
    ("ゆみ"
     (3 . "弓")
     (4 . "引弔弖")
     (5 . "弘弗")
     (6 . "弛")
     (7 . "弟")
     (8 . "弦弩弥")
     (9 . "弧弭弯")
     (10 . "弱")
     (11 . "強張弸")
     (12 . "弾弼")
     (13 . "彁")
     (15 . "彈")
     (16 . "彊")
     (17 . "彌")
     (22 . "彎"))
    ("らいすき"
     (6 . "耒")
     (10 . "耘耕耙耗")
     (11 . "耜")
     (13 . "耡")
     (16 . "耨"))
    ("りゅう"
     (16 . "龍")
     (22 . "龕"))
    ("るまた"
     (4 . "殳")
     (8 . "殴")
     (9 . "段")
     (10 . "殷殺")
     (11 . "殻")
     (12 . "殼")
     (13 . "毀殿")
     (15 . "毆毅"))
    ("れいづくり"
     (8 . "隶")
     (16 . "隷")
     (17 . "隸"))
    ("ろ"
     (11 . "鹵")
     (20 . "鹹")
     (24 . "鹽鹸"))
    ("わかんむり"
     (2 . "冖")
     (4 . "冗")
     (5 . "写")
     (9 . "冠")
     (10 . "冤冢冥冦")
     (11 . "冨")
     (14 . "冩")
     (16 . "冪"))
    ))

(defvar kakusuu-table
  '[0					; ignore
    "一丶丿乙亅"			; 1
    "七丁乂乃九了二亠人儿入八冂冖冫几凵刀力勹匕匚匸十卜卩厂厶又" ; 2
    "下三上丈万与个丸久乞也于亡兀凡刃刄勺千叉口囗土士夂夊夕大女孑\
子宀寸小尢尸屮山巛川工已己巳巾干幺广廴廾弋弓彑彡彳才" ; 3
    "丐丑不中丹之予云五互井亢介仇今什仍仁仄仆仏仂从允元兮公六内円\
冗凶刈切分勾勿匂匁化区匹午升卅卆卞厄及双反友壬夬太天夫夭孔少\
尤尹尺屯乢巴幻廿弌引弔弖心戈戸手扎支攴攵文斗斤方旡无日曰月木\
欠止歹殳毋比毛氏气水火爪父爻爿片牙牛犬王" ; 4
    "丘且世丗丕丙丱主丼乎乍乏以仕仔仗仞仭仙仟他代付令兄冊册冉囘写\
冬処凧凹出凸刊刋加功匆包北卉半占卮卯夘去収叶叩可句古号司史只\
叱召台叮叨右叺叭囚四圧圦外央失夲奴孕它尓尻尼屶巨巧左匝市布平\
幼広庁庄弁弍弘弗必戉戊打払斥旦旧札朮本末未正此母民永氷汁汀氾\
犯玄玉瓦甘生用甲申田由疋癶白皮皿目矛矢石示礼禾穴立赱辷辺込" ; 5
    "丞両争亙亘亦亥交伊仮会价企伎休仰件伍伉仲伝任伐伏伜兇光充先兆\
全共再冱冰决冲凩凪刑刎列刔劣匈匡匠卍印危吉吃叫吁向后合吊吐吋\
同名吏各因回団圭在地圷圸壮夙多夛夷夸如奸好妁妃妄字存安宇守宅\
寺尖尽屹州巡帆幵年并式弐弛当忖忙戍戎戌成扞扣扛扠扱托扨收攷旭\
旨旬早曳曲有机朽朿朱朶朴朸杁朷次死毎気汚汗江汕汝汐池汎汢灰灯\
牝牟犲瓜百竹米糸缶网羊羽考老而耒耳聿肉肌肋自至臼舌舛舟艮色艸\
艾芋虍虫血行衣襾西辻迅阡" ; 6
    "串乕乱亊亜些亨佛位佚何伽估佝佐作伺似住伸佗但佇体低佃伯伴佑余\
伶佞克児兌兎兵冏冴冶冷况刪初判別利刧劬劫助劭努励労匣医却即卵\
吸吽呀含听吟君呉吾吭吼告吹吮呈吶呑吠否吻吩呆呂吝呎囲囮困図坎\
圻均坑坐址坏坂坊壯壱声売夾妓妝妥妊妣妨妙妖妍孝孚孛完宏宋宍寿\
対尨局尿屁尾岐岌岑妛岔巫巵希帋序床庇廷弄弃弟形役彷応忌志忍忘\
快忻忤忸忱忰我戒戻扼技抉抗抒抄抓択投抖把抜批扶扮抃抔抑找折抂\
改攻孜攸斈旱曵更来杆杏杞材杉条杖束村杜杓杙李杣杢杤杠求汞汪汽\
沂汲決沍沙沚沁汰沢沖沈沌沛汾汨汳沐没沒沃灸災灼牡牢状狂狃狆狄\
犹玖瓧甫甬甸男町甼疔皀皃矣社私秀禿究竍系糺罕肝肛肓肖肘肚臣良\
芍芒花芸芝芙芳見角言谷豆豕豸貝赤走足身車辛辰迂迄辿近迎返邑邨\
那邦酉釆里阯阪阨阮防麦" ; 7
    "並乖乳事亞享京依価佳侃佶供佼使侈侍侏侘佻佩佰侮併侑佯來例侖侠\
儘侭兒兔免兩其具典冽凭函刮券刳刻刷刺制刹到劵劾効劼協卒卓卦卷\
参取受叔咏呵咎呟呼呱呷咋呰呪咒周呻咀知呶咄咆味命和咐固国囹坩\
坤垂坦坡坿坪垉尭夜奄奇奈奉奔侫委姑妻始姉妾姓姐妲妬姆妹学孥孟\
孤季宛官宜実宗宙定宕宝尚届屆居屈岳岸岡岬岫岨岱岷岻岾峅岼岶岩\
帖帚帙帑帛幸庚底店府庖延廸弦弩弥彿往徃径征徂彼忽忠忝念忿怐怡\
怪怯怙性怛怕怖怫怦怏怜怺怩或戔房所承抻拂拔押拗拐拡拑拒拠拘招\
拙拓拆担抽抵拈拝拍拌披拊拇抱抛抹拉抬放斉斧於易旺昊昆昏昇昌昔\
昃明旻服朋杲杳枉果杭枝杵松枩枢析杼枕東杯板枇杪枌枋枚林枡枠杰枅\
枦杷欧欣武歩歿殀殴毒毟氓氛沓泳泄沿泱泓河泣況沽泗泅沮沼沱治注\
泥沾波泊泛泌泯沸法泡泙沫油沺泪泝炎炙炊炒炉爭爬牀版牧物狐狎狗\
狙狛狒玩瓩画畄疚疝的盂直盲矼祀祁祈祉秉穹空突竏竺籵罔羌者冐\
育肯肩股肴肱肢肥肪肬肭臥舎舍苅芥芹芫芟芯芭芬芦若英苑芽苦茎苗\
茉茂虎虱衫表軋迚述迭迫邯邱邵邸邪采金長門阜陂阿阻陀附隶隹雨青\
非鼡" ; 8
    "乗亟亰亭亮俄係俔侯俟俊俎信侵促俗俘便俛保俑俚俐侶俤俥俣兪冑冒\
冠凾剏剄剋削前則剃剌勁勅勃勇匍南卑単貞卻卸厚厖厘叙叛哀哇咽咳\
咢咸咥咬哄哈哉咨咫咲哂咤品咼咾哘咯圀囿垈垣垓垠型垢城垤垪垰垳\
変奕奐奎契奏姙娃威姨姻姦姜姿姪姥姚姶孩客宦室宣宥専封屋屍屎屏\
昼峡峙峠峇巻巷帥帝幽度庠廻建廼弧弭弯彖彦後很徇待律徊怨急思怎\
怠怒怱恊恠悔恢恪恟恒恆恰恍恨恃恤恂恬恫扁拏拜按挌括拮挟拱挂拷\
拶指持拾拯拭拵挑挧故政斫施昂映昨昵春昭是星昶昧昴昜昿曷朏柆栄\
架枷柯柑柬枳柩枸枯査柴柵柞柿柘柊柁柝柱柢柔某柏柎柄枹柚柳栂栃\
柾柤柧柮枴歪殆殄殃段毘泉洟洩海活洶洫洪洽洸洙洲洵洋浄津洒浅洗\
染洞派洛洌洳炯炬炸炭炳炮為点熨爰爼牲牴狠狭狡狩独狢珂珈珊珍珎\
玻玲珀玳瓮瓲瓱瓰甚畋畏界畍畑畉畆畊疫疥疣癸発皈皆皇盃盈盆看盻\
盾相省眈眉眇眄県矜矧研砂砕砌砒祇祝神祖祐禹禺科秋秕秒穽窃穿竒\
竓竕竿笂粁粂籾紆紀級糾紅紂約缸罘美耐耶耻胃胤胡胛胥胙胎胆胝胄\
背肺胚胖胞脉臭臾舁苛茄苣苟苫苒苴苔苧苳范苻苹苞茅茆苜苓苙苡苺\
荒茜草荘茶虐虹虻衂衍衿衵衽衷衲袂要計訂訃負赴軌軍迦迥迢迪迯迩\
邇逆送退追逃迷郁郊郎酋酊重閂限陌陋陏面革韋韭音頁風飛食首香鳬" ; 9
    "乘亳倚倭俺倨倶倔倪倹倦個候倖倥倅借俶倡倩倬値倒俳倍俾俵俯倣俸\
們倆倫倉修党兼冓冤冢冥冦凅准凄凋凍凉剞剣剛剤剔剥剖勍勉匪匿卿\
原叟唔員哥唏哽哮哭唆哨唇哲唐唄哺哩哢哦唖圄圃埖埀埃埋埒埓埔埆\
夏奚奘套娚娥姫娟娯娑娘娠娜娉娩孫宴家害宮宰宵宸容尅射将屐屑展\
屓峽峨峩峻峭島峰峯峪峺差師席帯帰庫座庭弉弱徑従徐徒恚恁恩恐恭\
恵恣恕息恥恙恋恷悧悦悁悍悟悃悚悄悛悌悩悖悒悋悗扇拿挙挈拳捍挾\
挨捐挫振挺捜挿捉捏捌挽捕捗效敏料旃旆旁旄旅既晏晃晄時晋晉晁晒\
書朔朕朗栢案桜格核桓栞桔框栩桂桀校桁栲根栽桟桎株栓栴桑桃桐梅\
栗栫桍桙栖桧档残殊殉殷殺氣氤泰浤浣涓浩浚消浹浸浙涎涕涅浜浮浦\
浴浬流涙浪涌涛涜烏烝烈烙烋烟特狹狷狸狼狽珪珥珠珮班琉珞珱畛畜\
畔畝畚留畠痂疳痃疵疾疽症疹疸疼疲病疱皋皰益盍眩真眞眥眦眛眠眤\
矩砦砥砠砧破砲砺砿祠祗祟祚祕祓祢祥秘秧秬称秤秦租秩秣秡窈竚站\
竝竜笈笏笑笊笆笋粃粋粉粍粐紜紘紗索紙純素紐納紕紊紛紡紋缺罟罠\
羔翁翅翆耆耄耘耕耙耗耿耽胸脅脇胯脂脆脊胴能脈胱致舐航般舫舩舮\
芻茵茴荊茨茲茱荀茹茸荏荐荅茯茫茗茘茖荷華莉虔蚓蚩蚤蚕蚊蚌蚣蚪\
蚋衄袁衾袞衰袖袗袒袙袢被袍記訖訓訐訌訊託討豈豺豹貢財起赳躬軒\
辱迹迴迺迸逅逝造速逐通逓途透連郢郡郤郛酌酒酎配釖針釘釜釡釛釟\
閃院陥降除陞陣陝陟陛陜陦隻隼飢馬骨高髟鬥鬯鬲鬼" ; 10
    "乾亀假做偃偕偐偽偶偈健偲偖偬側偸停偵偏偉兜冕冨凌凰剱剰剩剪副\
勘動勗務勒匏匐匙區厠參啀啓售商唱啜唾啅啄啖啗唸問唯唳唹啌啝啣\
國圉圈堊域基埼堀執埴堆堂培埠堋埣埜壷梦婀婉婚娵娶娼婆婢婦婁婪\
婬孰寃寅寄寂宿密寇專將尉崗崋崖崕崎崟崛崑崔崇崢崩崘崙崚巣帶帷\
常帳庵康庸庶強張弸彗彩彫彪彬徠從徙得徘悪患悉悠惟悸惓惚惨情悴\
悽惜惆悵悼惇惘惧戛戚戝扈掖掩掛掎掬据掀掘掲捲控採捨授捷推捶接\
措掃探掏掉掟捺捻排捧捫掠捩掫掵掴赦敕敍敘救教敖敗斎斛斜斬断旌\
族旋晝晦晞晤晧晨晟晢曼曹曽朖望桿條桝梳械桷梟梧梗梏梱梭梓梔梢\
梛梯梃桶梶桴梵梨梠梁梺桾梍梼梹欸欷欲殍殻毬毫淨淺淫液淵渕渊淹\
涯渇涵淇渓涸淆混淬渋淑淳渉淞深清済淒淅淙淡添淀淘淤涼淪淋淮淌\
淕淦烱焉烹烽爽牾牽犁倏猗猊猜猖猝猪猛猟率球現理琅珸瓠瓷瓶瓸甜\
産異畦畤畢略畧畩疏痍痕痔痊痒皐皎盗盒盛盖眼眷眺眸硅祭票祷移窄\
窓窒窕章竡竟笳笥笘笙第笞笛符笨笠笹笶笵粗粘粕粒経絅絃紺細紮紫\
終紹紳紲組紿紬紵絆累絋袴羞羝羚習翌翊耜聊聆粛脣脚脛脩脱脳脯舂\
舵舸舷船舳舶莅莖莊莚莪莞莟莢莎荵荻荼莫莓莠莨莇茣莽莵荳菓菊菌\
菜著菩萌處虚蚶蛍蛆蛇蛋蚰蚫蚯蛄蛉蛎衒術袈袋袤袮袰袵袿袷袱裃裄\
裳規視覓設訛訝許訣訟訥訪訳谺豚豼貨貫責貪販貧貮貭跂趾趺躯軛転\
軟軣逕逑這逡逍逞逖逗逋逢逎逸週進逮郭郷都部郵酔酖酘釈野釼釦釵\
釧釣釶閉閇閊陷陰険陲陬陳陶陪陸隆陵雀寉雪雫頃頂馗魚鳥鹵鹿麥麸\
麻黄黒" ; 11
    "傀傚傘備傅傍傑償凖凱剴割創剳勞勤勝募博厥厦厩厨喝営喙喀喚喊喜\
喟喫喬喧喉啻啾善喘喪喞單喋啼喃喩喇喰喨圍圏堙堰堝堪堅場堕塚堤\
堵塔報堡塁塀堽堺堯壹壺壻奥奢奠媛婿媒媚孱寒寓寔寐富尋尊就属屠\
屡嵋嵜嵌嵎嵐嵒嵩巽幄幃幀幅帽幇幾廁廂廃廊弑弾弼彭御徨循復悳惠\
惣悲悶惑惱愕慌惶愀惴惺惻惰愎愉愃愡戞戟扉掌掣插描握援掾揩換揮\
揆揣揉揃提揶揄揖揚揺揀敢敬散敞敦敝斑斐斌斯旒暁景暑晶晴晰智晩\
普暃最曾替期朞朝椄棧椅椁棺棋棊棘検棍棕椒植森棲棗椎棣棟棹棠棚\
棒棉椋椀椚椙椛棆棔棡棯椈椌椏椡椪椣椥椨椦椢欹款欺欽歯殘殖殕殼\
毳毯湶渚渥渭湮温渦渙渠減湖港渾渣滋湿湘湫渫湊測湛湍渟渡湯渺湎\
渤満渝游湧湾渮湃湟湲溂溌焔焜煮焼焦然無焚焙爲牌牋犀犇犂猫猴猩\
猶猥猯琢琴琥琵琺琳琲琶甥甦畫畳番畭畴疎痙痣痛痘痞痢發登皖皓皴\
盜着睇短硯硬硝硫硲硴禄稈稀税稍程窗窘窖竢竣竦童筈筐筋笄策筍筌\
筅筑等答筒筏筆筝粤粢粥粧粟粡粨粭粫絲絵給結絢絞絳絖絎絨絮絶統\
絣絡絏翕翔耋聒腋腔腎脹腆脾腓腑胼腕舒舜菷菴萎菅菫菰菎菽菖萃菘\
萋菁萇菟菠菲萍萠莱菱菻萄萓萢萬葵萩葬葉落蛙蛔蛞蛩蛬蛟蛤蛭蛛蛮\
蛯衆街裁装裂裙補裕裡覃覚覗覘觚觜觝詠訶詁詐詞証詔診訴詛詑詒註\
詆評詈象貂貽賀貴貲貰貸貯貼貳買費貶貿賁赧越超趁跏距跚跖跌跛跋\
躰軼軻軽軸軫辜逶逵逹運過遇遂達遅道遍遊遥鄂酣酢酥鈕釉量鈞釿鈔\
鈍鈑鈎鈬鈩開間閑閏閔閖隋随階隅隍隊陽隈雁雇集雄雲雰靫靱靭項須\
順颪飯馭馮黍黹" ; 12
    "亂亶傳傴僅傾傲債催傷僉僊僧傭僂働剿剽勧勢勦勠勣匯嗚嗅嗟嗄嗣嗜\
嗤嗔嘆圓園嗇塋塊塙塒塑塞填塗塘墓塰塢塩壼夢奧奨媼嫁嫌媾嫉嫋嫂\
媽嫐孳寛寝尠嵬嵯嵳嵶幌幕幎幹廈廉彁彙微徭惡愛意感愚愆慈惹愁惷\
想愍愈慍慨愧愾慊慎愼愴慄愽戀戡戦搜搭搖携搆搓搦搶摂掻損搗搨搏\
搬搾数斟新暎暘暗暈暇暉暄暖會楳椶楹楷楽棄業極楸楫楢楯楔楚楮椿\
椹椽楠楓椰楡楊楝楼椴楙楜楪楴楾榁楕楞歇歃歳毀殿溪溢滑漢源溝溘\
滉溷滓準溽溯滄溲滞溺滔漠溥滂溟滅溶溜滝溏煙煥煕煦煢煌照煎煖煤\
煬煉煩爺牒献猷猿猾獅瑛瑕瑚瑟瑞瑁瑜瑶琿瑙甞畸畷當痾痿痼瘁痰痴\
痲痳痺皙盞盟睚睨睫睡睛督睥睦矮碚碗碎碍碕碁碓碇硼碌碆祺禁祿禀\
禍禅禎福禽稔稚稠稗稜稟稘稙窟竪筺筵筥筴筧筮節筬筰筱粳粲粱粮經\
継絹絛綏続絽綛綉罨罫罪署置罩罧義群羣羨耡聖聘肅肄肆腦腱腫腥腸\
腹腴腰腺腟腮舅辞艇艀葦葭萼葛葷萱葫葺葱董葩葡葆葯葎萪蒄萵萸葮\
葹葢蒂蒸蓄蓉號虞虜蜒蛾蜆蜈蛸蜀蜃蛻蜑蜉蜂蛹蜊蜍衙裝裔裘裏裟褂\
裾裼裨裸裲褄解触詼該詭詰詣誇詬詩試詢詳誠詫誅誂誉誄話詮豊豢貉\
貅貊賊賈資賃賂賄賎賍跪跫跨跟跡践跣跳路躱較載軾輊輅輌辟農遏遐\
遑遒遉遁逼逾遖違遠遣鄒酬酩酪鉞鉛鉗鉅鉉鈷鉱鉤鉈鉦鉄鈿鉢鉋鈴鉐\
鉚閘閠閙隘隕隗隔雅雌雎雋雉雍電雹雷零靖靴靹韮韵頑頏頌頓頒預飭\
飲飮飫飩飾飼飽馴馳骭髢鳫鳩鳧鳰麁黽鼎鼓鼠" ; 13
    "僞僖僑僥僭僣像僮僕僚兢冩劃匱厭厮厰嘔嘉嗷嘖嘗嗾嗽嗹嘛團圖塲境\
塹塾墅塵増墨壽夐夥奩奬奪嫦嫣嫗嫡嫩嫖孵實寢寡寤察寧寞寥寨對層\
嶌嶋嶇嶄嶂幗幔廓廏廐廖彰徴徳慇愨愿愬態慕慂慘慣慳慷慚慴慥慟慓\
慢慵慯慱截搴摧摺摶摘摸摎敲斡旗暢暮暝暦榮榔榎槐概構槁槓榾槎槊\
様榛榱槍槌槙槇榻槃榧榑模榜榕榴榊榠榲榿樮槝槞歌歉歴殞毓滿滯演\
漑滬滸滾漬漆滲漸漕漱漲滴滌漂漫漾漣漁漏漉滷潅漓熈熏煽熄熊熕熔\
爾犒犖獄獏瑯瑤瑰瑣瑳瑪瑠甄甃甅疑瘋瘍瘉皸皹盡睿睾睹碪碣磁碑碧\
碯碵碩禊稱穀種稲窪窩竭端竰篏箇管箝箕箘箟箍箜箚算箋箒箏箔箙箆\
粹精糀粽維綺綮綣綱綵緇綽綬緒総綜綻綢綴綯緋綿網綾緑綸綟練綰綫\
緕罰翠翡聟聚聡聞聢肇腐膃膈膏腿膊膀膜膂臧臺與蒭蓊蓋蒹蒿蒟蓑蒜\
蓍蒔蒻蒐蓐蓁蓆蒼蓖蒲蒙蓙蒡蓚蔦蔵蜿蜷蜻蜥蜘蜩蜚蜜蜴蝋裹製裴褐\
褌複褊褓褝覡誨誡誑語誤誥誌誦誚誓説読認誣誘豪貍貌賑赫趙踈跼踊\
踉跿輕輒輓輔辣逧遡遞遙遘遜遭適鄙酳酵酷酸酲銕銜銀銖銃銭銑銓銛\
銅鉾銘銚閣関閨閤閧閥隙隠際障雑需静靤鞅靼鞄靺鞆鞁韶頗領頚颯颱\
飴駅駆駄駁馼骰髪髣髦魁魂鳶鳳鳴麼皷鼻齊" ; 14
    "價儉億儀僵儁儂僻凛劍劇劈劉勲匳噎器噐嘘嘴嘱嘶噌噂嘲嘸噴噛嘩墮\
墟墜墳墫墹墸嫺嫻嬉嬌嬋寫審寮導履嶐嶢嶝幟幢幡幣幤廣廢廝廠廚廛\
廟廡弊彈影徹慰慶慧慙慫慝憂慾慮憇憎憬憔憚憧憫憮憤憐戯戮撃摯摩\
撮撰撒撞撤撓撚播撥撫撲撩撈撕撹數敵敷暫暴樞槨樂樓樣横槻樛槿権\
槲槧樟樅槽樗樋樊標樒樫槫槭槹樔樌樢歐歓歎殤毆毅滕潁漿澁澀澑潰\
澗澆潔潺潸潤潯潟潜潛潭澄澂潮潼潘澎潦潴熬熟熱獎獗瑩瑾璋璃畿瘟\
瘧瘠瘡痩瘢瘤皚皺監盤瞎瞋瞑磆磑確磋磔碾碼磐磅磊禝稻稼稽稿稾稷\
穂穃窮窯窰篋篁箱箴箭箸篆篇篌箪範糊糅糎糂緲緜縁緩緘緊緝縄緤線\
緞緻締緡編緬縅罸罵罷羯羮翫翦翩聨膓膠膝膣膚膕膤舗舖舞蔔蔆蔭蔚\
蔡蓴蔗蒋蔘蔬蔟蔕蔓蔑蓬蔀蓼蓮蓿蝟蝦蝸蝌蝎蝴蝗蝨蝕蝶蝮蝙蝓蝠蝣\
蝪蝉蝿衝褒褞褥褪褫觧謁課誼諏諄諸誰請諍諾誕談調諂誹諒論諚諫豌\
豎貎賛賜賞賤質賣賠賓賦賚趣踐踝踞踟踏踪輙輝輜輟輩輛輪輦遨遮遯\
遺遵選遷遼鄭鄲鄰醉醋醇醂鋪鋭鋏銹鋤銷鋳鋒鋲鋩閲閭霄震霆霈霊靠\
鞋鞍鞏鞐頤頡餃餌餉養餅餓駈駕駒駟駛駝駘駐駑髴髯髮髭髫髱鬧魄魃\
魅魯魴鴈鴉鴃鴆鴇麩麪麾黎黙齒" ; 15
    "儒儕儔儚儖冀冪凝劒劔劑勳叡噫噤嘯噬噸噺噪圜墺壊墾壌壇壁壅奮嬢\
嬖學寰嶮嶬廨廩彊彜徼憙憖憩憲憊憑懌懊憶懐懈憾懆憺懍戰擇據擔撼\
擒擅操撻擁擂整旙曉曁暹暾曇曄暼暸樺橄機橘橋樹樵橡橇樽橢橙橦橈\
樸樶橲橸歙歔殪殫澤澣濂澳激澡濁澹澱濃濆澪燈燒燕熹熾燉燃燔燎燐\
燗獣獨獪獲璢璞瓢甌甎甍疂瘴瘰瘻盥盧瞠瞞磚磬磧磨禦頴穐穎穏積穆\
窶窺築篝篩篦篥篭簑篤糖糒糘緯縊縣縞縡縒縦縟縉縋縢縛繁縫罹羲翰\
耨膩膳膰膨膵臻興舘艙艘蕩蕚蕎蕀蕨蕣蕉蕘蕈蕁蕊蕋蕃蕪蔽蕕蕭蕗薫\
薪薦薄薬螢螟螂融衛衞衡褸襁褶覩親覦諤諳謂諧諌諱謔諠諺諢諮諜諦\
諷諞謀諭諛謡諡豫豬賢賭赭踴蹂踵蹄踰躾輯輳輻輸輹辨辧隨遲遶還避\
醒醍醐醗錦錢錏鋺鋸錮鋼錯錙錠錐錆錫錚錘錣錬録錺錵錻鍄閼閻閹閾\
險隧隣隷雕霍霓霎霑霏霖靜靦鞘頼頷頬頸頽頭餘餐餒餔館駭駮駱駲骸\
骼髷髻鬨鮓鮎鮃鮒鮑鮗鮖鴛鴨鴟鴕鴫鴒鴣鴪鴦鴬鴎鶯麈麭默黔龍龜" ; 16
    "優儡勵厳嚇嚆嚔嚏嚊嚀墻壓壑壕壗嬰嬬嬲嬪嬶孺嶽嶷嶼嶺彌徽應懃懇\
懋懦戲擘擱擬擦擠擡擢擣擯斂曖檢檐橿檠檄檎檣檀檗檍檪檜歛氈濱濟\
濕濠濡濬濯濤濘濮濛濔濶燮營燠燬燦燭燧燥燵爵牆犠獰環甑癇癌癈療\
癆盪瞰瞳瞥瞭瞶瞹矯磯磽礁磴禪禧穉穗窿竃簒簀篠簇篳篷篶簍簓簗簔\
糢糠糟糜糞縲總縱繆繦縮績繊縵縻縹繃縷縺罅翳翼聰聳聲聴聯膽臆膾\
臀膿臂膺臉膸臈舉艚艝艱薗薤薈薑薊薨薔薛薙薇薜蕾蕷薐薮薀虧螯蟋\
螽螫蟄螳螺螻蟀蟆蟇蟐襃襄褻襌襍覬覯覧謌謠謹謙謇講謚謝謖謄謐謗\
謎豁谿貔購賽賺賻趨蹊蹇蹉蹌蹐蹈轅轄轂輾輿邂遽邁邀醢鍋鍔鍵鍠鍬\
鍾鍼鍛鍮鍍錨鍖鍜闇闊闃闍闌隱隰隲隸雖霙霞霜鞠鞜韓顆頻颶餝餡餞\
餤餠馘駻駿駸騁醜鮟鮪鮠鮭鮫鮨鮮鮴鵄鴻鵁鴿鴾鵆鵈麋黏黛黜點黝黻\
鼾齋齔齢龠" ; 17
    "儲叢嚠壘壙彝懲懣懴戴戳擧擴擾擲擺擽擶攅斃斷旛曙曚曜朦櫁檻櫃櫂\
檮檸檳檬歟歸殯瀁瀉瀋濺瀑濫瀏濾燻燼燹燿獵璧甕甓癜癖癒癘瞿瞼瞽\
瞬瞻礒礎礑礇禮穢穫穣穡竅竄簡簣簧簪簟糧繪繝繞織繕繙繚繧繖繍羂\
翹翻聶職臍臑臨舊艟藁藏薩藉薯薺薹藐藤藩藍蟲蟯蟠蟒襖襟襠覆観覲\
觴謳謦謫謾謬謨豐貘贄贅贈贇蹠蹟蹙蹤蹣蹕轉轆轌邃醫醯醤醪釐鎰鎧\
鎬鎖鎗鎮鎭鎚鎔鎌鎹闕闔闖闘雙雜雛難霤鞫鞦鞣鞭鞨額顎顔顏顕顋題\
類餬餮馥騎騏験騅騒駢髀鬆鬩魏魍魎鯀鯊鮹鯉鯆鯏鯑鯒鵝鵞鵑鵠鵜鵐\
鵡鵤鵙麌麿黠鼕鼬" ; 18
    "嚥嚮壞壜壟嬾寳寵廬懷懶攀曠曝櫛櫚櫟櫓櫞櫑殱瀞瀧濳瀛瀚瀦瀕瀬瀝\
瀘爆爍牘犢獸獺璽瓊瓣疆疇癡矇礙禰穩簷簫簽簸簿簾籀繩繭繹繋繰羃\
羆羅羹羶羸臓臘艢艤艶藝藥藕藪藜藻蘭蟷蠎蟹蠏蠍蟻蟾蟶蠅襞襦襤覈\
覇譌證譁譏譎警識譖譛譚譜贊贋蹶蹴蹲蹼軅轎轍辭邊鏥鏖鏡鏗鏨鏘鏃\
鏑鏝鏐鏈鏤關隴離霪霧靡鞴鞳韜韲韻願顛餽餾饂騙騨髄鯣鯨鯢鯤鯔鯖\
鯛鯡鯱鯰鯲鯵鶏鵲鶉鶇鵯鵬鵺麓麕麒麑麗麹黼" ; 19
    "勸嚶嚴壤壥孃孅孀寶巌巉廰懸懺攘曦朧欄櫪櫨瀟瀰瀾瀲爐犧獻瓏癢矍\
礬礪礫礦竇競籍籌籃籏糯繼纂繻繽辮纃繿罌譱耀聹臙臚艨蘂藷藹蘊蘇\
蘓蘋藺蘆蘢藾蠑蠖蠕襭襪覺觸譯議護譲譫譟譬贏贍躁躇躄躅軆轗醵醸\
醴釋鐡鐚鐘鐔鐓鐙鐃鐐鐇闡霰露響飄飃饉饅馨騷騫騰鬪鰕鰉鰓鰌鰍鰆\
鰈鰒鰊鰔鰄鰛鰐鶚鶤鶩鶫鹹麺黨黥齣齟齠齡" ; 20
    "儺儷囂嚼囁囃囀屬巍廱懽懼懾攜攝曩櫻欅櫺殲灌爛瓔癨癩癪癧龝竈籐\
籖籔糲續纎纈纏纐纉罍艦艪蘗蘖蘯蘚蘰蠢蠣蠡襯覽譽譴贐贓贔躋躊躍\
轟轜辯邉醺鐵鐶鐫鐸鐺闥闢霸霹顧飆飜饐饋饑饒饌驅驂驀驃騾髏鬘魑\
魔鰮鰥鰭鰤鰯鰡鰰鷄鶴鷁鶻鶸鶺鶲鷂鷆鷏麝黯齎齦齧" ; 21
    "儼儻囈嚢囎巓巒彎懿攤變權歡灑灘疊疉癬禳穰竊竸籘籟籠糴纒罎羇聽\
聾臟艫襲襴襷覿讀讃贖躓躑躔轡轢鑄鑓鑁鑒霽霾韃顫饗饕驚驕驍鬚鬻\
鰹鰺鰻鰾鱈鱆鱇鰲鷙鷓齪齬龕" ; 22
    "巖攣攪攫欒癰籤籥纖纓纔蘿蠱蠧讌讎讐躙轣轤邏鑛鑑鑠鑢鑞鑚靨顯驛\
驗體髓髑髞鬟鱒鱗鱚鷸鷲鷦鷭鷯鷺麟黐黴齏" ; 23
  "囑囓攬癲矗罐羈艷蠶蠹衢讓讒讖軈釀鑪靈靄靆靂韈韆顰驟鬢魘鱠鱧鷽鷹\
鹽鹸鼇齷齲齶" ; 24
  "廳欖灣籬糶纛臠蠻覊觀讙躡釁鑰靉顱鬣黌鼈" ; 25
  "欝矚讚鑵鑷驢鬮鱶黶"			; 26
  "纜躪鑽鑼鑾鑿顴顳驥驤鱸黷"		; 27
  "钁驩鸚"				; 28
  "欟爨驪鬱鸛"				; 29
  "驫鸞"				; 30
  ])

;; 92.7.24 by T.Shingu -- Completely modified.
;; 92.8.24 by T.Shingu -- Bug fixed.
;; 92.9.17 by K.Handa  -- Now table contents are strings (not lists of strings)
;;;###autoload
(defun busyu-input ()
  (interactive)
  (let ((loop1 t)
	(loop2)
	(loop3)
	(val1)
	(val2)
	(busyuname)
	(nbusyu (1- (length busyu-table)))
	(kaku1)
	(kaku2))
    (while (or loop1 loop2 loop3)
      (if loop1
	  (progn
	    (setq loop1 nil)
	    (setq kaku1 (string-to-int (read-from-minibuffer
					(format "部首画数(1-%d): " nbusyu)
					(if kaku1 (int-to-string kaku1)))))
	    (and (< 0 kaku1) (<= kaku1 nbusyu) ; 92.9.30 by K.Mitani
		 (setq loop2 t))))
      (let ((inhibit-quit t))
	(if loop2
	    (progn
	      (setq loop2 nil)
	      (setq busyuname
		    (menu:select-from-menu
		     (list 'menu "部首:" (aref busyu-table kaku1))))
	      (if quit-flag
		  (progn
		    (setq quit-flag nil)
		    (setq loop1 t))
		(setq loop3 t)
		(setq val1 (cdr (assoc busyuname busyu-kaku-alist))))))
	(if loop3
	    (progn
	      (setq loop3 nil)
	      (setq kaku2
		    (menu:select-from-menu
		     (list 'menu "総画数:"
			   (cons (cons "*" val1)
				 (mapcar (function
					  (lambda (x)
					    (cons (format " %s画" (car x))
						  (cdr x))))
					 val1)))))
	      (if quit-flag
		  (progn
		    (setq quit-flag nil)
		    (setq loop2 t))
		(setq val2
		      (menu:select-from-menu 
		       (list 'menu "漢字:"
			     (if (stringp kaku2)
				 (busyu-break-string kaku2)
			       (apply (function nconc)
				      (mapcar (function 
					       (lambda (x)
						 (busyu-break-string (cdr x))))
					      kaku2))))))
		(if quit-flag
		    (progn
		      (setq quit-flag nil)
		      (setq loop3 t))
		  (insert val2)))))))))

;;;###autoload
(defun kakusuu-input ()
  (interactive)
  (let ((loop t)
	(kaku nil)
	(nkakusuu (1- (length kakusuu-table)))
	(val))
    (while loop
      (setq kaku (string-to-int
		  (read-from-minibuffer (format "画数(1-%d): " nkakusuu)
					(if kaku (int-to-string kaku)))))
      (if (not (and (< 0 kaku) (<= kaku nkakusuu))) ; 92.9.30 by K.Mitani
	  (setq loop nil)
	(let ((inhibit-quit t))
	  (setq val 
		(menu:select-from-menu
		 (list 'menu "漢字:"
		       (busyu-break-string (aref kakusuu-table kaku)))))
	  (if quit-flag
	      (setq quit-flag nil)
	    (insert val)
	    (setq loop nil)))))))

(defun busyu-break-string (str)
  (let ((len (length str))
	(i 0) j l)
    (while (< i len)
      (setq j (char-bytes (sref str i)))
      (setq l (cons (substring str i (+ i j)) l))
      (setq i (+ i j)))
    (nreverse l)))
