;;; sun-eos.el --- Intereactively loads the XEmacs/SPARCworks interface

;; Copyright (C) 1995 Sun Microsystems, Inc.

;; Maintainer:	Eduardo Pelegri-Llopart <eduardo.pelegri-llopart@Eng.Sun.COM>
;; Author:      Eduardo Pelegri-Llopart <eduardo.pelegri-llopart@Eng.Sun.COM>

;; Keywords:	SPARCworks EOS Era on SPARCworks load

;;; Commentary:

;; If manual loading is desired...
;; Please send feedback to eduardo.pelegri-llopart@eng.sun.com

;;; Code:

(load "sun-eos-load.el")
(eos::start)

;;; sun-eos-eos.el ends here
