;;; custom-load.el --- automatically extracted custom dependencies


;;; Code:

(custom-add-loads 'mouse '("mouse"))
(custom-add-loads 'minibuffer '("simple" "minibuf"))
(custom-add-loads 'log-message '("simple"))
(custom-add-loads 'environment '("frame" "minibuf" "modeline" "sound"))
(custom-add-loads 'sound '("sound"))
(custom-add-loads 'auto-save '("files"))
(custom-add-loads 'mail '("simple"))
(custom-add-loads 'editing-basics '("cmdloop" "simple" "files" "lisp"))
(custom-add-loads 'help-appearance '("help"))
(custom-add-loads 'lisp '("lisp" "find-func"))
(custom-add-loads 'help '("help"))
(custom-add-loads 'keyboard '("cmdloop"))
(custom-add-loads 'warnings '("simple"))
(custom-add-loads 'backup '("files"))
(custom-add-loads 'frames '("frame" "window-xemacs" "gui"))
(custom-add-loads 'abbrev '("files"))
(custom-add-loads 'find-function '("find-func"))
(custom-add-loads 'dired '("files"))
(custom-add-loads 'killing '("simple"))
(custom-add-loads 'paren-blinking '("simple"))
(custom-add-loads 'find-file '("files"))
(custom-add-loads 'files '("files"))
(custom-add-loads 'fill-comments '("simple"))
(custom-add-loads 'windows '("window" "window-xemacs"))
(custom-add-loads 'fill '("simple"))
(custom-add-loads 'faces '("faces"))
(custom-add-loads 'vc '("files"))
(custom-add-loads 'isearch '("isearch-mode"))
(custom-add-loads 'modeline '("modeline"))
(custom-add-loads 'editing '("simple"))
(custom-add-loads 'matching '("simple" "isearch-mode"))

;;; custom-load.el ends here
