;;; DO NOT MODIFY THIS FILE
(if (featurep 'tl-autoloads) (error "Already loaded"))

;;;### (autoloads (view-charset) "char-table" "tl/char-table.el")

(autoload 'view-charset "char-table" "\
Display character table of CHARSET." t nil)

;;;***

;;;### (autoloads (view-charset-by-menu) "chartblxmas" "tl/chartblxmas.el")

(autoload 'view-charset-by-menu "chartblxmas" "\
Display character table of CHARSET by pop-up menu." t nil)

;;;***

(provide 'tl-autoloads)
