;;; custom-load.el --- automatically extracted custom dependencies


;;; Code:

(custom-add-loads 'extensions '("eldoc"))
(custom-add-loads 'eldoc '("eldoc"))
(custom-add-loads 'mouse '("id-select"))
(custom-add-loads 'minibuffer '("detached-minibuf" "savehist"))
(custom-add-loads 'mail '("highlight-headers" "mail-extr" "ph" "smtpmail"))
(custom-add-loads 'uniquify '("uniquify"))
(custom-add-loads 'xemacs-build-report '("xemacs-build-report"))
(custom-add-loads 'hypermedia '("browse-url"))
(custom-add-loads 'lisp '("elp"))
(custom-add-loads 'elp '("elp"))
(custom-add-loads 'applications '("uniquify"))
(custom-add-loads 'keyboard '("edmacro"))
(custom-add-loads 'highlight-headers-faces '("highlight-headers"))
(custom-add-loads 'crontab '("crontab"))
(custom-add-loads 'comm '("ph"))
(custom-add-loads 'id-select '("id-select"))
(custom-add-loads 'frames '("detached-minibuf"))
(custom-add-loads 'shadowfile '("shadowfile"))
(custom-add-loads 'smtpmail '("smtpmail"))
(custom-add-loads 'files '("shadowfile"))
(custom-add-loads 'detached-minibuf '("detached-minibuf"))
(custom-add-loads 'languages '("crontab"))
(custom-add-loads 'edmacro '("edmacro"))
(custom-add-loads 'faces '("facemenu" "highlight-headers"))
(custom-add-loads 'passwd '("passwd"))
(custom-add-loads 'browse-url '("browse-url"))
(custom-add-loads 'processes '("passwd"))
(custom-add-loads 'news '("highlight-headers"))
(custom-add-loads 'highlight-headers '("highlight-headers"))
(custom-add-loads 'savehist '("savehist"))
(custom-add-loads 'ph '("ph"))
(custom-add-loads 'mail-extr '("mail-extr"))
(custom-add-loads 'matching '("id-select"))
(custom-add-loads 'facemenu '("facemenu"))

;;; custom-load.el ends here
