;;; custom-load.el --- automatically extracted custom dependencies


;;; Code:

(custom-add-loads 'holidays '("calendar"))
(custom-add-loads 'calendar '("calendar"))
(custom-add-loads 'local '("calendar"))
(custom-add-loads 'diary '("calendar"))
(custom-add-loads 'appt '("appt"))

;;; custom-load.el ends here
