;;; custom-load.el --- automatically extracted custom dependencies


;;; Code:

(custom-add-loads 'menu '("x-menubar"))
(custom-add-loads 'environment '("x-toolbar"))
(custom-add-loads 'buffers-menu '("x-menubar"))
(custom-add-loads 'toolbar '("x-toolbar"))
(custom-add-loads 'info '("x-toolbar"))
(custom-add-loads 'x '("x-faces" "x-font-menu"))

;;; custom-load.el ends here
